SystemOrganization addCategory: #'Orca-Core-Cache'!
SystemOrganization addCategory: #'Orca-Core-Server'!
SystemOrganization addCategory: #'Orca-Core-Misc'!
SystemOrganization addCategory: #'Orca-Core-Translator'!
SystemOrganization addCategory: #'Orca-Core-RemoteObjects'!

OMeta2 subclass: #OrcaOMeta2
	instanceVariableNames: 'currentMethod result inputConverter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

OrcaOMeta2 subclass: #OrcaAstReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

OrcaAstReader subclass: #OrcaAbstractPrinter
	instanceVariableNames: 'indentation'
	classVariableNames: 'ClassTextOutputStream JavascriptCharacters JavascriptPrintStream TextOutputStream'
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

!OrcaAbstractPrinter class methodsFor: 'converting' stamp: 'ag 4/14/2011 14:15'!
asJSString: aString
	"Copied from JSStream class >> encodeString:on:"
	
	| aStream char value encoded last |
	aStream := JavascriptPrintStream ifNil: [ JavascriptPrintStream := TextStream on: '' ] ifNotNil: [ JavascriptPrintStream reset; yourself ].
	aStream nextPut: $".
	1 to: aString size do: [ :index | 
		char := aString at: index.
		value := char greaseInteger.
		value < JavascriptCharacters size 
			ifFalse: [ aStream nextPut: char ]
			ifTrue: [
				encoded := JavascriptCharacters at: value + 1.
				encoded isString 
					ifTrue: [ aStream nextPutAll: encoded ]
					ifFalse: [
						"avoid that browsers mistakenly take the output as a closing tag"
						(last = $< and: [ char = $/ ])
							ifTrue: [ aStream nextPutAll: '\/' ]
							ifFalse: [ aStream nextPut: encoded ] ] ].
		last := char ].
	aStream nextPut: $".
	^ aStream contents! !

!OrcaAbstractPrinter class methodsFor: 'converting classes private' stamp: 'ag 1/9/2011 20:23'!
convert: aBlock to: output
	
	aBlock value:
		output reset; yourself.
	^ output contents! !

!OrcaAbstractPrinter class methodsFor: 'converting classes' stamp: 'ag 1/5/2011 21:22'!
convertClass: aClass
	
	^ self
		convertClass: aClass
		retrievingFrom: [ :methodSelector |
			self convertCompiledMethod: methodSelector ]! !

!OrcaAbstractPrinter class methodsFor: 'converting classes' stamp: 'ag 4/6/2011 15:52'!
convertClass: aClass retrievingFrom: aBlock
	"Return the converted version of the whole class. aBlock takes one parameter (a CompiledMethod) and returns
	the code for the method as string. A static (reused) output-stream is used to print the code."
	
	^ self
		convert: [ :output | self convertClass: aClass to: output retrievingFrom: aBlock ]
		to: (TextOutputStream ifNil: [ TextOutputStream := OrcaTextStream for: self ])! !

!OrcaAbstractPrinter class methodsFor: 'converting classes private' stamp: 'ag 4/12/2011 12:21'!
convertClass: aClass to: output retrievingFrom: aBlock
	
	| firstPrinted firstPrintedMemorizer |
	firstPrinted := false.
	firstPrintedMemorizer := [ :nextPartValid | nextPartValid and: [ firstPrinted or: [ firstPrinted := true. false ] ] ]. "Pretty complex. Remember, whether first line has been printed."
	output << OrcaSqueakAstToJsAst classFunction << '(' << (self asJSString: aClass name) << ',' << self space << '{' << self newline.
	self
		printPart: 'superclass' with: (self printSuperclass: aClass) to: output guardedBy: firstPrintedMemorizer;
		printPart: 'instanceVariables' with: (self printVariables: aClass instVarNames) to: output guardedBy: firstPrintedMemorizer;
		printPart: 'classInstanceVariables' with: (self printVariables: aClass class instVarNames) to: output guardedBy: firstPrintedMemorizer;
		printPart: 'classVariables' with: (self printVariables: aClass classVarNames) to: output guardedBy: firstPrintedMemorizer;
		printPart: 'classMethods' with: (self printMethodsOf: aClass class retrievingFrom: aBlock) to: output guardedBy: firstPrintedMemorizer;
		printPart: 'instanceMethods' with: (self printMethodsOf: aClass retrievingFrom: aBlock) to: output guardedBy: firstPrintedMemorizer.
	output << self newline << '})'.! !

!OrcaAbstractPrinter class methodsFor: 'converting classes' stamp: 'ag 4/13/2011 17:34'!
convertClasses: anArrayOfClasses retrievingFrom: aBlock withCalledFunctionsSource: anObject
	"Return the code for all the classes in anArrayOfClasses. aBlock takes on parameter (a class) and returns the code for the class.
	aSet contains the names of functions, that are called from any method of the code.
	The Message #calledFunctions is sent to anObject in order to retrieve a collection of function-names."
	
	| code |
	code := (self
		convert: [ :output | output << (self
							printEach: [ :class | aBlock value: class ]
							of: anArrayOfClasses
							separatedBy: (';', self newline))]
		to: (ClassTextOutputStream ifNil: [ ClassTextOutputStream := OrcaTextStream for: self ])).
	^ (self doesNotUnderstandClassWithMethods: anObject calledFunctions), code! !

!OrcaAbstractPrinter class methodsFor: 'converting classes' stamp: 'ag 4/13/2011 16:41'!
doesNotUnderstandClassName
	"This is named with a small letter to not collide with Squeak-classes in our st.-namespace.
	The convention is to name meta-slots in the namespace with small letter instead
	of underscores or something."
	
	^ 'doesNotUnderstandClass'! !

!OrcaAbstractPrinter class methodsFor: 'converting classes' stamp: 'hk 5/2/2011 11:30'!
doesNotUnderstandClassWithMethods: aDictionaryOfMethodNames

	| cr functions crr |
	cr := self newline, self indentation.
	crr := cr, self indentation.
	functions := TextStream with: ''.
	aDictionaryOfMethodNames keysAndValuesDo: 
		[ :aSTFunctionCallName :aJSFunctionCallName | 
			functions << aJSFunctionCallName << ':' << self space << 'function(){' << crr << 
			'return this.doesNotUnderstand_(st.Message.selector_arguments_(st.string('.
			aSTFunctionCallName javascriptOn: functions.
			 functions << '), st.array(st.toArray(arguments)))); ' << '},' << crr].
	functions := functions contents.
	^ ((TextStream with: OrcaSqueakAstToJsAst classFunction, '("') << self doesNotUnderstandClassName 
			<< '", {' << cr <<
		'instanceMethods: {' << cr <<
			self indentation << functions << cr <<
		'}, ' << cr <<
		'classMethods: {' << cr <<
			self indentation << functions << cr <<
		'}' << self newline << 
		'});' << self newline) contents! !

!OrcaAbstractPrinter class methodsFor: 'pretty-print' stamp: 'ag 1/5/2011 20:01'!
indentation
	
	self subclassResponsibility! !

!OrcaAbstractPrinter class methodsFor: 'initialization' stamp: 'ag 4/14/2011 14:15'!
initialize
	
	super initialize.
	self initializeJavascriptCharacters.! !

!OrcaAbstractPrinter class methodsFor: 'converting' stamp: 'ag 4/14/2011 14:17'!
initializeJavascriptCharacters
	"Copied from JSStream class >> initializeEncoding"
	
	| printer escapes |
	JavascriptCharacters ifNotNil: [ ^ self ].
	
	printer := GRNumberPrinter new.
	printer base: 16; padding: $0; digits: 2.
	JavascriptCharacters := Array new: 127.
	0 to: 31 do: [ :index | 
		JavascriptCharacters 
			at: index + 1
			put: '\x' , (printer print: index) ].
	32 to: 126 do: [ :index | 
		JavascriptCharacters 
			at: index + 1
			put: index asCharacter ].
	escapes := #( 0 '\0' 7 '\a' 8 '\b' 9 '\t' 10 '\n' 12 '\f' 13 '\r' 34 '\"' 92 '\\' ).
	1 to: escapes size - 1 by: 2 do: [ :index | 
		JavascriptCharacters 
			at: (escapes at: index) + 1
			put: (escapes at: index + 1) ].! !

!OrcaAbstractPrinter class methodsFor: 'converting' stamp: 'ag 4/6/2011 15:52'!
inputConverter

	^ OrcaSqueakAstToJsAst! !

!OrcaAbstractPrinter class methodsFor: 'pretty-print' stamp: 'ag 1/5/2011 20:01'!
newline
	
	self subclassResponsibility! !

!OrcaAbstractPrinter class methodsFor: 'converting classes private' stamp: 'ag 1/10/2011 08:01'!
printEach: aBlock of: aCollection separatedBy: aString
	
	| result isFirst |
	isFirst := true.
	result := ''.
	aCollection do: [ :each | 
		isFirst
			ifTrue: [ isFirst := false ]
			ifFalse: [ result := result, aString ].
		result := result, (aBlock value: each)].
	^ result! !

!OrcaAbstractPrinter class methodsFor: 'converting classes private' stamp: 'ag 1/10/2011 10:16'!
printMethodsOf: aClass retrievingFrom: aBlock
	
	| methods |
	methods := aClass methodDict values.
	^ methods 
		ifEmpty: [ false ]
		ifNotEmpty: [
			'{', self newline,
			(self
				printEach: [ :eachmethod | | code |
					code := aBlock value: eachmethod.
					self indentation, self indentation, code key, ':', self space, code value ]
				of: methods
				separatedBy: ',', self newline),
			self newline, self indentation, '}' ].! !

!OrcaAbstractPrinter class methodsFor: 'converting classes private' stamp: 'ag 1/10/2011 10:43'!
printPart: aString with: aValue to: output guardedBy: aGuardBlock
	"Print a slot. If aValue is a String (should return false then), this is skipped.
	aGuardBlock is called with whether this part will be printed and returns, whether we have to prepend a colon + newline ('guards' whether we are the first line)."
	
	(aGuardBlock value: aValue isString)
		ifTrue: [ output << ',' << self newline ].
	aValue isString ifTrue: [
		output << self indentation << aString << ':' << self space << aValue ]! !

!OrcaAbstractPrinter class methodsFor: 'converting classes private' stamp: 'ag 4/13/2011 16:41'!
printSuperclass: aClass
	
	^ OrcaSqueakAstToJsAst prefixForGlobals, 
		(aClass superclass
			ifNil: [ self doesNotUnderstandClassName ]
			ifNotNil: [ aClass superclass name ])! !

!OrcaAbstractPrinter class methodsFor: 'converting classes private' stamp: 'ag 4/12/2011 12:29'!
printVariables: aCollection
	"Print a collection containing all members of aCollection separated by colons"
	
	"TODO its not good, that the printer accesses the escape-char from the ast-to-ast"
	
	^ aCollection
		ifEmpty: [ false ]
		ifNotEmpty: [
			'[', self space,
			(self
				printEach: [ :each | '"', OrcaSqueakAstToJsAst escapeCharForIdentifier, each, '"' ]
				of: aCollection
				separatedBy: ',', self space),
			self space, ']' ]! !

!OrcaAbstractPrinter class methodsFor: 'converting classes' stamp: 'ag 1/10/2011 09:10'!
resetTextStreams
	"These helper-TextStreams are recreated lazily"
	
	ClassTextOutputStream := TextOutputStream := JavascriptPrintStream := nil.! !

!OrcaAbstractPrinter class methodsFor: 'pretty-print' stamp: 'ag 1/5/2011 20:01'!
space
	
	self subclassResponsibility! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
arguments =
	
	translate*:identifiers
	-> [ self functionParameters: identifiers ]! !

!OrcaAbstractPrinter methodsFor: 'translate tokens' stamp: 'lw 4/6/2011 17:59'!
array = 
	
	translateWith(#expressionBase)*:contents
	-> [ '[', (contents separatedBy: ', '), ']' ]! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
assignment =
	
	translate! !

!OrcaAbstractPrinter methodsFor: 'pretty print' stamp: 'lw 4/6/2011 17:59'!
cSpace
	
	^ self class space! !

!OrcaAbstractPrinter methodsFor: 'translate tokens' stamp: 'lw 4/6/2011 17:59'!
comment =
	
	^string:comment
	-> [ self createComment: comment ]! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
comments =
	
	translateNamed('comment')*:commentStrings
	-> [commentStrings
		ifNil: [''] 
		ifNotNil: 
			[commentStrings separatedBy: '']]! !

!OrcaAbstractPrinter methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
createComment: aString

	| result |
	self indent.
	result := TextStream with: ''.
	result << self indentation << '// '.
	aString do: 
			[ :each |
			each = Character cr
				ifTrue: [result << self newline << self indentation <<  '//']
				ifFalse: [
					each = Character lf
						ifFalse: [result << each]]].
	result << self newline.
	self unindent.
	^ result contents! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
expression =
	
	expressionBase:expr
	-> [ self indentation, expr ]! !

!OrcaAbstractPrinter methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
expression: anExpressionBodyString withAssignments: aStringCollection
	
	^ ((aStringCollection isNil or: [ aStringCollection isEmpty ])
			ifTrue: [ '' ]
			ifFalse: [
				(aStringCollection separatedBy: self cSpace, '=', self cSpace),
				self cSpace, '=', self cSpace ])
					, anExpressionBodyString! !

!OrcaAbstractPrinter methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
expression: anExpressionBodyString withAssignments: aStringCollection withComments: aStringOfComments
	
	^ (aStringOfComments ifNil: ['']),
		((aStringCollection isNil or: [ aStringCollection isEmpty ])
			ifTrue: [ '' ]
			ifFalse: [
				(aStringCollection separatedBy: self cSpace, '=', self cSpace),
				self cSpace, '=', self cSpace ])
					, anExpressionBodyString! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
expressionBase =
	
	translateNamed('comments')?:comments translate:expressionBody translate*:assignments
	-> [ self expression: expressionBody withAssignments: assignments withComments: comments ]! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
functionCall =
	// Can be either an invokation of a named function of an object OR an invokation of a anonymous function
	
	&anything:receiverNode (nil | translate):receiver (^string | translate):functionName translate:arguments
	-> [ self
			functionCall: functionName
			to: ((receiverNode notNil and: [ receiverNode name = 'functionDeclaration' ])
					ifTrue: [ '(', receiver, ')' ]
					ifFalse: [ receiver ] )
			withArguments: arguments ]! !

!OrcaAbstractPrinter methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
functionCall: aFunctionNameOrDeclarationString to: aReceiverString withArguments: anArgumentsString
	
	| function |
	function := (aFunctionNameOrDeclarationString startsWith: 'function') "no test yet, necessary?"
		ifTrue: ['(', aFunctionNameOrDeclarationString, ')'] 
		ifFalse: [ aFunctionNameOrDeclarationString].
	
	^ (aReceiverString
		ifNotNil: [ aReceiverString, '.' ]
		ifNil: [ '' ]), function, anArgumentsString! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
functionContent =
	// This is intentionally not named 'statements', to deny all nested statements-nodes
	// The ast-to-ast step must flatten all statements into one list of expressions here
	
	[ self indent ]
	translate*:expressions
	[ self unindent ]
	-> [ expressions separatedBy: ';', self newline ]! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
functionDeclaration =
	
	translate:arguments
	translate:statements
	-> [ self functionDeclaration: arguments withBody: statements ]! !

!OrcaAbstractPrinter methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
functionDeclaration: aParameterString withBody: aStatementsString
	"aParameterString already contains the parantheses; #functionParameters: should be used."
	
	^ 'function', self cSpace, aParameterString, self cSpace, '{', self newline,
			(aStatementsString ifEmpty: [ '' ] ifNotEmpty: [ aStatementsString, ';' ]),
		self newline, self indentation, '}'! !

!OrcaAbstractPrinter methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
functionDeclaration: aParameterString withComment: aCommentString withBody: aStatementsString

	"aParameterString already contains the parantheses; #functionParameters: should be used."
	^ 'function', self cSpace, aParameterString, self cSpace, '{', self newline,
			(aCommentString ifNil: [ '' ]),
			(aStatementsString ifEmpty: [ '' ] ifNotEmpty: [ aStatementsString, ';' ]),
		self newline, self indentation, '}'! !

!OrcaAbstractPrinter methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
functionParameters: aStringCollection

	^ '(', ((aStringCollection collect: [ :aString | aString trimLeft ]) separatedBy: ', '), ')'! !

!OrcaAbstractPrinter methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
hasSelector: aSymbol
	"Adding the superclass method dictionary to the searcharea for matching methods is required because the printers don't implement all the methods on their own"
	
	^ ((super hasSelector: aSymbol) or: [ self class superclass includesSelector: aSymbol ])
		ifTrue: [ true ]
		ifFalse: [ self error: 'Unknown selector: ', aSymbol asString. false ]! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
header =
	
	^string:functionName arguments:arguments
	-> [ functionName -> arguments ]! !

!OrcaAbstractPrinter methodsFor: 'translate tokens' stamp: 'lw 4/6/2011 17:59'!
identifier =
	
	^string:identifier
	-> [ identifier ]! !

!OrcaAbstractPrinter methodsFor: 'pretty print' stamp: 'lw 4/6/2011 17:59'!
indent
	
	self indentation: self indentation, self class indentation! !

!OrcaAbstractPrinter methodsFor: 'accessing' stamp: 'lw 4/6/2011 17:59'!
indentation
	^ indentation! !

!OrcaAbstractPrinter methodsFor: 'accessing' stamp: 'lw 4/6/2011 17:59'!
indentation: anObject
	indentation := anObject! !

!OrcaAbstractPrinter methodsFor: 'initialization' stamp: 'lw 4/6/2011 17:59'!
initialize

	super initialize.
	self 
		indentation: ''.! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
javaScriptMethod =
	// This does not return a string, but an association from the method-name to the method-declaration
	
	translate:functionNameAndArguments
	translate:statements
	-> [ functionNameAndArguments key -> 
		(self
			functionDeclaration: functionNameAndArguments value
			withBody: statements) ]! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
method =
	// This does not return a string, but an association from the method-name to the method-declaration
	
	translate:functionNameAndArguments
	translateNamed('comments')?:comment
	translate:statements
	-> [ functionNameAndArguments key -> 
		(self
			functionDeclaration: functionNameAndArguments value
			withComment: comment
			withBody: statements) ]! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
newExpression =
	
	// Here is a function-call, which is handled a little bit differently. Code copied from functionCall.
	
	{ &anything:receiverNode (nil | translate):receiver (^string | translate):functionName translate:arguments }
	-> [ 'new (', (self
			functionCall: functionName, ')'
			to: ((receiverNode notNil and: [ receiverNode name = 'functionDeclaration' ])
					ifTrue: [ '(', receiver, ')' ]
					ifFalse: [ receiver ] )
			withArguments: arguments) ]! !

!OrcaAbstractPrinter methodsFor: 'pretty print' stamp: 'lw 4/6/2011 17:59'!
newline
	
	^ self class newline! !

!OrcaAbstractPrinter methodsFor: 'translate tokens' stamp: 'lw 4/6/2011 17:59'!
number =
	
	^number:number
	-> [ number asString ]! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
objectLiteral =

	translateNamed(#slotAssignment)*:result
	-> [ '{', (result separatedBy: ',', self cSpace) , '}' ]! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
paranthesizedExpression =
	
	expressionBase:value
	-> [ '(', value, ')' ]! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
reservedWord =
	
	^string! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
returnExpression =
	
	expressionBase:expr
	-> [ self indentation, 'return ', expr ]! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
slotAccess =
	
	(nil | translate):object ^symbol:slotName
	-> [ (object ifNotNil: [ object, '.' ] ifNil: [ '' ]), slotName asString ]! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
slotAssignment =
	
	^symbol:slotname translate:value
	-> [  slotname, self cSpace, ':', self cSpace, value ]! !

!OrcaAbstractPrinter methodsFor: 'translate tokens' stamp: 'lw 4/6/2011 17:59'!
string =
	
	^string:string
	-> [ self class asJSString: string ]! !

!OrcaAbstractPrinter methodsFor: 'pretty print' stamp: 'lw 4/6/2011 17:59'!
unindent
	
	self indentation: (self indentation allButLast: self class indentation size)! !

!OrcaAbstractPrinter methodsFor: 'translate complex' stamp: 'lw 4/6/2011 17:59'!
variableDeclaration =
	
	translateNamed('identifier')+:variableNames translateNamed('assignment')?:assignment
	-> [ self variableDeclaration: variableNames withAssignment: assignment ]! !

!OrcaAbstractPrinter methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
variableDeclaration: variableNames withAssignment: assignment
	"Acts as an expression, not terminated by a semi colon"
	
	| oneVar firstLine |
	firstLine := self indentation, 'var ', ((oneVar := variableNames size = 1)
		ifTrue: [ variableNames first ]
		ifFalse: [ (variableNames separatedBy: ';', self cSpace, 'var ')]).
	^ assignment 
		ifNil: [ firstLine ]
		ifNotNil: [
			oneVar
				ifTrue: [ self expression: assignment withAssignments: { firstLine } ]
				ifFalse: [ 
					firstLine, ';', self newline, self indentation, (self expression: assignment withAssignments: variableNames) ]]! !

OrcaAbstractPrinter subclass: #OrcaPrettyPrinter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

!OrcaPrettyPrinter class methodsFor: 'pretty-print' stamp: 'lw 1/4/2011 17:37'!
indentation

	^ '    '! !

!OrcaPrettyPrinter class methodsFor: 'pretty-print' stamp: 'sec 3/2/2011 16:08'!
newline
	
	^ Character lf asString! !

!OrcaPrettyPrinter class methodsFor: 'pretty-print' stamp: 'lw 1/4/2011 17:37'!
space

	^ ' '! !

!OrcaPrettyPrinter methodsFor: 'initialization' stamp: 'lw 4/6/2011 17:59'!
initialize
	"Do everything indented x2 from the start, to have json-like pretty-print for the whole js-file."
	
	super initialize.
	self
		indent;
		indent.! !

OrcaAbstractPrinter subclass: #OrcaPrinter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

!OrcaPrinter class methodsFor: 'pretty-print' stamp: 'lw 1/4/2011 17:37'!
indentation

	^ ''! !

!OrcaPrinter class methodsFor: 'pretty-print' stamp: 'ag 1/5/2011 20:02'!
newline
	
	^ ''! !

!OrcaPrinter class methodsFor: 'pretty-print' stamp: 'lw 1/4/2011 17:30'!
space

	^ ''! !

!OrcaAstReader class methodsFor: 'converting private' stamp: 'ag 12/22/2010 16:20'!
applyRule: aRule on: aMatcher
	
	^ super applyRule: #translate on: aMatcher! !

!OrcaAstReader class methodsFor: 'converting private' stamp: 'ag 4/6/2011 18:12'!
convertAst: anOrcaNodeTree

	| matcher result |
	matcher := self matcherOn: {anOrcaNodeTree} readStream.
	matcher currentMethod: nil.
	result := self applyRule: #translate on: matcher.
	^ result! !

!OrcaAstReader class methodsFor: 'converting private' stamp: 'lw 4/7/2011 17:21'!
createInputConverterFrom: aString as: aRule insideMethod: aCompiledMethod
	
	^ self inputConverter convert: aString as: aRule insideMethod: aCompiledMethod.! !

!OrcaAstReader class methodsFor: 'converting private' stamp: 'ag 12/8/2010 22:13'!
inputConverter
	"An AstReader should know, which converter-class can provide the ast wants to read"
	
	self subclassResponsibility! !

!OrcaAstReader methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
hasSelector: aSymbol
	"Offers a hook to catch, if an unknown node occurs"
	
	^ self class includesSelector: aSymbol! !

!OrcaAstReader methodsFor: 'ometa helpers' stamp: 'lw 4/6/2011 17:59'!
translate =
	// Translate the next node using it's name as rule
	
	translateNamed(nil)! !

!OrcaAstReader methodsFor: 'ometa helpers' stamp: 'lw 4/6/2011 17:59'!
translateNamed :nodeName =
	// Assert, that the next node has the name nodeName and translate it using it's name as rule
	
	translateNamedWith(nodeName. nodeName)! !

!OrcaAstReader methodsFor: 'ometa helpers' stamp: 'lw 4/6/2011 17:59'!
translateNamedWith :nodeName :targetRule =
	// Assert, that the next node has the name nodeName and translate it with targetRule
	
	translateNamedWithArgument(nodeName. targetRule. nil)! !

!OrcaAstReader methodsFor: 'ometa helpers' stamp: 'lw 4/6/2011 17:59'!
translateNamedWithArgument :nodeName :targetRule :argument =
	// Assert, that the next node has the name nodeName, use targetRule to convert the node and pass argument when calling targetRule
	
	// Lookahead for one node
	&anything:node
	
	// Some assertions, e.g. about the nodeName
	?[ node class = OrcaNode ]
	?[ nodeName isNil | (node name = nodeName )]
	[ targetRule ifNil: [ node name asSymbol ] ifNotNil: [ targetRule asSymbol ]]:appliedRule
	?[ self hasSelector: appliedRule ]
	
	// If an argument is given, call the targetRule with that argument. Else, call the targetRule without arguments
	(( ?[ argument notNil ] { apply(appliedRule. argument):result } )
	| { apply(appliedRule):result })
	
	// Return the parsed content of the node
	-> [ result ]! !

!OrcaAstReader methodsFor: 'ometa helpers' stamp: 'lw 4/6/2011 17:59'!
translateWith :aRule =
	// Translate the next node with aRule, regardless of it's name
	
	translateNamedWith(nil. aRule)! !

!OrcaAstReader methodsFor: 'ometa helpers' stamp: 'lw 4/6/2011 17:59'!
translateWithArgument :aRule :argument =
	// Translate the next node with aRule, regardless of it's name, and pass aRule the argument
	
	translateNamedWithArgument(nil. aRule. argument)! !

OrcaAstReader subclass: #OrcaAstToAst
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

!OrcaAstToAst methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
passThroughAndTranslate =
	
	// TODO add all primitive values?..
	
	^symbol
|	string
|	number
|	char
|	nil
	
|	({ translate*:children }:node
	-> [ OrcaNode named: node name with: children ])! !

!OrcaAstToAst methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
renameTo :nodeName =
	
	translate*:contents
	-> [ OrcaNode named: nodeName with: contents ]! !

!OrcaAstToAst methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
translate = 
	
	^translate
|	passThroughAndTranslate! !

OrcaAstToAst subclass: #OrcaJsAstOptimizer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

!OrcaJsAstOptimizer class methodsFor: 'converting' stamp: 'ag 4/6/2011 18:12'!
inputConverter
	
	self error: 'The Js-Ast-Optimizer is not yet functional!!'.
	^ OrcaSqueakAstToJsAst! !

!OrcaJsAstOptimizer methodsFor: 'convert special cases' stamp: 'lw 4/6/2011 17:59'!
forLoop =
	// Special case of keywordExpression
	

	translate:receiver symbol:selector translate*:arguments
	?[ selector = #do: ]
	?[ arguments size = 1 ]
	[ arguments first ]:block
	?[ block size >= 1 ]
	?[ block first name = 'arguments' ]
	?[ block first size = 1 ]
	-> [ self forLoopOver: receiver do: block ]! !

!OrcaJsAstOptimizer methodsFor: 'convert special cases' stamp: 'lw 4/6/2011 17:59'!
forLoopOver: anArrayNode do: aBlockNode

	| arrayName statements aStatementsNode aTemporariesNode loopVariableName |
	arrayName := 'arr$'.
	loopVariableName := aBlockNode first first.
	aStatementsNode := aBlockNode detect: [:each | each name = 'statements'] ifNone: [ nil ].
	aTemporariesNode := aBlockNode detect: [:each | each name = 'temporaries'] ifNone: [ nil ].
	aStatementsNode ifNotNil: [ statements := self apply: #translate withArgs: aStatementsNode].
	^ OrcaNode statements:
		{OrcaNode variableDeclaration:
			{OrcaNode identifier: arrayName.
			  OrcaNode expression: anArrayNode}.
		 OrcaNode forLoop:
			{OrcaNode identifier: arrayName.
			loopVariableName.
			aTemporariesNode.
			statements}}! !

!OrcaJsAstOptimizer methodsFor: 'convert complex' stamp: 'lw 4/6/2011 17:59'!
keywordExpression = 
	
	forLoop
|	messageSendExpression // ...not correct ...
! !

OrcaAstToAst subclass: #OrcaSqueakAstToJsAst
	instanceVariableNames: 'calledFunctions referedClasses temps'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

!OrcaSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 4/11/2011 16:06'!
arrayFunction
	
	^ self prefixForGlobals, 'array'! !

!OrcaSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 4/11/2011 16:06'!
blockFunction
	
	^ self prefixForGlobals, 'block'! !

!OrcaSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 4/11/2011 16:10'!
charFunction
	
	^ self prefixForGlobals, 'char'! !

!OrcaSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 4/11/2011 21:11'!
classFunction
	
	^ self prefixForGlobals, 'class'! !

!OrcaSqueakAstToJsAst class methodsFor: 'escaping' stamp: 'robi 1/4/2011 15:21'!
escapeCharForIdentifier
	
	^ '$'! !

!OrcaSqueakAstToJsAst class methodsFor: 'escaping' stamp: 'robi 1/4/2011 15:21'!
escapeCharForMethod
	
	^ '_'! !

!OrcaSqueakAstToJsAst class methodsFor: 'escaping' stamp: 'ag 4/12/2011 12:21'!
escapeGlobal: aStringOrSymbol
	"This only gets called, if it's clear, that aStringOrSymbol is really a class"
	
	^  self prefixForGlobals, aStringOrSymbol asString! !

!OrcaSqueakAstToJsAst class methodsFor: 'escaping' stamp: 'ag 4/12/2011 12:19'!
escapeReservedWord: aStringOrSymbol
	
	aStringOrSymbol ifNil: [ ^nil ].
	^ ((self jsReservedWords includes: aStringOrSymbol asString)
		ifTrue: [ self escapeCharForMethod ]
		ifFalse: [ '' ]),
			aStringOrSymbol asString! !

!OrcaSqueakAstToJsAst class methodsFor: 'converting' stamp: 'LT 3/1/2011 10:10:30.562'!
ignoredPrimitives
	"This returns an array of CompiledMethod. All primitive-declarations in these methods are ignored.
	This means, these methods must have a correct software-fallback-implementation."
	
	^ {
	self method: #@ of: Number.
	self method: #replaceFrom:to:with:startingAt: of: String.
	self method: #replaceFrom:to:with:startingAt: of: Symbol.
	self method: #replaceFrom:to:with:startingAt: of: Array.
	self method: #replaceFrom:to:with:startingAt: of: ByteString.
	self method: #atEnd of: PositionableStream.
	self method: #next of: ReadStream.
	self method: #species of: Object
	}! !

!OrcaSqueakAstToJsAst class methodsFor: 'converting' stamp: 'ag 4/6/2011 15:52'!
inputConverter
	
	^ OrcaParser! !

!OrcaSqueakAstToJsAst class methodsFor: 'mapping names' stamp: 'ag 4/12/2011 13:46'!
jsFunctionNameFor: aStringOrSymbol
	
	| result |
	result := ''.
	aStringOrSymbol asString do: [ :each | | jsIdentifier |
		jsIdentifier := self squeakCharacterToJsIdentifier at: each ifAbsent: [each asString].
		result := result, jsIdentifier].
	^ self escapeReservedWord: result! !

!OrcaSqueakAstToJsAst class methodsFor: 'mapping names' stamp: 'ag 3/29/2011 10:46'!
jsInstVarNameFor: aString
	
	^ self escapeCharForIdentifier asString, aString! !

!OrcaSqueakAstToJsAst class methodsFor: 'mapping names' stamp: 'ag 4/12/2011 12:35'!
jsKeywordFor: aString
	"thisContext is not covered, just a stub leading to a not-defined-error.
	The returned association maps from the literal evaluating to the squeak-object to the literal evaluating to the underlying js-object."
	
	^ aString
		caseOf: {
			[ 'self' ] -> [ 'this' ].
			[ 'true' ] -> [ self prefixForGlobals, 'true' ].
			[ 'false' ] -> [ self prefixForGlobals, 'false' ].
			[ 'nil' ] -> [ self prefixForGlobals, 'nil' ].
			"In Squeak, using super as identifier (e.g. a := super. ) without calling functions on it, seems to have the same effect as using self."
			"Calling super-functions is not handled here"
			[ 'super' ] -> [ 'this' ].
			"thisContext is not implemented in any way"
			[ 'thisContext' ] -> [ 'THIS_CONTEXT' ].
		}
		otherwise: [ self error: 'Cannot translate keyword to JS: ', aString ]! !

!OrcaSqueakAstToJsAst class methodsFor: 'escaping' stamp: 'ag 4/12/2011 12:30'!
jsReservedWords
	"http://javascript.about.com/library/blreserved.htm"
	
	^ #('abstract' 'as' 'boolean' 'break' 'byte' 'case' 'catch' 'char' 'class' 'continue' 'const' 'debugger' 'default' 'delete' 'do' 'double' 'else' 'enum' 'export' 'extends' 'final' 'finally' 'float' 'for' 'function' 'goto' 'if' 'implements' 'import' 'in' 'instanceof' 'int' 'interface' 'is' 'long' 'namespace' 'native' 'new' 'null' 'package' 'private' 'protected' 'public' 'return' 'short' 'static' 'super' 'switch' 'synchronized' 'this' 'throw' 'throws' 'transient' 'try' 'typeof' 'use' 'var' 'void' 'volatile' 'while' 'with' 'arguments'
	) 
	asOrderedCollection
		add: self namespaceForGlobals;
		yourself! !

!OrcaSqueakAstToJsAst class methodsFor: 'escaping' stamp: 'ag 4/12/2011 12:31'!
namespaceForGlobals

	^ 'st'! !

!OrcaSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 4/11/2011 16:09'!
nonLocalReturnFunction
	
	^ self prefixForGlobals, 'nonLocalReturn'! !

!OrcaSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 4/11/2011 16:06'!
numberFunction
	
	^ self prefixForGlobals, 'number'! !

!OrcaSqueakAstToJsAst class methodsFor: 'mapping names' stamp: 'ag 4/12/2011 16:28'!
prefixForCommunication
	
	^ self prefixForGlobals, 'communication.'! !

!OrcaSqueakAstToJsAst class methodsFor: 'mapping names' stamp: 'ag 4/12/2011 12:32'!
prefixForGlobals

	^ self namespaceForGlobals, '.'! !

!OrcaSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 4/11/2011 16:06'!
primitiveCallFunction
	
	^ self prefixForGlobals, 'primitiveDeclaration'! !

!OrcaSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 4/12/2011 16:26'!
serverBlockFunction
	
	^ self prefixForCommunication, 'serverBlock'! !

!OrcaSqueakAstToJsAst class methodsFor: 'mapping names' stamp: 'lw 2/17/2011 16:33'!
squeakCharacterToJsIdentifier
	"Several characters can be used for selectors in squeak but
	are not allowed for JavaScript functionname"
	
	"TODO check, if one of these might be a reserved word (would cause conflicts with unary message named that way). Builds exception-rule."
	
	^ (Dictionary newFrom:
		{$: -> ''.  
		$+ -> 'plus'.
		$- -> 'minus'.
		$* -> 'times'.
		$/ ->  'slash'.
		$\ -> 'backslash'.
		$~ -> 'tilde'.
		$< ->  'less'.
		$> -> 'greater'.
		$= -> 'equals'.
		$@ -> 'at'.
		$% -> 'percent'.
		$& -> 'and'.
		$? -> 'question'.
		$!! -> 'exclamation'.
		$` -> 'backtick'.
		$, -> 'comma'.
		$ -> 'plusminus'.
		$| -> 'pipe'})
			collect: [ :string | self escapeCharForMethod, string ]! !

!OrcaSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 4/11/2011 16:01'!
stringFunction
	
	^ self prefixForGlobals, 'string'! !

!OrcaSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 4/11/2011 16:05'!
superFunction
	
	^ self prefixForGlobals, 'super'! !

!OrcaSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 4/28/2011 11:52'!
unboxSlotObjectFunction
	
	^ self prefixForGlobals, 'unboxSlots'! !

!OrcaSqueakAstToJsAst methodsFor: 'information gathering' stamp: 'lw 4/27/2011 18:19'!
addCollectedClass: aClass

	self referedClasses add: aClass	
	! !

!OrcaSqueakAstToJsAst methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
addReturnExpression: statements
	
	(statements isEmpty or: [(statements last name = 'returnExpression') not])
		ifTrue: [
			statements add: (OrcaNode returnExpression: { OrcaNode reservedWord: 'this' })].! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'ag 4/12/2011 13:41'!
arguments =
	
	lookaheadTemps
	renameTo('arguments')! !

!OrcaSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'lw 4/6/2011 17:59'!
array =
	// Pack inner nodes in expression-nodes
	
	translate*:elements
	arrayBase(elements collect: [ :each | OrcaNode expression: { each }])! !

!OrcaSqueakAstToJsAst methodsFor: 'ometa helpers' stamp: 'lw 4/6/2011 17:59'!
arrayBase :collection =
	
	wrapAsPrimitive(
		self class arrayFunction.
		'array'.
		collection)! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'lw 4/6/2011 17:59'!
binaryExpression =
	
	messageSendExpression! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'lw 4/6/2011 17:59'!
block =
	
	translateNamed('arguments')?:arguments
	methodContent:statements
	
	// All return-statements in the block are nonlocal returns. After that, the last expression into a return-expression like in a method.
	// Nonlocal returns are function calls in JS: nonLocalReturn()
	[ self renameNonLocalReturns: statements ]
	renameLastExpression(statements)
	
	[OrcaNode named: 'functionDeclaration' withoutNils: 
		{( arguments 
			ifNotNil: [ arguments ] 
			ifNil: [ OrcaNode arguments ] ).
		statements }]:blockContent
	
	-> [(OrcaNode functionCall: 
		{ nil.
		self class blockFunction.
		OrcaNode arguments: {blockContent}}) ]! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'ag 4/12/2011 12:04'!
braceExpression =
	// take the contents of the expression as it's js-arg-value. In a brace-expression, there is no assigment.
	
	translate*:expressions
	arrayBase(expressions)! !

!OrcaSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'lw 4/6/2011 17:59'!
byteArray =
	
	array! !

!OrcaSqueakAstToJsAst methodsFor: 'information gathering' stamp: 'lw 4/11/2011 12:37'!
calledFunctionNamesAt: aStringOrSymbol put: anotherStringOrSymbol 

	(aStringOrSymbol isString
			or: [aStringOrSymbol isSymbol])
		ifTrue: [^ self calledFunctions 
					at: aStringOrSymbol asSymbol 
					put: anotherStringOrSymbol asSymbol]! !

!OrcaSqueakAstToJsAst methodsFor: 'accessing' stamp: 'lw 4/6/2011 17:59'!
calledFunctions
	^ calledFunctions! !

!OrcaSqueakAstToJsAst methodsFor: 'accessing' stamp: 'lw 4/6/2011 17:59'!
calledFunctions: anObject
	calledFunctions := anObject! !

!OrcaSqueakAstToJsAst methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
cascadeReceiver
	
	^ OrcaNode identifier: ('cascadeReceiver', self class escapeCharForIdentifier)! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'lw 4/6/2011 17:59'!
cascadedExpression =
	
	translate:receiver
	translateWithArgument(#keywordExpression. self cascadeReceiver)*:functions
	-> [ self cascadedExpression: functions to: receiver ]! !

!OrcaSqueakAstToJsAst methodsFor: 'helpers' stamp: 'ag 4/12/2011 14:15'!
cascadedExpression: functions to: receiver
	"The statements-node created here is later flattened into one flat list of expressions."
	
	| statements |
	statements := OrcaNode functionDeclaration:
		{ OrcaNode arguments. 
		OrcaNode functionContent: 
			{ OrcaNode variableDeclaration: 
				{ self cascadeReceiver. 
				OrcaNode assignment: { receiver }}}}.
	statements last addAll: (functions collect: [ :each | self expression: OrcaNode expression add: each ]).
	statements last last name: 'returnExpression'.
	^ self
		functionCall: 'apply'
		to: statements
		withArguments: { OrcaNode reservedWord: 'this' }! !

!OrcaSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'lw 4/6/2011 17:59'!
character =
	
	char:char
	wrapAsPrimitive(self class charFunction. 'string'. char asString) // the 'string' here is the node inside the character()-function call. The Printer does not know about characters.
	! !

!OrcaSqueakAstToJsAst methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
checkSingleArgumentSelector: aSymbol
	"Answer, whether aSymbol is a selector for a keyword-message with one argument.
	aSymbol is already is keyword-selector.
	TODO Please implement a better way to do this."
	
	| tokens |
	tokens := aSymbol findTokens: $:.
	^ tokens size = 1 and: [ tokens first = (aSymbol asString subStrings: ':') first ]! !

!OrcaSqueakAstToJsAst methodsFor: 'information gathering' stamp: 'lw 4/27/2011 18:22'!
collectReferedClasses: aString
	
	(self isLocal: aString)
		ifFalse: [ (Smalltalk classNamed: aString) 
			ifNotNil: [ :aClass | 
				self addCollectedClass: aClass ] ]! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'lw 4/6/2011 17:59'!
comment =
	//Neccessary to avoid packing String into calls to the string-wrapper function
	^string:string
	
	-> [ OrcaNode comment: string asString ]! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'lw 4/6/2011 17:59'!
comments =
	
	anything:comments
	?[ comments name = 'comments' ]
	(?[comments isEmpty] -> [nil]
|	-> [ comments ])
	! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'lw 4/6/2011 17:59'!
doItSnippet =
	// No need to turn the last expression into a returnExpression, like in a method
	
	methodContent! !

!OrcaSqueakAstToJsAst methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
expression =

	comments?:comments
	translate*:nodes
	->[ (comments
			ifNil: [ nodes ]
			ifNotNil: [ {comments}, nodes  ])
		inject: OrcaNode expression 
		into: [ :newCollection :each | self expression: newCollection add: each ]]! !

!OrcaSqueakAstToJsAst methodsFor: 'helpers' stamp: 'ag 4/12/2011 14:15'!
expression: newCollection add: each
	"kein double dispatch, da Nodes namen haben und nicht nach klassen aufgeteilt sind."
	
	each name = 'returnExpression'
		ifTrue: 
			[ ^each ].
	each name = 'expression'
		ifTrue: 
			[ ^ newCollection 
				addAllLast: each;
				yourself ].
	^ newCollection 
		addLast: each; 
		yourself! !

!OrcaSqueakAstToJsAst methodsFor: 'helpers' stamp: 'ag 4/11/2011 20:05'!
functionCall: selector to: receiver withArguments: arguments
	"In case of a super-call: the functionName is a _super-function-call itself: _super('field')() "
	
	| functionName |
	functionName := selector isString ifTrue: [ self class jsFunctionNameFor: selector ] ifFalse: [ selector ].
	^ OrcaNode functionCall: {
		receiver.
		functionName.
		OrcaNode arguments: arguments }.
	
	! !

!OrcaSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'ag 4/14/2011 11:51'!
identifier =
	
	^string:name
	[ self jsIdentifierFor: name ]:result
	[ self collectReferedClasses: name ]
	-> [ result ]! !

!OrcaSqueakAstToJsAst methodsFor: 'initialization' stamp: 'ag 4/12/2011 17:57'!
initialize

	super initialize.
	self calledFunctions: Dictionary new.
	self referedClasses: Set new.
	self temps: Set new.
	self temps add: self cascadeReceiver first. "The cascadeReceiver$-variable must not be prepended with the namespace or escaped!!"! !

!OrcaSqueakAstToJsAst methodsFor: 'helpers' stamp: 'ag 4/14/2011 11:56'!
isLocal: aString
	
	^ self temps includes: aString asString! !

!OrcaSqueakAstToJsAst methodsFor: 'helpers' stamp: 'ag 4/14/2011 12:00'!
jsIdentifierFor: aString
	"Escape reserved js-words and instance variables."
	
	^ (self isLocal: aString)
	ifTrue: [
		"Temporary or method-argument"
		OrcaNode named: 'identifier' with: { self class escapeReservedWord: aString } ]
	ifFalse: [
		(self currentMethod notNil and: [ self currentMethod methodClass allInstVarNames includes: aString ])
			ifTrue: [
				"Access to instance/class-variable"
				OrcaNode named: 'slotAccess' with: { 
					OrcaNode named: 'reservedWord' with: { 'this' }. 
					(self class jsInstVarNameFor: aString) asSymbol } ]
			ifFalse: [
				"Access to global"
				OrcaNode named: 'identifier' with: { self class escapeGlobal: aString } ] ]! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'ag 4/11/2011 15:46'!
keywordExpression =

	messageSendExpression! !

!OrcaSqueakAstToJsAst methodsFor: 'ometa helpers' stamp: 'ag 4/12/2011 13:39'!
lookaheadTemps =
	// This does not change the input stream.
	
	&(anything*):identifiers
	[ self temps addAll: (identifiers collect: [ :each | each first ]) ]! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'ag 4/12/2011 13:40'!
messagePattern =
	
	^symbol:selector lookaheadTemps translate*:arguments
	-> [ OrcaNode named: 'header' with: { self class jsFunctionNameFor: selector }, arguments ]! !

!OrcaSqueakAstToJsAst methodsFor: 'ometa helpers' stamp: 'ag 4/11/2011 20:06'!
messageSendExpression =
	
	superCall
	
|	(translate:receiver ^symbol:selector translate*:arguments
	
	// TODO refactor; copied from #functionCall:to:withArguments:
	[ 
	self calledFunctionNamesAt: selector put: 
			(selector isString ifTrue: [ self class jsFunctionNameFor: selector ] ifFalse: [ selector ]) ]
	
	-> [ self functionCall: selector to: receiver withArguments: arguments ])! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'lw 4/6/2011 17:59'!
method =
	
	translateWith(#messagePattern):header
	comments?:comments
	methodContent:statements
	[ self addReturnExpression: statements ]
	
	-> [ OrcaNode named: 'method' withoutNils: { header. comments. statements }]! !

!OrcaSqueakAstToJsAst methodsFor: 'ometa helpers' stamp: 'lw 4/6/2011 17:59'!
methodContent  =
	// This can also be used in a place, where primitive-declarations are not allowed: the parser already handles that correctly
	// This flattens all expressions (cascaded expressions in the first place) into one flat collection of expressions

	translateNamed('temporaries')?:temporaries
	translateNamed('primitiveDeclaration')*:primitives
	translate?:statements
	-> [ self statements: statements withTemps: temporaries andPrimitives: primitives ]! !

!OrcaSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'lw 4/6/2011 17:59'!
number =
	// Number-literals are packed into a 'primitive' number-function-call
	
	^number:number
	wrapAsPrimitive(self class numberFunction. 'number'. number)! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'ag 4/12/2011 12:03'!
paranthesizedExpression =
	// I just found this method empty (without comment), don't really remember, why this works, but it does.
	
	translate! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'lw 4/6/2011 17:59'!
primitiveDeclaration =
	// Some primitive-declarations are just ignored, because the software implementation is perfectly fine.
	
	(?[ self class ignoredPrimitives includes: self currentMethod ]
	anything*
	-> [ nil ])
|	
	(anything // ignored for now. No need to remember the primitive-number, the js-implementation will know which method this is in
	-> [ OrcaNode expression:
		{ self functionCall: self class primitiveCallFunction to: nil withArguments: {} } ])! !

!OrcaSqueakAstToJsAst methodsFor: 'accessing' stamp: 'lw 4/11/2011 12:34'!
referedClasses
	^ referedClasses! !

!OrcaSqueakAstToJsAst methodsFor: 'accessing' stamp: 'lw 4/11/2011 12:34'!
referedClasses: anObject
	referedClasses := anObject! !

!OrcaSqueakAstToJsAst methodsFor: 'ometa helpers' stamp: 'ag 4/11/2011 21:43'!
renameLastExpression :statements =
	
	[ statements ifNotEmpty: [ 
		(statements last name = 'expression'
			and: [statements last first name = 'functionCall']
				and: [statements last first second = 'nonLocalReturn'])   
		 			ifFalse: [ statements last name: 'returnExpression' ] ] ]
																				// WOW this is a NICE one!!!! LAST FIRST SECOND !!!!!!
! !

!OrcaSqueakAstToJsAst methodsFor: 'helpers' stamp: 'ag 4/11/2011 16:12'!
renameNonLocalReturns: statements
	
	^ statements doWithIndex: [ :st :index |
		st name = 'returnExpression' 
			ifTrue: [ | contents |
				st name: 'expression'.
				contents := st first.
				st removeAll; add: (
					self
						functionCall: self class nonLocalReturnFunction
						to: nil
						withArguments: { contents }) ]]! !

!OrcaSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'ag 4/12/2011 12:17'!
reservedWord =
	
	^string:keyword
	-> [ OrcaNode reservedWord: (self class jsKeywordFor: keyword) ]! !

!OrcaSqueakAstToJsAst methodsFor: 'convert exceptions' stamp: 'ag 4/11/2011 21:12'!
serverBlock =
	
	&anything:block
	translateNamed('block') ^symbol:selector
	?[ selector = #onServer ]
	-> [ self
		functionCall: self class serverBlockFunction
		to: nil
		withArguments: { OrcaNode string: block originalCode } ]! !

!OrcaSqueakAstToJsAst methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
statements: aNode withTemps: anElement andPrimitives: anElementCollection
	"All nils in the primitives-collection are removed (can occur, see #primitiveDeclaration)"
	
	| result |
	result := aNode ifNil: [ OrcaNode statements ] ifNotNil: [ aNode ].
	anElement ifNotNil: [ result addFirst: anElement; yourself ].
	anElementCollection ifNotNil: [
		result addAllFirst: (anElementCollection reject: [ :each | each isNil ]); yourself ].

	^ result name: 'functionContent'; yourself! !

!OrcaSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'lw 4/6/2011 17:59'!
string =
	// String/Symbol-literals are packed into a 'primitive' string-function-call
	
	^string:string // Also covers symbols
	wrapAsPrimitive(self class stringFunction. 'string'. string asString)! !

!OrcaSqueakAstToJsAst methodsFor: 'convert exceptions' stamp: 'lw 4/6/2011 17:59'!
superCall =
	// An exception to a message-send-expression: super-calls are translated in this way.
	
	{ ^string:superKeyword }:superNode ^symbol:selector translate*:arguments
	?[ superNode name = 'reservedWord' & superKeyword = 'super' ]
	-> [ 
		self
			functionCall: 
				(self
					functionCall: self class superFunction
					to: nil
					withArguments: { OrcaNode string: (self class jsFunctionNameFor: selector asString) })
			to: nil
			withArguments: arguments ]! !

!OrcaSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'lw 4/6/2011 17:59'!
symbol =
	
	string! !

!OrcaSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'lw 4/6/2011 17:59'!
symbolNode =
	
	^anything:symbolNode
	?[ symbolNode name  = 'symbol']
	-> [symbolNode at: 1]! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'ag 4/12/2011 13:38'!
temporaries =
	
	lookaheadTemps
	
	renameTo('variableDeclaration'):temporaries
	[ temporaries ifEmpty: [ nil ] ifNotEmpty: [ temporaries ] ]! !

!OrcaSqueakAstToJsAst methodsFor: 'accessing' stamp: 'ag 4/11/2011 20:47'!
temps
	^ temps! !

!OrcaSqueakAstToJsAst methodsFor: 'accessing' stamp: 'ag 4/11/2011 20:47'!
temps: anObject
	temps := anObject! !

!OrcaSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'lw 4/6/2011 17:59'!
unaryExpression =

	serverBlock // [ 'a + b' ] onServer value   -->   onServer("a + b").value()
|	messageSendExpression! !

!OrcaSqueakAstToJsAst methodsFor: 'ometa helpers' stamp: 'ag 4/12/2011 12:05'!
wrapAsPrimitive :functionName :nodeName :wrappedObject =
	// Wrap an object into a node, that will result in a function-call to functionName with the object as argument
	
	[ OrcaNode
		named: nodeName
		with: ( wrappedObject isCollection & wrappedObject isString not
			ifTrue: [ wrappedObject ]
			ifFalse: [ { wrappedObject } ]) ]:primitiveObject
	-> [ self
		functionCall: functionName
		to: nil
		withArguments: { primitiveObject } ]! !

!OrcaOMeta2 class methodsFor: 'helpers' stamp: 'ag 2/1/2011 10:34'!
allMethodsDo: aBlock
	"Execute aBlock with each parsable method in the image and return the sum of visited methods and classes."
	
	| numberOfObjects |
	numberOfObjects := 0.
	Smalltalk allClasses do: [ :eachClass |
		eachClass methodDict notEmpty  ifTrue: [
			numberOfObjects := numberOfObjects + 1.
			eachClass methodDict do: [ :eachMethod |
				numberOfObjects := numberOfObjects + 1.
				aBlock value: eachMethod ]]].
	^ numberOfObjects! !

!OrcaOMeta2 class methodsFor: 'helpers' stamp: 'ag 12/15/2010 16:25'!
allMethodsDoAndReport: aBlock
	"Execute aBlock for each method in the image, print which methods have failed
	and notify each 500 methods. Print status-report at the end."
	
	| converted succeeded total |
	converted := -1.
	succeeded := 0.
	total := Smalltalk allClasses inject: 0 into: [ :sum :each | sum + each methodDict size].
	Transcript cr; show: 'Total methods in image: ', total asString.
	[
		self allMethodsDo: 
			[ :eachMethod |
				[ converted := converted + 1. aBlock value: eachMethod. succeeded := succeeded + 1.
				(converted \\ 500 = 0) ifTrue: [ Transcript cr; show: 'Converted ', converted asString, ' methods.' ]
				]
					on: Exception do: [ :err | Transcript cr; show: '	', converted asString, ': ', eachMethod name, ' (', err, ')' ]].
	]
		ensure: [ Transcript cr; show: 'Finished!! ', succeeded asString, ' of ', converted asString, ' operations succeeded!!' ].! !

!OrcaOMeta2 class methodsFor: 'helpers' stamp: 'lw 4/7/2011 09:52'!
applyRule: aRule on: aMatcher

	^ aMatcher apply: aRule! !

!OrcaOMeta2 class methodsFor: 'helpers' stamp: 'ag 12/13/2010 14:34'!
classNamed: aClassOrClassname
	
	^ Smalltalk classNamed: aClassOrClassname asString! !

!OrcaOMeta2 class methodsFor: 'converting string-result' stamp: 'ag 4/13/2011 14:14'!
convert: aString as: aRule
	
	^ (self convert: aString as: aRule insideMethod: nil) result! !

!OrcaOMeta2 class methodsFor: 'converting' stamp: 'ag 4/13/2011 14:18'!
convert: aString as: aRule insideMethod: aCompiledMethod
	"Translate aString in a new parse using aRule. aCompiledMethod is additional contextual information, that influences
	e.g. printing of literals, if the class of aCompiledMethod contains instance-variables. It can be nil.
	Return the finished parser, that contains information about the parsing process."
	
	| matcher result input |
	matcher := self new 
		inputConverter: (self createInputConverterFrom: aString as: aRule insideMethod: aCompiledMethod); 
		yourself.
	input := matcher inputConverter isString 
				ifTrue: [ matcher inputConverter ] 
				ifFalse: [ { matcher inputConverter result } ]. "The outermost Collection (former an S2JNode) is not evaluated by Ometa "
	matcher 
		initInput: (OM2LazyStream for: input readStream withPos: 1);
		currentMethod: aCompiledMethod.
	result := self applyRule: aRule on: matcher.
	^ matcher finalize; 
			result: result; 
			yourself! !

!OrcaOMeta2 class methodsFor: 'tests' stamp: 'ag 12/15/2010 16:26'!
convertAllMethods
	"Convert all methods in this image."
	
	self allMethodsDoAndReport: [ :eachMethod | 
		self convertCompiledMethod: eachMethod].! !

!OrcaOMeta2 class methodsFor: 'tests' stamp: 'LT 3/16/2011 14:07:52.228'!
convertAllMethodsOf: aClass
	
	^ aClass methodDict notEmpty
		ifTrue: [ | methods |
			methods := Dictionary new.
			aClass methodDict valuesDo:
				[ :method | | result |
					[ result := self convertCompiledMethod: method ]
						on: Exception do: [ :err | self error: 'Failed to convert ', method printString, ': ', err ].
					methods at: result key put: result value].
			methods]
		ifFalse: [ self error: 'Cannot convert class ', aClass name ]! !

!OrcaOMeta2 class methodsFor: 'converting string-result' stamp: 'ag 4/13/2011 14:14'!
convertCompiledMethod: aCompiledMethod
	
	^ (self converterOfCompiledMethod: aCompiledMethod) result! !

!OrcaOMeta2 class methodsFor: 'converting string-result' stamp: 'ag 4/13/2011 14:15'!
convertDoIt: aString
	
	^ self convert: aString as: #doItSnippet! !

!OrcaOMeta2 class methodsFor: 'converting string-result' stamp: 'ag 4/13/2011 14:15'!
convertMethod: aString
	
	^ self convert: aString as: #method! !

!OrcaOMeta2 class methodsFor: 'converting string-result' stamp: 'ag 4/13/2011 14:16'!
convertMethod: aStringOrSymbol of: aClassOrClassname
	
	^ self convertCompiledMethod: (
		self
			method: aStringOrSymbol
			of: aClassOrClassname)! !

!OrcaOMeta2 class methodsFor: 'tests' stamp: 'ag 4/6/2011 15:52'!
convertRandomMethods: aNumber
	"Match some random methods from the image. Return a String summarizing the result.
	Failed methods are printed to the Transcript."
	
	| runs successes |
	runs := successes := 0.
	aNumber timesRepeat: 
		[ | method |
			method := self randomMethod.
			method ifNotNil: [
				[ runs := runs + 1. OrcaParser convertCompiledMethod: method. successes := successes + 1 ]
					on: Error do: [ :err | Transcript cr; cr; show: 'Failed to parse: ', method name, '. ', err ]]].
	^ runs asString, ' runs, ', successes asString, ' successes'! !

!OrcaOMeta2 class methodsFor: 'converting string-result' stamp: 'ag 4/13/2011 14:18'!
convertStatements: aString
	
	^ self convert: aString as: #statements! !

!OrcaOMeta2 class methodsFor: 'converting' stamp: 'ag 4/13/2011 14:19'!
converterOfCompiledMethod: aCompiledMethod
	"Convert aCompiledMethod and return the parser, that contains the parsing results."
	
	^ self
		convert: (
			(aCompiledMethod methodClass compilerClass == Compiler)
				ifTrue: [ self sourceOf: aCompiledMethod ]
				ifFalse: [ self decompiledSourceOf: aCompiledMethod ])
		as: #method
		insideMethod: aCompiledMethod! !

!OrcaOMeta2 class methodsFor: 'converting private' stamp: 'lw 4/7/2011 17:20'!
createInputConverterFrom: aString as: aRule insideMethod: aCompiledMethod
	
	self subclassResponsibility! !

!OrcaOMeta2 class methodsFor: 'helpers' stamp: 'ag 2/1/2011 10:31'!
decompiledSourceOf: aCompiledMethod
	
	^ aCompiledMethod decompile decompileString! !

!OrcaOMeta2 class methodsFor: 'helpers' stamp: 'ag 1/19/2011 15:59'!
method: aStringOrSymbol of: aClassOrClassname
	
	^ self
		method: aStringOrSymbol
		of: aClassOrClassname 
		ifAbsent: [ self error: 'Method ', aStringOrSymbol asString, ' not found in class ', aClassOrClassname asString ]! !

!OrcaOMeta2 class methodsFor: 'helpers' stamp: 'ag 1/19/2011 15:59'!
method: aStringOrSymbol of: aClassOrClassname ifAbsent: aBlock
	
	^ ((self classNamed: aClassOrClassname)
		lookupSelector: aStringOrSymbol asSymbol)
			ifNil: [ aBlock value ]! !

!OrcaOMeta2 class methodsFor: 'helpers' stamp: 'ag 1/10/2011 08:58'!
print: aString to: aFilename
	"Create new file aFilename (or override) and write aString in it"
	
	^ aString squeakToUtf8 putOn: (FileStream forceNewFileNamed: aFilename)! !

!OrcaOMeta2 class methodsFor: 'helpers' stamp: 'ag 2/1/2011 10:33'!
randomMethod
	
	| class result |
	result := nil.
	[ result isNil ] whileTrue: [
		class := Smalltalk allClasses atRandom.
		class methodDict notEmpty
			ifTrue: [ result := class methodDict atRandom ]].
	^ result! !

!OrcaOMeta2 class methodsFor: 'helpers' stamp: 'ag 12/8/2010 18:18'!
sourceOf: aCompiledMethod
	
	^ aCompiledMethod getSource asString! !

!OrcaOMeta2 methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
H =
	
	[ self halt ]! !

!OrcaOMeta2 methodsFor: 'information gathering' stamp: 'lw 4/11/2011 16:08'!
calledFunctions

	^ self inputConverter calledFunctions! !

!OrcaOMeta2 methodsFor: 'accessors' stamp: 'lw 4/6/2011 17:59'!
currentMethod
	
	^ currentMethod! !

!OrcaOMeta2 methodsFor: 'accessors' stamp: 'lw 4/6/2011 17:59'!
currentMethod: aCompiledMethod
	
	currentMethod := aCompiledMethod! !

!OrcaOMeta2 methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
fail =
	
	?[ false ]! !

!OrcaOMeta2 methodsFor: 'helpers' stamp: 'lw 4/7/2011 09:50'!
finalize

	"Do nothing. Is overwritten in OrcaParser"
	self apply: #end.
! !

!OrcaOMeta2 methodsFor: 'accessing' stamp: 'lw 4/7/2011 17:43'!
inputConverter
	^ inputConverter! !

!OrcaOMeta2 methodsFor: 'accessing' stamp: 'lw 4/7/2011 17:43'!
inputConverter: anObject
	inputConverter := anObject! !

!OrcaOMeta2 methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
nil =
	
	anything:object ?[ object isNil ]
	-> [ object ]! !

!OrcaOMeta2 methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
node: aString
	
	^ self node: aString with: {}! !

!OrcaOMeta2 methodsFor: 'information gathering' stamp: 'lw 4/11/2011 16:08'!
referedClasses

	^ self inputConverter referedClasses! !

!OrcaOMeta2 methodsFor: 'accessing' stamp: 'lw 4/7/2011 17:18'!
result
	^ result! !

!OrcaOMeta2 methodsFor: 'accessing' stamp: 'lw 4/7/2011 17:18'!
result: anObject
	result := anObject! !

OrcaOMeta2 subclass: #OrcaParser
	instanceVariableNames: 'comments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

!OrcaParser commentStamp: 'ag 12/7/2010 23:09' prior: 0!
tokens
	Rules, that read one single element in squeal source-code. They all don't return an S2JNode, all are parsed to the
	according Squeak-object (e.g. Number, Symbol, String).
	Spaces before and after the tokens in the source-code are NOT read!!
	This is because in some cases, tokens must be read exactly, without emitting any spaces.
	The things returned from here must not be converted anymore, especially the <> operator is unnecessary.
	
complex elements
	Bigger parts of squeak source-code. Most return an S2JNode (using #node:with:), which contains further complex elements or object read
	by rules in the tokens-protocol. As convention, the nodes (except for helper-rules like primary and messagePattern) should be named like the rules.
	Each rule is supposed to read spaces before applying itself!! This way, rules from this protocol can be called serially, without
	reading spaces inbetween. However, spaces must be read between single tokens.
	
helpers
	Squeak-methods, that are called from within ometa-rules (or helper-rules).
	OMeta2 return-parts of the rules (righthand of ->) cannot contain multiple expressions (?)...
!

!OrcaParser class methodsFor: 'converting private' stamp: 'lw 4/7/2011 17:20'!
createInputConverterFrom: aString as: aRule insideMethod: aCompiledMethod
	
	^ aString! !

!OrcaParser class methodsFor: 'parsing' stamp: 'ag 12/1/2010 11:04'!
parseNumber: aString
	"Parse aString to an instance of Number. Return nil, if parsing failed.
	Return nil also, if a number could be read, but not the whole String was processed.
	(Number readFrom: aString does not fail in that case)."
	
	| stream result |
	stream := ReadStream on: aString.
	result := Number readFrom: stream ifFail: [ nil ].
	stream atEnd ifFalse: [ result := nil ].
	^ result! !

!OrcaParser methodsFor: 'parse easy' stamp: 'lw 4/6/2011 17:59'!
array =
	
	$(
		(
			(spaces
				(((($# spaces)? array:array) -> [ array ])
				| nodeNamed(#symbolBaseForArray. 'symbol')
				| literalTokenBase
				):element
			) -> [ element ]
		)*:contents
	ch($))
	-> [ OrcaNode array: contents ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
binaryExpression =
	
	(binaryExpression | unaryExpression | primary):receiver
	spaces:comment binaryExpressionMessage:message
	-> [ OrcaNode named: 'binaryExpression' withoutNils:
		{ self extractCommentOf: comment. receiver }, message ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
binaryExpressionMessage =
	// Not returning a node, but a simple collection
	
	binarySelector:selector
	(unaryExpression | primary):argument
	-> [ { selector. argument } ]! !

!OrcaParser methodsFor: 'parse tokens' stamp: 'lw 4/6/2011 17:59'!
binarySelector =
	
	<$| | ((specialCharacter | $-) specialCharacter*)>:selector
	-> [ selector asSymbol ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
block =
	
	<
	ch($[):comment1
		( (ch($:) spaces identifier:argumentName -> [ argumentName ])+:arguments ch($|)?:argumentSeparator )?
		temporaries?:temporaries
		statements?:statements
	ch($]):comment2
	>:originalCode
		
	// Forbidden (and not covered above): Arguments without separator bar, but with statements or temporaries
	?[ argumentSeparator notNil | arguments isNil | ((temporaries isNil or: [temporaries isEmpty]) & (statements isNil or: [statements isEmpty])) ]
	
	-> [ (OrcaNode named: 'block' withoutNils: { self extractCommentOf: comment1.
			OrcaNode named: 'arguments' withoutNilsNotEmpty: arguments.
			temporaries. statements.
			self extractCommentOf: comment2})
				originalCode: originalCode ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
braceExpression =
	
	ch(${):comment1
		(expression:expr ch($.) -> [ expr ] )*:expressions
		expression?:lastExpression
	ch($}):comment2
	
	-> [ self braceExpressionWith: expressions and: lastExpression ]! !

!OrcaParser methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
braceExpressionWith: expressions and: lastExpression
	
	^ OrcaNode named: 'braceExpression' withoutNils: 
		((expressions
			ifNotNil: [ expressions ]
			ifNil: [ {} ])
				asOrderedCollection
					add: lastExpression;
					yourself)! !

!OrcaParser methodsFor: 'parse easy' stamp: 'lw 4/6/2011 17:59'!
byteArray =
	// Byte array can only contain number-objects, don't pack them into nodes
	
	$# ch($[)
		(
			(spaces number:number [ number first ]:byte
			?[ (byte isInteger) & (byte <= 255) & (byte >= 0)] 
			) -> [ byte ]
		)*:contents
	ch($])
	-> [ OrcaNode byteArray: contents ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
cascadedExpression =
	// Constructs a node, whose first node is the receiver and all further nodes are messages sent to that receiver
	
	(keywordExpression | binaryExpression | unaryExpression):mainExpression
	(
		ch($;)
		(
		((spaces binarySelector:selector (unaryExpression | primary):argument) -> [ OrcaNode binaryExpression: { selector. argument }])
		| (((spaces keyword:selector (binaryExpression | unaryExpression | primary):argument) -> 
				[ selector -> argument])+:selectorsAndArguments -> [ (self keywordMessage: selectorsAndArguments) name: 'keywordExpression' ])
		| (spaces unarySelector:selector -> [ OrcaNode unaryExpression: { selector } ])
		):cascade
		-> [ cascade ]
	)+:cascades
	-> [ self cascadedExpression: mainExpression with: cascades ]! !

!OrcaParser methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
cascadedExpression: mainExpression with: cascades
	"The receiver of the mainExpression is the receiver of the whole cascade.
	It is the first node, the other nodes are the messages sent to that receiver"
	
	^ OrcaNode named: 'cascadedExpression' withoutNils: 
		((cascades
			ifNotNil: [ cascades ]
			ifNil: [ {} ])
				asOrderedCollection
					addFirst: mainExpression;
					addFirst: mainExpression removeFirst;
					yourself)! !

!OrcaParser methodsFor: 'ometa helpers' stamp: 'lw 4/6/2011 17:59'!
ch :c =
	// This reads a given element and all spaces before it and return the element
	// ch was supposed to mean character, but this can be applied to any object
	// Called like this: ch($=)
	
	spaces:comment exactly(c)
	-> [ comment ]! !

!OrcaParser methodsFor: 'parse easy' stamp: 'lw 4/6/2011 17:59'!
character =
	// The method in the super class is #char, don't confuse it!!
	
	( ($$ <anything>:value) | (seq('Character space') -> [ $  ]):value )
	-> [ OrcaNode character: value asCharacter ]! !

!OrcaParser methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
consumedBy: aBlock
	
	| result |
	result := super consumedBy: aBlock.
	result isString
		ifFalse: [ result := result separatedBy: '' ].
	^ result! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
doItSnippet =
	
	temporaries?:temps
	statements?:statements
	-> [ OrcaNode named: 'doItSnippet' withoutNils: { temps. statements } ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
expression =
	
	[ self startCollectingComments ]
	((spaces identifier:identifier (token(':=') | ch($_))) -> [ identifier ])*:variableNames 
	(cascadedExpression | keywordExpression | binaryExpression | unaryExpression | primary):expressionBody
	[ self stopCollectingComments ]:comment
	
	// First child is the expressionBody, further children are names of variables, that are assigned by this expression
	-> [ OrcaNode named: 'expression' withoutNils: ({ comment. expressionBody }, variableNames) ]! !

!OrcaParser methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
extractCommentOf: anOrderedCollection
	
	| result |
	true ifTrue: [^ nil]. "TODO skip all comments for now..."
	result := ''.
	anOrderedCollection do: [:item |
		item name = 'comment' ifTrue: [
			result := result , item first
			]].
	^ result isEmpty
		ifTrue: [nil]
		ifFalse: [self comment: result]! !

!OrcaParser methodsFor: 'helpers' stamp: 'lw 4/7/2011 09:50'!
finalize

	self apply: #spaces.
	super finalize! !

!OrcaParser methodsFor: 'parse easy' stamp: 'lw 4/6/2011 17:59'!
identifier =
	
	nodeNamed(#identifierBase. 'identifier')
|	(<$_ identifierBase>:decompiledIdentifier -> [ OrcaNode identifier: decompiledIdentifier ])! !

!OrcaParser methodsFor: 'parse tokens' stamp: 'lw 4/6/2011 17:59'!
identifierBase =
	
	<letter letterOrDigit*>! !

!OrcaParser methodsFor: 'commentTracing' stamp: 'lw 4/6/2011 17:59'!
initialize

	comments := Stack new.
	comments push: OrcaNodeNilObject new! !

!OrcaParser methodsFor: 'parse tokens' stamp: 'lw 4/6/2011 17:59'!
keyword =
	
	<identifierBase $:>:selector
	-> [ selector asSymbol ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
keywordExpression =
	
	(binaryExpression | unaryExpression | primary):receiver
	keywordExpressionMessage:message
	-> [OrcaNode keywordExpression: { receiver }, message ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
keywordExpressionMessage =
	
	(((spaces keyword:selector ( binaryExpression | unaryExpression | primary ):argument) -> [ selector -> argument ])+:selectorAndArguments)
	-> [ self keywordMessage: selectorAndArguments ]! !

!OrcaParser methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
keywordMessage: selectorAndArgumentNames
	"selectorAndArgumentNames contains associations, the part of the selector and the argument name.
	Resulting OrcaNode will have the selector as symbol on the first position and the argumentNames at the other positions."
	
	| selector result |
	selector := ''.
	result := OrcaNode named: 'keyword'.
	selectorAndArgumentNames do: [ :each |
		selector := selector, each key.
		result add: each value].
	^ result
		addFirst: selector asSymbol;
		yourself! !

!OrcaParser methodsFor: 'parse tokens' stamp: 'lw 4/6/2011 17:59'!
literalReservedWord =
	// Copy of reservedWord, with only the static values
	
	<(seq('true') | seq('false') | seq('nil'))>:reservedWord
	&(binarySelector | space | end | $] | $. | $; | $} | $)) // Look ahead to ensure, that this token ends here, don't match e.g. self2, but match self/2
	-> [ OrcaNode reservedWord: reservedWord ]! !

!OrcaParser methodsFor: 'parse easy' stamp: 'lw 4/6/2011 17:59'!
literalToken =
	// Anything, that can be evaluated to an object without context
	
	literalTokenBase
|	(($# spaces array:array) -> [ array ])! !

!OrcaParser methodsFor: 'parse tokens' stamp: 'lw 4/6/2011 17:59'!
literalTokenBase =
	// Things, that can be evalutated to an object without any context (array-literals excluded)
	
	literalReservedWord
|	number
|	symbol
|	character
|	string
|	byteArray! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
messagePattern =
	
	((spaces keyword:selectorPart spaces identifier:argumentName) -> [ selectorPart -> argumentName ])+:selectorAndArgumentNames
	-> [ self keywordMessage: selectorAndArgumentNames ]
|	spaces binarySelector:selector spaces identifier:argumentName
	-> [ OrcaNode binaryExpression: { selector. argumentName } ]	
|	spaces unarySelector:unarySelector
	-> [ OrcaNode unaryExpression: { unarySelector } ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
method =
	// Only one temporaries-declaration allowed, but can be before or after primitive-declaration
	
	messagePattern:head
	[ self startCollectingComments ]
	temporaries?:temps
	primitiveDeclaration*:primitives
	temporaries?:secondTemps ( ?[secondTemps isNil]  | (?[temps isNil] [secondTemps]:temps) )
	[ self stopCollectingComments ]:comment
	statements?:statements
	
	-> [ OrcaNode named: 'method' withoutNils: { head. comment. temps }, primitives, { statements } ]! !

!OrcaParser methodsFor: 'ometa helpers' stamp: 'lw 4/6/2011 17:59'!
nodeNamed :aRule :aNodeName =
	// Helper rule to encapsulate the result of a rule into a node
	
	apply(aRule):result
	-> [ OrcaNode named: aNodeName asString with: { result } ]! !

!OrcaParser methodsFor: 'parse easy' stamp: 'lw 4/6/2011 17:59'!
number =
	// Called that way because of super-class-method
	// The string is parsed once, assuring, that
	
	<
		($-? digit+ $r)?
		$-?
		(digit | upper)+
		($. (digit | upper)+)?
		($e ($- | $+)? (digit | upper)+)?
	>:result
	?[ (self class parseNumber: result) notNil ]
	-> [ OrcaNode number: (self class parseNumber: result) ]
	
	// An alternative way to evaluate a number...
	| ($# <digit>:value) -> [ OrcaNode number: value asNumber ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
primary =
	// something, that evaluates to an object
	
	spaces (literalToken | reservedWord | identifier):result -> [ result ]
|	block
|	braceExpression
|	ch($() expression:expr ch($)) -> [ expr name: 'paranthesizedExpression'; yourself ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
primitiveDeclaration =
	// Reads generally all primitive declarations, there are many of them
	// <primitive: 'a'> <primitive: 2> <primitive: 'a' module: 'b'> <version: 'ab'> <command> etc. etc.
	
	ch($<)
		(
			primitiveDeclarationRule(#keywordExpressionMessage)
			| (space primitiveDeclarationRule(#binaryExpressionMessage))
			| primitiveDeclarationRule(#unaryExpressionMessage)
		):message
	ch($>)
	-> [ OrcaNode named: 'primitiveDeclaration' with: { message } ]! !

!OrcaParser methodsFor: 'ometa helpers' stamp: 'lw 4/6/2011 17:59'!
primitiveDeclarationRule :aRule =
	// Read everything, that is inside the <>-primitive-declaration; write it back on the input stream, followed
	// by input, that will not be matched. Apply the rule and read the fake-input. Return the result after applying the rule.
	
	// This allows things, that are forbidden, like <primive: a> (a is an identifier and has been accepted, because it's also a unarySelector
	
	<
		((
			<spaces (literalToken | unarySelector | binarySelector | keyword)>:input ?[ (input includesSubString: '>') not ]
		) -> [ input ])*
	>:primitiveInput
	[ self apply: aRule withArgs: primitiveInput, '^^' ]:result
	$^ $^
	-> [ result ]! !

!OrcaParser methodsFor: 'parse easy' stamp: 'lw 4/6/2011 17:59'!
reservedWord =
	
	<(seq('true') | seq('false') | seq('nil') | seq('self') | seq('super') | seq('thisContext') )>:reservedWord
	&(binarySelector | space | end | $] | $. | $; | $} | $)) // Look ahead to ensure, that this token ends here, don't match e.g. self2, but match self/2
	-> [ OrcaNode reservedWord: reservedWord ]! !

!OrcaParser methodsFor: 'parse tokens' stamp: 'lw 4/6/2011 17:59'!
space =
	// A comment is equivalent to a space-character.
	// spaces are ignored (return nil), but comments result in a special node and are collected
	
	^space -> [ nil ]
	| (($" <(~$" anything)*>:result $") -> [ OrcaNode comment: result  ])! !

!OrcaParser methodsFor: 'parse easy' stamp: 'lw 4/6/2011 17:59'!
spaces =

	space*:whitespaces
	[ comments top addAllWhitoutNils: whitespaces ]! !

!OrcaParser methodsFor: 'parse tokens' stamp: 'lw 4/6/2011 17:59'!
specialCharacter =
	
	$+ | $* | $/ | $\ | $~ | $< | $> | $= | $@ | $% | $& | $? | $!! | $` | $, | $! !

!OrcaParser methodsFor: 'commentTracing' stamp: 'lw 4/6/2011 17:59'!
startCollectingComments

	comments push: (OrcaNode comments: {})! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
statements =
	
	(expression?:expr ch($.) -> [ expr ] )*:expressions
	(ch($^)?:returnSign expression:lastExpression ch($.)? ~expression)?
	-> [ self statementsWithExpressions: expressions lastExpression: lastExpression returnSign: returnSign ]! !

!OrcaParser methodsFor: 'helpers' stamp: 'lw 4/6/2011 17:59'!
statementsWithExpressions: expressions lastExpression: lastExpression returnSign: returnSign
	"Build the contents of a statements-node from the information gathered while parsing 'statements'"
	
	"Rename the last expression, if it was a return-expression"
	lastExpression notNil & returnSign notNil ifTrue: [
		lastExpression name = 'expression'
			ifTrue: [ lastExpression name: 'returnExpression' ]
			ifFalse: [ self error: 'Unexpected node at return-sign: ', lastExpression printString ]].
	expressions
		add: lastExpression;
		reject: [ :each | each isNil ].
	^ OrcaNode named: 'statements' withoutNilsNotEmpty: expressions! !

!OrcaParser methodsFor: 'commentTracing' stamp: 'lw 4/6/2011 17:59'!
stopCollectingComments
	
	^ comments pop ifEmpty: [ nil ]! !

!OrcaParser methodsFor: 'parse easy' stamp: 'lw 4/6/2011 17:59'!
string =
	// Called that way because of super-class-method
	
	$' ((~$' anything) | (($' $') -> [ $' ]))*:values $'
	-> [ OrcaNode string: (values asOrderedCollection inject: '' into: [ :a :b | a, b asString ]) ]! !

!OrcaParser methodsFor: 'parse easy' stamp: 'lw 4/6/2011 17:59'!
symbol =
	
	$# spaces (symbolBase | (string:stringSymbol -> [ stringSymbol first ])):value 
	-> [ OrcaNode symbol: value asSymbol ] 
	! !

!OrcaParser methodsFor: 'parse tokens' stamp: 'lw 4/6/2011 17:59'!
symbolBase =
	// This does not return a symbol, just consume input. Inside of array, symbols can be represented without
	// the #-character
	
	binarySelector
|	<keyword+ identifierBase>
|	<keyword+>
|	identifierBase
|	($: | $. | ($[ $]) | ${ | $} | $] | $^):result -> [ result asSymbol ]! !

!OrcaParser methodsFor: 'parse tokens' stamp: 'lw 4/6/2011 17:59'!
symbolBaseForArray =
	
	(~literalReservedWord
	(
		symbolBase
		| $[
	):result)
	-> [ result asSymbol ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
temporaries =
	
	ch($|)
		(spaces identifier:identifier -> [ identifier ])*:variableNames
	ch($|)
	-> [ OrcaNode temporaries: variableNames ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
unaryExpression =
	
	(unaryExpression | primary):receiver unaryExpressionMessage:message
	-> [ OrcaNode unaryExpression: {receiver. message} ]! !

!OrcaParser methodsFor: 'parse complex' stamp: 'lw 4/6/2011 17:59'!
unaryExpressionMessage =
	
	spaces unarySelector:message
	-> [ message ]! !

!OrcaParser methodsFor: 'parse tokens' stamp: 'lw 4/6/2011 17:59'!
unarySelector =
	
	identifierBase:selector ~$:
	-> [ selector asSymbol ]! !

!Boolean methodsFor: '*Orca-Core' stamp: 'lw 5/5/2011 10:24'!
asRemoteAnswerTo: aSession

	^ self serializeToJavaScript! !

!Boolean methodsFor: '*Orca-Core' stamp: 'ag 4/12/2011 12:00'!
definitionLine
	
	^ OrcaSqueakAstToJsAst jsKeywordFor: self asString! !

Dictionary subclass: #OrcaSlotObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Misc'!

!OrcaSlotObject class methodsFor: 'instance creation' stamp: 'ag 4/28/2011 13:29'!
newFrom: anArray
	
	(anArray allSatisfy: [ :association |
		(association respondsTo: #key) and: 
		[association respondsTo: #value] and:
		[association key isSymbol] ])
			ifFalse: [ self error: 'Cannot create slot-object from this array. Need only associations with symbols as keys: ', anArray ].
	^ super newFrom: anArray! !

!OrcaSlotObject methodsFor: 'serializing' stamp: 'ag 3/31/2011 13:24'!
definitionLine
	
	^ '{}'! !

!OrcaSlotObject methodsFor: 'serializing' stamp: 'ag 4/28/2011 12:39'!
fillInstVars: codeLines guardedBy: alreadySerialized
	"This classes instance variables are ignored, the slots are the only thing, that counts."
	
	self keysAndValuesDo: [ :key :value |
		value serializeToJavaScript: codeLines guardedBy: alreadySerialized.
		codeLines addLast: 
			(alreadySerialized at: self), '.', key, 
			'=', (alreadySerialized at: value) ].
	codeLines addLast: OrcaSqueakAstToJsAst unboxSlotObjectFunction, '(', (alreadySerialized at: self), ')'.! !

!OrcaSlotObject methodsFor: 'clientside' stamp: 'ag 4/28/2011 12:57'!
slotsDo: doBlock
	"On the client-side, this method iterated over all slot-names of the object, like the 'in'-operator of javascript.'"
	
	^ self keys do: doBlock! !

OrderedCollection subclass: #OrcaNode
	instanceVariableNames: 'name originalMethodName originalCode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

!OrcaNode commentStamp: 'ag 11/18/2010 13:09' prior: 0!
This is only a named collection, that is supposed to hold further collections or strings to form a tree!

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:31'!
arguments
	
	^ self arguments: {}! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:26'!
arguments: aCollection
	
	^ self named: 'arguments' with: aCollection! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:59'!
array: aCollection

	^ self named: 'array' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:27'!
assignment: aCollection

	^ self named: 'assignment' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:15'!
binaryExpression: aCollection

	^ self named: 'binaryExpression' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:31'!
block

	^ self block: {}.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:26'!
block: aCollection

	^ self named: 'block' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'LT 2/24/2011 13:34:07.814'!
braceExpression: aCollection

	^ self named: 'braceExpression' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:37'!
byteArray: aCollection

	^ self named: 'byteArray' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:12'!
cascadedExpression: aCollection

	^ self named: 'cascadedExpression' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:05'!
character: aCharacter

	aCharacter isCharacter ifFalse: [ self error: 'need a Character' ].
	^ self named: 'character' with: { aCharacter }.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'LT 4/5/2011 13:50:00.746'!
comment: aString

	aString isString ifFalse: [ self error: 'need a String' ].
	^ self named: 'comment' with: { aString }! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'LT 4/5/2011 13:49:56.145'!
comments: anOrderedCollection

	^ self named: 'comments' with: anOrderedCollection! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/25/2011 13:57'!
expression

	^ self named: 'expression' with: {}.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:27'!
expression: aCollection

	^ self named: 'expression' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:27'!
functionCall: aCollection
	"aCollection = { receiver. aMessageNameString. argumentsNode }"
	
	^ self named: 'functionCall' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:27'!
functionContent: aCollection

	^ self named: 'functionContent' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:28'!
functionDeclaration: aCollection

	^ self named: 'functionDeclaration' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:28'!
identifier: aString

	aString isString ifFalse: [ self error: 'need a String' ].
	^ self named: 'identifier' with: { aString }.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'robi 3/22/2011 13:47'!
javaScriptMethod: aCollection
	"aCollection = { header. statements }"
	^ self named: 'javaScriptMethod' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:11'!
keywordExpression: aCollection

	^ self named: 'keywordExpression' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'robi 3/22/2011 13:48'!
method: aCollection
	"aCollection = { header. statements }"
	^ self named: 'method' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation' stamp: 'lw 2/22/2011 17:45'!
named: aString
	
	^ self named: aString with: {}! !

!OrcaNode class methodsFor: 'instance creation' stamp: 'ag 11/18/2010 13:08'!
named: aString with: anOrderedCollection
	
	^ self new
		name: aString;
		addAll: anOrderedCollection;
		yourself! !

!OrcaNode class methodsFor: 'instance creation' stamp: 'ag 11/18/2010 13:58'!
named: aString withoutNils: anOrderedCollection
	
	anOrderedCollection ifNil: [ ^ nil ].
	^ self
		named: aString
		with: (anOrderedCollection reject: [ :each | each isNil])! !

!OrcaNode class methodsFor: 'instance creation' stamp: 'ag 11/21/2010 19:15'!
named: aString withoutNilsNotEmpty: anOrderedCollection
	"Returns nil, if resulting node would be empty"
	
	| result |
	result := self named: aString withoutNils: anOrderedCollection.
	^ (result isNil or: [result isEmpty])
		ifTrue: [ nil ]
		ifFalse: [ result ]! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 19:05'!
newExpression: aCollection

	^ self named: 'newExpression' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:28'!
number: aNumber

	aNumber isNumber ifFalse: [ self error: 'need a Number' ].
	^ self named: 'number' with: { aNumber }.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'LT 2/24/2011 13:34:59.983'!
objectLiteral: aCollection

	^ self named: 'objectLiteral' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:51'!
paranthesizedExpression: aCollection

	^ self named: 'paranthesizedExpression' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:28'!
reservedWord: aString

	aString isString ifFalse: [ self error: 'need a String' ].
	^ self named: 'reservedWord' with: { aString }.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:29'!
returnExpression: aCollection

	^ self named: 'returnExpression' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:29'!
slotAccess: aCollection

	^ self named: 'slotAccess' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:29'!
slotAssignment: aCollection

	^ self named: 'slotAssignment' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:31'!
statements

	^ self statements: {}.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:24'!
statements: aCollection

	^ self named: 'statements' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:29'!
string: aString

	aString isString ifFalse: [ self error: 'need a String' ].
	^ self named: 'string' with: { aString }.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:06'!
symbol: aSymbol

	aSymbol isSymbol ifFalse: [ self error: 'need a Symbol' ].
	^ self named: 'symbol' with: { aSymbol }.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:32'!
temporaries
	
	^ self temporaries: {}! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 18:24'!
temporaries: aCollection
	
	^ self named: 'temporaries' with: aCollection! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:48'!
unaryExpression: aCollection

	^ self named: 'unaryExpression' with: aCollection.! !

!OrcaNode class methodsFor: 'instance creation helpers' stamp: 'lw 2/22/2011 17:29'!
variableDeclaration: aCollection

	^ self named: 'variableDeclaration' with: aCollection.! !

!OrcaNode methodsFor: 'comparing' stamp: 'ag 11/21/2010 19:19'!
= anObject
	
	^ super = anObject and: 
		[ name = anObject name ]! !

!OrcaNode methodsFor: 'adding' stamp: 'lw 4/5/2011 18:19'!
addAllWhitoutNils: aCollection
	
	aCollection do: [ :each | each ifNotNil: [self add: each]]
! !

!OrcaNode methodsFor: 'testing' stamp: 'ag 4/6/2011 15:52'!
includesChildNamed: aString
	
	^ self name = aString
		or: [ self anySatisfy: [ :node | 
			(node isKindOf: OrcaNode) and: [
			node includesChildNamed: aString ] ] ]! !

!OrcaNode methodsFor: 'testing' stamp: 'LT 12/1/2010 14:06:25.154'!
isArrayNode

	^ name = 'array' or: [name = 'byteArray' or: [name = 'braceExpression']]! !

!OrcaNode methodsFor: 'testing' stamp: 'LT 2/24/2011 13:54:04.347'!
isAssociationNode

	^ (self name = 'binaryExpression' and: [ (self at: 2) = #-> ])
	or: [ self name = 'expression' and: [ self size = 1 ] and: [ (self at: 1) isAssociationNode ]]! !

!OrcaNode methodsFor: 'accessing' stamp: 'ag 1/25/2011 14:45'!
name
	
	^ name! !

!OrcaNode methodsFor: 'accessing' stamp: 'ag 1/25/2011 14:45'!
name: aString
	
	name := aString! !

!OrcaNode methodsFor: 'accessing' stamp: 'lw 3/28/2011 17:40'!
originalCode
	^ originalCode! !

!OrcaNode methodsFor: 'accessing' stamp: 'ag 1/19/2011 19:17'!
originalCode: anObject
	"This is only set on 'block'-nodes. Never mind. :-)"
	
	originalCode := anObject! !

!OrcaNode methodsFor: 'printing' stamp: 'ag 11/19/2010 00:39'!
printString
	
	| result first |
	first := true.
	result := name, ' ('.
	self do: [ :each |
		first ifTrue: [first := false] ifFalse: [result := result, ', '].
		result := result,  each printString].
	^ result, ')'! !

ProtoObject subclass: #Js
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Misc'!

!Js commentStamp: 'ag 4/5/2011 17:21' prior: 0!
This is a place, where programmers can define global shortcuts to access or execute common functions with long names.
The superclass of this class is set to nil to remove unnecessary methods.
Code inside this class should be short and is pure client-side code.!

!Js class methodsFor: 'shortcuts' stamp: 'ag 4/5/2011 17:47'!
@ aString
	"This shortcuts calling the jQuery-$-function with one string as parameter.
	This is client-side code."
	
	^ (self Global perform: '$') value: aString! !

!Js class methodsFor: 'stubs' stamp: 'hk 4/5/2011 13:02'!
Document

	self clientSideResposibility.! !

!Js class methodsFor: 'stubs' stamp: 'hk 4/5/2011 13:03'!
Global

	self clientSideResposibility.! !

!Js class methodsFor: 'shortcuts' stamp: 'ag 4/28/2011 13:15'!
addToBody: htmlCode
	
	Js body innerHTML: Js body innerHTML, htmlCode.! !

!Js class methodsFor: 'shortcuts' stamp: 'ag 4/28/2011 13:14'!
body
	
	^ (Js Document getElementsByTagName value: 'body') first! !

!Js class methodsFor: 'stubs' stamp: 'ag 4/12/2011 14:06'!
clearIntervals
	"This method clears all intervals, that has been set through code like 'Js Global setInterval value: aBlock value: aTime'.
	Only valid on the client-side. Such javascript-intervals sometimes don't disappear automatically when reloading a tab in the browser."
	
	self clientSideResposibility! !

!Js class methodsFor: 'shortcuts' stamp: 'ag 4/5/2011 17:48'!
jit
	
	^ self Global perform: '$jit'! !

ProtoObject subclass: #OrcaFutureResult
	instanceVariableNames: 'computationCompleted result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-RemoteObjects'!

!OrcaFutureResult class methodsFor: 'instance creation' stamp: 'lw 4/5/2011 12:25'!
newFrom: aBlock

	^ self new
		xxxStartComputation: aBlock;
		yourself! !

!OrcaFutureResult methodsFor: 'error handling' stamp: 'lw 4/5/2011 12:19'!
doesNotUnderstand: aMessage

	computationCompleted wait.
	self become: result.
	^ aMessage sendTo: result.! !

!OrcaFutureResult methodsFor: 'error handling' stamp: 'lw 4/5/2011 12:24'!
initialize

	computationCompleted := Semaphore new.! !

!OrcaFutureResult methodsFor: 'error handling' stamp: 'lw 4/5/2011 12:51'!
xxxStartComputation: aBlock

	[ result := aBlock value.
	computationCompleted signal ] fork! !

ProtoObject subclass: #OrcaRemoteObject
	instanceVariableNames: 'id session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-RemoteObjects'!

!OrcaRemoteObject commentStamp: 'lw 5/5/2011 10:47' prior: 0!
An OrcaRemoteObject is a RemoteObject, which rests on the Client represented by session. On this Client, it is reachable by id.

OrcaRemoteObject doesn't inherit from ProtoObject, because Monticello is not capable of organizing Classes without the ProtoObject class interface.
To have trigered #doesNotUnderstand: as often as possible, we don't inherit from Object.

Some Problems with the CogVM:
ifNil:, ifTrue:, ifFalse: is optimized, therefore nil, true and false are not remoteObjects but should be transmitted like every Immutable.

class is not send but the Object's class returned...
There is no workaround for this, which also works with the squeakVM
!

!OrcaRemoteObject class methodsFor: 'instance creation' stamp: 'LT 5/3/2011 17:30'!
newWithId: aNumber andSession: aSession

	^ self basicNew 
		xxxId: aNumber; 
		xxxSession: aSession! !

!OrcaRemoteObject methodsFor: 'accessing' stamp: 'lw 5/5/2011 10:40'!
asRemoteAnswerTo: aSession

	session = aSession 
		ifTrue:  [ ^ 'return reachableObjectNamed(', self xxxId asString, ');' ]
		ifFalse: [ "This implementation is copied from Object. Because OrcaRemoteObject does not inherit from Object, super is not possible."
			^ 'return ', (Dictionary newFrom: { 'remoteId' -> (aSession addReachableObjectFor: self) }) asJson]! !

!OrcaRemoteObject methodsFor: 'accessing' stamp: 'lw 5/5/2011 10:19'!
doesNotUnderstand: aMessage
	
	session channel isAlive ifTrue: [ ^ session passMessage: aMessage to: self ]! !

!OrcaRemoteObject methodsFor: 'testing' stamp: 'lw 4/14/2011 12:04'!
isRemote

	^ true! !

!OrcaRemoteObject methodsFor: 'accessing' stamp: 'lw 4/14/2011 12:58'!
xxxId
	^ id! !

!OrcaRemoteObject methodsFor: 'accessing' stamp: 'lw 4/14/2011 12:58'!
xxxId: anObject
	id := anObject! !

!OrcaRemoteObject methodsFor: 'accessing' stamp: 'LT 5/3/2011 12:49'!
xxxSession: anObject
	session := anObject! !

!ProtoObject methodsFor: '*Orca-Core' stamp: 'ag 4/13/2011 17:36'!
definitionLine
	
	^ (OrcaSqueakAstToJsAst escapeGlobal: self class name),
		'._newInstance()' ! !

!ProtoObject methodsFor: '*Orca-Core' stamp: 'ag 4/6/2011 15:52'!
fillInstVars: codeLines guardedBy: alreadySerialized
	
	self class allInstVarNames withIndexDo: [ :varName :i | | varValue |
		varValue := self instVarAt: i. "This really works :D"
		varValue serializeToJavaScript: codeLines guardedBy: alreadySerialized.
		codeLines addLast: 
			(alreadySerialized at: self), '.', (OrcaSqueakAstToJsAst jsInstVarNameFor: varName), 
			'=', (alreadySerialized at: varValue) ].! !

!ProtoObject methodsFor: '*Orca-Core' stamp: 'LT 4/19/2011 10:46'!
serializeToJavaScript
	
	| code objectIds |
	code := OrderedCollection new. "codeLines"
	objectIds := Dictionary new. "alreadySerialized"
	self serializeToJavaScript: code guardedBy: objectIds.
	code addLast: 'return ', (objectIds at: self).
	^ (code separatedBy: ';'), ';' "function() { ... } is wrapped around this code on the server"! !

!ProtoObject methodsFor: '*Orca-Core' stamp: 'ag 4/14/2011 11:31'!
serializeToJavaScript: codeLines guardedBy: alreadySerialized
	
	(alreadySerialized includesKey: self) ifFalse: [
		alreadySerialized at: self put: 'obj', (alreadySerialized size + 1) asString.
		codeLines addFirst:  'var ', (alreadySerialized at: self), '=', (self definitionLine).
		self fillInstVars: codeLines guardedBy: alreadySerialized ].! !

!HTTPConnection methodsFor: '*Orca-Core' stamp: 'ag 4/18/2011 14:16'!
interact
	"We refine this to bring up the debugger when an error occurs inside the Swazoo."



	"longer description is below method"
	| interactionBlock |
	interactionBlock := 
		[[	[ [true] whileTrue: 
				[self getAndDispatchMessages.
				Processor yield] 
			]
			on: Error
			do: [:ex | 
				
				((Smalltalk at: #OrcaDebugSwazooInternalErrors ifAbsent: [nil]) == true)
					ifTrue: [ (ex class = SwazooStreamNoDataError) | (ex class = SpSocketError) ifFalse: [ self halt ] ]
				] ]
		ifCurtailed: [ 
			(Delay forMilliseconds: 50) wait. "to finish sending, if any"
			self isOpen ifTrue: [self close]] ].
	self server isMultiThreading 
		ifTrue: [self loop: (interactionBlock forkAt: Processor userBackgroundPriority)]
		ifFalse: [interactionBlock value].
	^self

"I represent a specifc connection with an HTTP client (a browser, probably) over which will come an HTTP request.  Here, I fork the handling of the request so that the current thread (which is most likely the HTTP server main loop) can carry on with the next request.  This means that more than one request may being handled in the image at a time, and that means that the application developer must worry about thread safety - e.g the problem of a given business object being updated by more than one HTTP request thread.
For a GemStone implementation of Swazoo, one may want only one request is handled at a time, multi-threadedness being handled by having multiple gems.  This is a nice option because the application developer does not have to worry about thread safety in this case - GemStone handles the hard stuff.
*And* the thing called a loop that was in this method was no such thing.  In all circumstances, >>getAndDispatchMessages handles exactly one requst and then closes the socket!! (very non-HTTP1.1).  Anyway, for now I'm just going to make that explicit.  This needs to be re-visited to support HTTP 1.1."! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'ag 2/3/2011 12:02'!
classesWithEmptyCategories
	
	^ (self ourClasses select: [ :class |
			class organization categories anySatisfy: [ :category | class organization isEmptyCategoryNamed: category ] ]),
		self ourEmptyClassCategories! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'LT 4/13/2011 18:57:36.014'!
classesWithUnclassifiedMethods
	
	^ self ourClasses select: [ :class |
		(class name endsWith: 'Test') not & (class name endsWith: 'Tests') not &
		(class organization categories includes: 'as yet unclassified' asSymbol) ]! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'ag 2/3/2011 12:02'!
countCategoriesOf: aClass
	
	^ (aClass organization categories
		reject: [ :cat | cat == 'no messages' asSymbol ])
			size! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'ag 4/6/2011 17:50'!
debugSwazooErrors: aBoolean
	
	^ Smalltalk
		at: #OrcaDebugSwazooInternalErrors
		put: aBoolean! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'ag 3/28/2011 18:01'!
grep: aString
	"Search the source-code of our packages and return a collection of all CompiledMethods, that contain aString in their source-code."
	
	| methods |
	methods := OrderedCollection new.
	self ourClasses do: [ :class |
		class methodDict values do: [ :compiledMethod |
			(compiledMethod value getSource asString includesSubString: aString)
					ifTrue: [ methods add: compiledMethod ]]].
	^ methods! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'LT 4/14/2011 14:47:00.861'!
ourClasses
	
	^ self ourPackages 
		inject: OrderedCollection new 
		into: [ :all :each | all union: each classesAndMetaClasses].! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'LT 4/14/2011 14:56:07.417'!
ourEmptyClassCategories
	
	^ self ourPackages 
		collect: [ :each | each systemCategories] 
		thenSelect: [ :cat | SystemOrganization isEmptyCategoryNamed: cat]! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'LT 4/14/2011 14:42:18.905'!
ourPackageNames
	
	^ #('Orca-Core' 'Orca-Tools' 'Orca-Tests' 'Orca-Examples' 'Orca-Widgets' )! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'LT 4/14/2011 14:42:13.083'!
ourPackages
	
	^ self ourPackageNames 
		collect: [ :each |
			PackageOrganizer default
			packageNamed: each asSymbol
			ifAbsent: [ ]]
		thenSelect: [ :each | each notNil ]! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'ag 2/3/2011 12:03'!
removeEmptyCategories
	"Removes method-categories and class-categories. Return, how many categories have been removed."
	
	| removedCategories |
	removedCategories := 0.
	self ourClasses do: [ :class | | categoriesBefore |
		categoriesBefore := self countCategoriesOf: class.
		class organization removeEmptyCategories.
		removedCategories := removedCategories + categoriesBefore - (self countCategoriesOf: class) ].
	self ourEmptyClassCategories do: [ :cat |
		SystemOrganization removeCategory: cat.
		removedCategories := removedCategories + 1].
	^ removedCategories! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'ag 4/6/2011 14:07'!
renamePackagesNamed: oldName to: newName
	
	PackageOrganizer default packages do: [ :pk | | classes |
		pk classesAndMetaClasses do: [ :cl |
			cl organization categories do: [ :cat |
				(cat = oldName asSymbol)
					ifTrue: [ cl organization
						renameCategory: oldName
						toBe: newName ] ] ]].! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'ag 4/6/2011 14:07'!
renamePrefixFrom: oldPrefix to: newPrefix

	PackageOrganizer default packages do: [ :pk | | classes |
		pk classes do: [ :cl | | n |
			n := cl name.
			(n beginsWith: oldPrefix) ifTrue: [
				(RenameClassRefactoring
					rename: cl
					to: newPrefix, (n copyFrom: (oldPrefix size + 1) to: n size))
						execute ] ]].! !

!Utilities class methodsFor: '*Orca-Core' stamp: 'lw 4/6/2011 18:08'!
withinEachMethodOf: aClass replace: aRegularExpressionString with: aString ignoreCase: aBool

	| regexRefactoring |
	regexRefactoring := ORRegexRefactoring new
		replace: aRegularExpressionString with: aString ignoreCase: aBool;
		yourself.

	aClass methodDict do: [ :aCompiledMethod | 
			aClass compile: (regexRefactoring execute: aCompiledMethod getSource) ]! !

!String methodsFor: '*Orca-Core' stamp: 'lw 5/5/2011 10:25'!
asRemoteAnswerTo: aSession

	^ self serializeToJavaScript! !

!String methodsFor: '*Orca-Core' stamp: 'ag 4/6/2011 15:52'!
definitionLine
	
	^ OrcaSqueakAstToJsAst stringFunction asString, '(', (OrcaAbstractPrinter asJSString: self), ')'! !

!String methodsFor: '*Orca-Core' stamp: 'sw 3/30/2011 14:24'!
jsEval

	^ Js Global eval value: self! !

!Error methodsFor: '*Orca-Core' stamp: 'lw 5/5/2011 10:24'!
asRemoteAnswerTo: aSession

	^ 'return ', (Dictionary newFrom: { 'error' -> self messageText }) asJson! !

!MIMEDocument class methodsFor: '*Orca-Core' stamp: 'hk 1/26/2011 10:40'!
extendedMIMEdatabase
	| d |
	(d := self defaultMIMEdatabase)
		 at: 'js' put: 'text/javascript';
		 at: 'hqx' put: 'application/mac-binhex40';
		 at: 'cpt' put: 'application/mac-compactpro';
		 at: 'pdf' put: 'application/pdf';
		 at: 'ps' put: 'application/postscript';
		 at: 'ai' put: 'application/postscript';
		 at: 'eps' put: 'application/postscript';
		 at: 'rtf' put: 'text/rtf';
		 at: 'ics' put: 'text/calendar';
		 at: 'bin' put: 'application/octet-stream';
		 at: 'dms' put: 'application/octet-stream';
		 at: 'lha' put: 'application/octet-stream';
		 at: 'lzh' put: 'application/octet-stream';
		 at: 'exe' put: 'application/octet-stream';
		 at: 'class' put: 'application/octet-stream';
		 at: 'zip' put: 'application/zip';
		 at: 'gtar' put: 'application/x-gtar';
		 at: 'swf' put: 'application/x-shockwave-flash';
		 at: 'sit' put: 'application/x-stuffit';
		 at: 'tar' put: 'application/x-tar';
		 at: 'au' put: 'audio/basic';
		 at: 'snd' put: 'audio/basic';
		 at: 'mid' put: 'audio/midi';
		 at: 'midi' put: 'audio/midi';
		 at: 'mpga' put: 'audio/mpeg';
		 at: 'mp2' put: 'audio/mpeg';
		 at: 'mp3' put: 'audio/mpeg';
		 at: 'aiff' put: 'audio/x-aiff';
		 at: 'aif' put: 'audio/x-aiff';
		 at: 'aifc' put: 'audio/x-aiff';
		 at: 'rm' put: 'audio/x-pn-realaudio';
		 at: 'ram' put: 'audio/x-pn-realaudio';
		 at: 'rpm' put: 'audio/x-pn-realaudio-plugin';
		 at: 'ra' put: 'audio/x-realaudio';
		 at: 'wav' put: 'audio/x-wav';
		 at: 'css' put: 'text/css';
		 at: 'mpeg' put: 'video/mpeg';
		 at: 'mpg' put: 'video/mpeg';
		 at: 'mpe' put: 'video/mpeg';
		 at: 'qt' put: 'video/quicktime';
		 at: 'mov' put: 'video/quicktime';
		 at: 'avi' put: 'video/x-msvideo';
		 at: 'movie' put: 'video/x-sgi-movie'.
	^ d! !

SwazooResource subclass: #OrcaDispatcher
	instanceVariableNames: 'componentMap sessions renderer handlers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

!OrcaDispatcher commentStamp: 'ag 4/27/2011 09:46' prior: 0!
Setters, that have to be invoked during construction (before calling initialize):
 - #renderer:
 - #uriPattern:
!

!OrcaDispatcher class methodsFor: 'http utils' stamp: 'ag 4/20/2011 15:46'!
answer: content withStatus: aNumber
	
	^ HTTPResponse ok
		statusCode: aNumber;
		entity: content! !

!OrcaDispatcher class methodsFor: 'http utils' stamp: 'ag 4/19/2011 15:09'!
answerCode: code
	
	^ self
		answerContent: code
		withType: 'text/javascript'! !

!OrcaDispatcher class methodsFor: 'http utils' stamp: 'ag 4/20/2011 15:52'!
answerContent: data withType: aString
	
	| response |
	response := HTTPResponse ok.
	^ response
		contentType: aString;
		entity: data;
		yourself! !

!OrcaDispatcher class methodsFor: 'http utils' stamp: 'ag 4/19/2011 15:09'!
answerHTML: htmlCode
	
	^ self
		answerContent: htmlCode
		withType: 'text/html'! !

!OrcaDispatcher class methodsFor: 'http utils' stamp: 'ag 4/20/2011 15:47'!
answerStatus: aNumber
	
	^ self answer: '' withStatus: aNumber! !

!OrcaDispatcher class methodsFor: 'http utils' stamp: 'hk 4/27/2011 14:14'!
answerZippedCode: zippedData

	^ HTTPResponse ok
		addHeaderName: 'Content-Encoding' value: 'gzip';
		contentType: 'text/javascript';
		entity: zippedData;
		yourself.! !

!OrcaDispatcher class methodsFor: 'instance creation' stamp: 'ag 4/28/2011 14:07'!
createInstance
	"Create a default instance of the dispatcher. This can be overwritten without touching installOn: etc."
	
	^ (self newWithRenderer: (self defaultRenderer))
		uriPattern: self defaultRootPath;
		yourself! !

!OrcaDispatcher class methodsFor: 'configuration' stamp: 'ag 4/19/2011 11:47'!
defaultPort

	^ 8887! !

!OrcaDispatcher class methodsFor: 'configuration' stamp: 'ag 4/20/2011 16:09'!
defaultRenderer
	
	^ OrcaDefaultHTMLRenderer! !

!OrcaDispatcher class methodsFor: 'configuration' stamp: 'ag 4/28/2011 14:07'!
defaultRootPath
	
	^ '/'! !

!OrcaDispatcher class methodsFor: 'configuration' stamp: 'ag 4/28/2011 14:07'!
defaultSiteName
	
	^ self name! !

!OrcaDispatcher class methodsFor: 'setup' stamp: 'ag 4/27/2011 09:47'!
hardCleanup
	"This cleans up all http-server-resources in the image. Caution!! Not only Orca-related resources are cleand up. This is usefull, if the image becomes slow and is filled up with connection-objects while developing."
	
	SwazooServer initSingleton.
	SwazooServer allInstancesDo: [ :swa | 
		swa stop; initSites].
	HTTPServer allInstancesDo: [ :http |
		http stop; initialize].
	HTTPConnection allInstancesDo: [ :conn |
		conn close].
	SwazooServer singleton start.
	Smalltalk garbageCollect.! !

!OrcaDispatcher class methodsFor: 'setup' stamp: 'ag 4/27/2011 09:49'!
hardRestart
	
	self hardCleanup.
	self start.! !

!OrcaDispatcher class methodsFor: 'setup' stamp: 'ag 1/13/2011 20:12'!
install
	
	^ self installOn: self defaultPort! !

!OrcaDispatcher class methodsFor: 'setup' stamp: 'ag 4/28/2011 14:08'!
installOn: aPort
	
	^ (SwazooSite
		newNamed: self defaultSiteName)
			host: '*'
			ip: '*'
			port: aPort;
			addResource: self createInstance;
			yourself! !

!OrcaDispatcher class methodsFor: 'setup' stamp: 'ag 1/13/2011 20:12'!
isInstalled
	
	^ self site notNil! !

!OrcaDispatcher class methodsFor: 'http utils' stamp: 'ag 4/21/2011 13:40'!
javascriptUrlFor: aRequest
	
	^ aRequest requestLine requestURI asString! !

!OrcaDispatcher class methodsFor: 'instance creation' stamp: 'ag 4/19/2011 13:59'!
newWithRenderer: anHTMLRenderer
	
	^ self basicNew
		renderer: anHTMLRenderer;
		initialize;
		yourself! !

!OrcaDispatcher class methodsFor: 'http utils' stamp: 'ag 4/21/2011 13:40'!
numberArgument: aString in: aRequest
	
	| matches |
	matches := (aString, '=([0-9]*)&?') asRegex submatchesIn: (self javascriptUrlFor: aRequest).
	^ matches 
		ifNotEmpty: [ matches first first asInteger ]
		ifEmpty: [ nil ]! !

!OrcaDispatcher class methodsFor: 'http utils' stamp: 'ag 4/20/2011 15:34'!
pathPart: aNumber of: aRequest
	
	| identifierTokens |
	identifierTokens := self pathSegments: aRequest.
	^ identifierTokens size >= aNumber
		ifTrue: [ identifierTokens at: aNumber ]! !

!OrcaDispatcher class methodsFor: 'http utils' stamp: 'ag 4/20/2011 15:34'!
pathSegments: aRequest
	
	^ (self urlOf: aRequest) findTokens: '/'! !

!OrcaDispatcher class methodsFor: 'http utils' stamp: 'ag 4/21/2011 11:14'!
redirectTo: aPath
	
	^ HTTPResponse new
		statusCode: 302;
		entity: '';
		location: aPath;
		yourself! !

!OrcaDispatcher class methodsFor: 'setup' stamp: 'ag 4/27/2011 09:49'!
restart
	
	! !

!OrcaDispatcher class methodsFor: 'setup' stamp: 'ag 4/28/2011 14:08'!
site
	
	^ SwazooServer singleton siteNamed: self defaultSiteName! !

!OrcaDispatcher class methodsFor: 'setup' stamp: 'ag 1/13/2011 20:12'!
start
	
	self isInstalled ifFalse: [ self install ].
	self site start.! !

!OrcaDispatcher class methodsFor: 'setup' stamp: 'ag 1/13/2011 20:13'!
uninstall
	
	self isInstalled ifTrue: [
		self removeAllClients.
		self resetContent.
		SwazooServer singleton removeSite: self site ].! !

!OrcaDispatcher class methodsFor: 'http utils' stamp: 'ag 4/20/2011 14:52'!
urlOf: aRequest
	
	^ aRequest requestLine requestURI identifier! !

!OrcaDispatcher methodsFor: 'dispatching' stamp: 'ag 4/26/2011 16:57'!
answerTo: aRequest
	
	(self pathHasTrailingSlash: aRequest)
		ifTrue: [ 
			(self isNewSessionRequest: aRequest)
				ifTrue: [ | component |
					component := self componentFor: aRequest.
					component
						ifNotNil: [ ^ self createNewSessionFrom: component for: aRequest ] ]
				ifFalse: [ | handler |
					handler := self validHandlerFor: aRequest.
					handler
						ifNotNil: [ ^ self invokeHandler: handler for: aRequest ] ] ]
		ifFalse: [ ^ self redirectToComponentPath: aRequest ].
	
	"Answer 404, if an illegal component or handler is addressed"
	^ self class answerStatus: 404! !

!OrcaDispatcher methodsFor: 'initialization' stamp: 'ag 4/28/2011 14:06'!
collectComponents
	"Implement the convenience-functionality, that subclasses of OrcaAbstractComponent are added automatically as components."
	
	Object allSubclassesDo: [ :each | 
		((each respondsTo: #isOrcaRootComponent) and: [ each isOrcaRootComponent ]) ifTrue: [
			self setComponent: each onPath: each uniquePath ] ].! !

!OrcaDispatcher methodsFor: 'initialization' stamp: 'ag 4/28/2011 14:06'!
collectHandlers
	"One instance of each handler is collected when creating a dispatcher-instance. Has to be re-initialized, if a handler is added/removed.
	This makes the OrcaDispatcher more dynamic, while maintaining the practical static configuration using the #defaultPathSuffix."
	
	^ Dictionary newFrom: 
		(OrcaRequestHandler subclasses collect: [ :each | | instance |
			instance := each new.
			instance pathSuffix -> instance ])! !

!OrcaDispatcher methodsFor: 'request' stamp: 'ag 4/20/2011 14:55'!
componentFor: aRequest
	
	^ self componentMap
		at: (self componentPathOf: aRequest)
		ifAbsent: [ nil ]! !

!OrcaDispatcher methodsFor: 'accessing' stamp: 'ag 4/18/2011 15:52'!
componentMap
	^ componentMap! !

!OrcaDispatcher methodsFor: 'accessing' stamp: 'ag 4/18/2011 15:52'!
componentMap: anObject
	componentMap := anObject! !

!OrcaDispatcher methodsFor: 'request' stamp: 'ag 4/20/2011 14:53'!
componentPathOf: aRequest
	
	^ self class pathPart: 1 of: aRequest! !

!OrcaDispatcher methodsFor: 'dispatching' stamp: 'ag 4/21/2011 16:34'!
createNewSessionFrom: aComponent for: aRequest
	
	| newSession |
	newSession := aComponent orcaSessionClass
						newWithId: self generateSessionId
						dispatcher: self
						rootComponent: aComponent.
	self sessions at: newSession id put: newSession.
	^ self class answerHTML: 
		(self renderer
			forClient: newSession
			request: aRequest)
				initialHTML! !

!OrcaDispatcher methodsFor: 'dispatching' stamp: 'ag 4/20/2011 17:01'!
generateSessionId
	
	| id |
	id := 0.
	[ self sessions anySatisfy: [ :each | each id = id] ]
		whileTrue:
			[ id := 16r100000000 atRandom. "0 .. 2 ^ 32" ].
	^ id! !

!OrcaDispatcher methodsFor: 'request' stamp: 'ag 4/21/2011 12:15'!
handlerFor: aRequest
	
	^ self handlers
		at: (self handlerPathOf: aRequest)
		ifAbsent: [ nil ]! !

!OrcaDispatcher methodsFor: 'configuration' stamp: 'ag 4/21/2011 13:12'!
handlerForResource: aSymbol
	"Try to find a handler-object for a resource identified by aSymbol. Return nil, if no such handler is configured."
	
	^ self handlers values
		detect: [ :each | each isHandlerFor: aSymbol ]
		ifNone: [ nil ]! !

!OrcaDispatcher methodsFor: 'request' stamp: 'ag 4/20/2011 14:53'!
handlerPathOf: aRequest
	
	^ self class pathPart: 2 of: aRequest! !

!OrcaDispatcher methodsFor: 'accessing' stamp: 'ag 4/19/2011 15:02'!
handlers
	^ handlers! !

!OrcaDispatcher methodsFor: 'accessing' stamp: 'ag 4/19/2011 15:02'!
handlers: anObject
	handlers := anObject! !

!OrcaDispatcher methodsFor: 'initialization' stamp: 'ag 4/28/2011 14:06'!
initialize
	
	super initialize.
	self componentMap: Dictionary new.
	self sessions: Dictionary new.
	self handlers: self collectHandlers.
	self collectComponents.! !

!OrcaDispatcher methodsFor: 'dispatching' stamp: 'ag 4/21/2011 17:29'!
invokeHandler: handler for: aRequest
	"If we don't have a session, create a 'session-mock', that contains only the component and it's requiredClasses."
	
	| session |
	session := self sessionFor: aRequest.
	session ifNil: [ | component |
		component := self componentFor: aRequest.
		session := component orcaEmptySessionClass
			newForComponent: component
			dispatcher: self ].
	^ handler handleRequest: aRequest forSession: session! !

!OrcaDispatcher methodsFor: 'dispatching' stamp: 'ag 4/20/2011 16:56'!
isNewSessionRequest: aRequest
	"A new session will be created, if no handler-path and no session-id is included in the request."
	
	^ (self sessionIdOf: aRequest) isNil and: [ (self handlerPathOf: aRequest) isNil ]! !

!OrcaDispatcher methodsFor: 'configuration' stamp: 'ag 4/21/2011 15:40'!
pathForResource: aSymbol
	"Try to find the path for a resource identified by aSymbol. Return nil, if no such path is configured."
	
	^ (self handlerForResource: aSymbol)
		ifNotNil: [ :handler | handler pathSuffix ]! !

!OrcaDispatcher methodsFor: 'dispatching' stamp: 'ag 4/21/2011 11:02'!
pathHasTrailingSlash: aRequest
	"To enable correct handling of initial javascript-resources, the initial request is asserted to have a trailing slash."
	
	| url |
	url := self class urlOf: aRequest.
	^ ((url findTokens: '/') size = 1 ==> [ url endsWith: '/' ])! !

!OrcaDispatcher methodsFor: 'configuration' stamp: 'ag 4/21/2011 14:49'!
pathSuffixpathForResource: aSymbol
	"Try to find the path for a resource identified by aSymbol. Return nil, if no such path is configured."
	
	^ (self handlerForResource: aSymbol)
		ifNotNil: [ :handler | handler pathSuffix ]! !

!OrcaDispatcher methodsFor: 'dispatching' stamp: 'ag 4/21/2011 11:14'!
redirectToComponentPath: aRequest
	"Forware to the pure 'component-path' of the request. Strip any segments but the first one and ensure a slash at the end."
	
	^ self class redirectTo: '/', (self componentPathOf: aRequest), '/'! !

!OrcaDispatcher methodsFor: 'accessing' stamp: 'ag 4/19/2011 11:44'!
renderer
	^ renderer! !

!OrcaDispatcher methodsFor: 'accessing' stamp: 'ag 4/19/2011 11:44'!
renderer: anObject
	renderer := anObject! !

!OrcaDispatcher methodsFor: 'dispatching' stamp: 'ag 4/27/2011 17:19'!
sessionDied: aSession
	
	self sessions
		removeKey: aSession id
		ifAbsent: [ "This should not happen, each session can only be removed once." ].! !

!OrcaDispatcher methodsFor: 'request' stamp: 'ag 4/20/2011 16:56'!
sessionFor: aRequest
	
	^ self sessions
		at: (self sessionIdOf: aRequest)
		ifAbsent: [ nil ]! !

!OrcaDispatcher methodsFor: 'request' stamp: 'ag 4/20/2011 16:56'!
sessionIdOf: aRequest
	
	^ self class numberArgument: 'id' in: aRequest! !

!OrcaDispatcher methodsFor: 'accessing' stamp: 'ag 4/20/2011 16:55'!
sessions
	^ sessions! !

!OrcaDispatcher methodsFor: 'accessing' stamp: 'ag 4/20/2011 16:55'!
sessions: anObject
	sessions := anObject! !

!OrcaDispatcher methodsFor: 'configuration' stamp: 'ag 4/21/2011 13:11'!
setComponent: anObject onPath: anURLPath
	
	^ self componentMap at: anURLPath put: anObject! !

!OrcaDispatcher methodsFor: 'dispatching' stamp: 'ag 4/21/2011 11:01'!
validHandlerFor: aRequest
	"Check, if aRequest addresses a handler. If so, check whether the request fullfills the handler's requirement of having an established session."
	
	| handler |
	handler := self handlerFor: aRequest.
	^ (handler notNil 
		and: [ handler needsSession ==> [ (self sessionFor: aRequest) notNil ] ])
			ifTrue: [ handler ]! !

!Character methodsFor: '*Orca-Core' stamp: 'lw 5/5/2011 10:24'!
asRemoteAnswerTo: aSession

	^ self serializeToJavaScript! !

!Character methodsFor: '*Orca-Core' stamp: 'ag 4/6/2011 15:52'!
definitionLine
	
	^ OrcaSqueakAstToJsAst charFunction asString, '(', (OrcaAbstractPrinter asJSString: self asString), ')'! !

!UndefinedObject methodsFor: '*Orca-Core' stamp: 'LT 5/2/2011 13:34'!
asRemoteAnswerTo: aClient

	^ self serializeToJavaScript! !

!UndefinedObject methodsFor: '*Orca-Core' stamp: 'ag 4/12/2011 12:00'!
definitionLine
	
	^ OrcaSqueakAstToJsAst jsKeywordFor: self asString! !

!Message methodsFor: '*Orca-Core' stamp: 'lw 5/5/2011 10:24'!
asRemoteAnswerTo: aSession

	| arguments messageJson |
	arguments := self arguments collect: [ :each | each asRemoteAnswerTo: aSession ].
	messageJson := 'return ', 
		(Dictionary newFrom: {'selector' -> self selector serializeToJavaScript.
	'args' -> arguments asArray}) asJson.
	^ messageJson ! !

!SHTextStylerST80 class methodsFor: '*Orca-Core' stamp: 'hk 3/22/2011 14:55'!
disableJsHighlighting

	self chooseDefaultStyleTable.! !

!SHTextStylerST80 class methodsFor: '*Orca-Core' stamp: 'hk 3/22/2011 12:03'!
enableJsHighlighting

	self styleTable: self jsStyleTable.! !

!SHTextStylerST80 class methodsFor: '*Orca-Core' stamp: 'hk 3/22/2011 14:54'!
jsStyleTable
	"color can be a valid argument to Color class>>colorFrom: , or nil to
	use the editor text color.
	Multiple emphases can be specified using an array e.g. #(bold italic).
	If emphasis is not specified, #normal will be used.
	if pixel height is not specified , then the editor font size will be used.
	"
								
^#(
	"(symbol	color		[emphasisSymbolOrArray		[textStyleName [pixelHeight]]])"		
	(default 								black 		)
	(invalid 								red 			)
	(excessCode 								red 			)
	(comment 								(cyan darker) 		)
	(unfinishedComment 					(red muchDarker) 	italic)
	(#'$'									(red muchDarker) 	)
	(character								(red muchDarker) 	)
	(integer									(red muchDarker) 	)
	(number								(red muchDarker) 	)	
	(#-										(red muchDarker) 	)
	(symbol									(blue muchDarker)			)	
	(stringSymbol							(blue muchDarker)			)	
	(literalArray							(blue muchDarker)			)
	(string									(magenta muchDarker)	normal			)
	(unfinishedString						red			normal			)
	(assignment								nil			bold			)
	(ansiAssignment 						nil			bold)
	(literal									nil			italic)
	(keyword								(blue muchDarker)			)
	(binary 								(blue muchDarker)			)	
	(unary									(blue muchDarker)			)
	(incompleteKeyword						(gray muchDarker)			underlined)
	(incompleteBinary 						(gray muchDarker)			underlined)	
	(incompleteUnary						(gray muchDarker)			underlined)
	(undefinedKeyword						red			)
	(undefinedBinary 						red			)	
	(undefinedUnary						red			)													
	(patternKeyword 						nil			bold)
	(patternBinary 							nil			bold)
	(patternUnary							nil			bold)	
	(#self 									(red muchDarker)	)
	(#super									(red muchDarker)	) 
	(#true 									(red muchDarker)	)
	(#false 									(red muchDarker)	)
	(#nil 									(red muchDarker)	)
	(#thisContext 							(red muchDarker)	)
	(#return								(red muchDarker)	)
	(patternArg 							(blue muchDarker)			)	
	(methodArg 								(blue muchDarker)			)
	(blockPatternArg 						(blue muchDarker)			)
	(blockArg 								(blue muchDarker)			)
	(argument 								(blue muchDarker)			)
	(blockArgColon							black		) 
	(leftParenthesis							black		) 
	(rightParenthesis						black		) 
	(leftParenthesis1						(green muchDarker)		) 
	(rightParenthesis1						(green muchDarker)		) 
	(leftParenthesis2						(magenta muchDarker)		) 
	(rightParenthesis2						(magenta muchDarker)		) 
	(leftParenthesis3						(red muchDarker)		) 
	(rightParenthesis3						(red muchDarker)		) 
	(leftParenthesis4						(green darker)		) 
	(rightParenthesis4						(green darker)		) 
	(leftParenthesis5						(orange darker)		) 
	(rightParenthesis5						(orange darker)		) 
	(leftParenthesis6						(magenta darker)		) 
	(rightParenthesis6						(magenta darker)		) 
	(leftParenthesis7						blue		) 
	(rightParenthesis7						blue		) 
	(blockStart 								black		) 
	(blockEnd 								black		) 
	(blockStart1								(green muchDarker)			) 
	(blockEnd1								(green muchDarker)			) 
	(blockStart2								(magenta muchDarker)		) 
	(blockEnd2								(magenta muchDarker)		) 
	(blockStart3								(red muchDarker)		) 
	(blockEnd3								(red muchDarker)		) 
	(blockStart4								(green darker)		) 
	(blockEnd4								(green darker)		) 
	(blockStart5								(orange darker)		) 
	(blockEnd5								(orange darker)		) 
	(blockStart6								(magenta darker)		) 
	(blockEnd6								(magenta darker)		) 
	(blockStart7								blue		) 
	(blockEnd7								blue		) 																																																		
	(arrayStart								black		) 
	(arrayEnd								black		) 
	(arrayStart1							black		) 
	(arrayEnd1								black		) 
	(leftBrace 								blue		) 
	(rightBrace 								blue		) 
	(cascadeSeparator 						black		) 
	(statementSeparator 						black		) 
	(externalCallType 						black		) 
	(externalCallTypePointerIndicator 		black		) 
	(primitiveOrExternalCallStart 			black		) 
	(primitiveOrExternalCallEnd				black		)
	(methodTempBar							gray		) 
	(blockTempBar 							gray		)
	(blockArgsBar							gray		)
	(primitive								(green muchDarker)		bold)
	(pragmaKeyword						(green muchDarker)		bold)
	(pragmaUnary							(green muchDarker)		bold)
	(pragmaBinary							(green muchDarker)		bold)									
	(externalFunctionCallingConvention		(green muchDarker)		bold) 
	(module									(green muchDarker)		bold)
	(blockTempVar 							gray		)
	(blockPatternTempVar					gray		)
	(instVar 								black		)
	(workspaceVar							black		)
	(undefinedIdentifier						red			)
	(incompleteIdentifier					(gray darker) 	(italic underlined))
	(tempVar								(gray darker)	)
	(patternTempVar						(gray darker)	)
	(poolConstant							(gray muchDarker)	)
	(classVar								(gray muchDarker)	)
	(globalVar								black		) )! !

!Array methodsFor: '*Orca-Core' stamp: 'ag 4/6/2011 15:52'!
asObject 
	
	^ OrcaSlotObject newFrom: self! !

!Array methodsFor: '*Orca-Core' stamp: 'ag 4/14/2011 11:33'!
definitionLine
	
	^ OrcaSqueakAstToJsAst arrayFunction, '([])'! !

!Array methodsFor: '*Orca-Core' stamp: 'LT 4/19/2011 10:51'!
fillInstVars: codeLines guardedBy: alreadySerialized
	"For each element in the array, .at_put_() is called. The first parameter can be printed as pure-Javascript-Number (e.g. .at_put_(1,obj2))."
	
	self withIndexDo: [ :each :i |
		each serializeToJavaScript: codeLines guardedBy: alreadySerialized.
		codeLines add: ((alreadySerialized at: self), '.at_put_(', i asString , ',', (alreadySerialized at: each), ')') ].! !

!BlockClosure methodsFor: '*Orca-Core' stamp: 'ag 1/19/2011 18:34'!
onServer
	"This is overridden when executing on the client. On the server, there is no change."
	
	^ self! !

!Number methodsFor: '*Orca-Core' stamp: 'lw 5/5/2011 10:24'!
asRemoteAnswerTo: aSession

	^ self serializeToJavaScript! !

!Number methodsFor: '*Orca-Core' stamp: 'ag 4/6/2011 15:52'!
definitionLine
	
	^ OrcaSqueakAstToJsAst numberFunction asString, '(', self asString, ')'! !

!Number methodsFor: '*Orca-Core' stamp: 'sec 2/16/2011 17:24'!
 aNumber
	{true. false}  atRandom
		ifTrue: [ ^ self + aNumber ]
		ifFalse: [ ^ self - aNumber ]! !

!Collection methodsFor: '*Orca-Core' stamp: 'ag 1/25/2011 16:03'!
separatedBy: aString
	
	| stream |
	stream := TextStream on: ''.
	self asStringOn: stream delimiter: aString.
	^ stream contents! !

!Collection methodsFor: '*Orca-Core' stamp: 'ag 4/27/2011 14:53'!
withUniqueEntries
	
	| set result |
	set := Set new.
	result := self species new.
	self do: [ :each |
		(set includes: each)
			ifFalse: [ set add: each. result add: each ] ].
	^ result! !

TextStream subclass: #OrcaTextStream
	instanceVariableNames: 'printer privateTextStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

!OrcaTextStream class methodsFor: 'instance creation' stamp: 'ag 4/6/2011 18:04'!
for: anOrcaAbstractPrinter
	
	^ (self
		on: '')
		printer: anOrcaAbstractPrinter;
		yourself! !

!OrcaTextStream methodsFor: 'initialization' stamp: 'ag 1/9/2011 20:08'!
initialize
	
	super initialize.
	self privateTextStream: (TextStream on: '').! !

!OrcaTextStream methodsFor: 'printing' stamp: 'ag 1/10/2011 10:14'!
printBlock: aConditionBlock prependedBy: prependBlock
	"aConditionBlock is evaluated with a 'mock'-stream. if true is returned, prependBlock is evaluated and aConditionBlock is evaluated with the actual self."
	
	(aConditionBlock value: self privateTextStream)
		ifTrue: [
			prependBlock value.
			aConditionBlock value: self].! !

!OrcaTextStream methodsFor: 'accessing' stamp: 'ag 1/9/2011 17:38'!
printer
	
	^ printer! !

!OrcaTextStream methodsFor: 'accessing' stamp: 'ag 4/6/2011 18:10'!
printer: anOrcaAbstractPrinterClass
	
	printer := anOrcaAbstractPrinterClass! !

!OrcaTextStream methodsFor: 'accessing' stamp: 'ag 1/9/2011 20:08'!
privateTextStream
	
	^ privateTextStream! !

!OrcaTextStream methodsFor: 'accessing' stamp: 'ag 1/9/2011 20:08'!
privateTextStream: aStream
	
	privateTextStream := aStream! !

Compiler subclass: #OrcaCompiler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

!OrcaCompiler methodsFor: 'public access' stamp: 'robi 3/16/2011 16:42'!
compile: origCode in: cls classified: aCategory notifying: notify ifFail: failBlock
	| code |
	
	(origCode includesSubString:  '<language: #JavaScript>')
		ifTrue: [ code := self stubCodeFor: origCode ]
		ifFalse: [ code := origCode ].

	 ^ super compile: code 
				in: cls 
				classified: aCategory 
				notifying: notify 
				ifFail: failBlock! !

!OrcaCompiler methodsFor: 'code generation' stamp: 'robi 3/16/2011 16:41'!
stubCodeFor: aMethodString

	^ ((aMethodString lineNumber: 1), 
		Character lf, 
		'self error: ''JavaScript code cannot be executed in Squeak environment.''')! !

!Object class methodsFor: '*Orca-Core' stamp: 'ag 4/27/2011 17:25'!
clientStartupClass
	
	^ self! !

!Object class methodsFor: '*Orca-Core' stamp: 'hk 4/29/2011 10:57'!
install

	((self respondsTo: #isOrcaRootComponent) and: [ self isOrcaRootComponent ])
		ifTrue: [
			OrcaDispatcher allInstances
				detect: [ :each | each parent notNil ]
				ifFound: [ :dispatcher |
					dispatcher setComponent: self onPath: self uniquePath ]
				ifNone: [].
		].! !

!Object class methodsFor: '*Orca-Core' stamp: 'ag 4/26/2011 17:58'!
isOrcaRootComponent
	
	^ false! !

!Object class methodsFor: '*Orca-Core' stamp: 'lw 4/19/2011 20:18'!
newOnClient: aClient
	
	| aMessage |
	aMessage := Message selector: #newOnClient.
	^ aClient passMessage: aMessage to: self! !

!Object class methodsFor: '*Orca-Core' stamp: 'LT 4/7/2011 14:18:06.624'!
newOnServer

	^ self new! !

!Object class methodsFor: '*Orca-Core' stamp: 'ag 4/28/2011 11:49'!
orcaKernelClasses
	"These classes are very common and included, if the convenience-configuration-mechanism is used (implementing isOrcaRootComponent with true on a class-side).
	These classes are not needed to start the system up, though.
	Association is included to be able to use the #asObject-functionality on an array of Associations to create a pure javascript-object."
	
	^ { self. Js. Error. OrderedCollection. Association }! !

!Object class methodsFor: '*Orca-Core' stamp: 'ag 4/27/2011 10:18'!
orcaTitle
	
	| charactersInClassName |
	charactersInClassName := ('[A-Za-z]' asRegex matchesIn: self name).
	^ (charactersInClassName separatedBy: '') asLowercase! !

!Object class methodsFor: '*Orca-Core' stamp: 'ag 4/27/2011 10:20'!
requiredClasses
	
	^ self orcaKernelClasses! !

!Object class methodsFor: '*Orca-Core' stamp: 'sec 4/27/2011 11:56'!
requiredScripts
	"this is a reminder that this method has to be overwritten for orca components"
	
	^ super requiredScripts! !

!Object class methodsFor: '*Orca-Core' stamp: 'ag 4/27/2011 10:19'!
uniquePath
	
	^ self orcaTitle! !

!Object methodsFor: '*Orca-Core' stamp: 'lw 5/5/2011 10:24'!
asRemoteAnswerTo: aSession

	^ 'return ', (Dictionary newFrom: { 'remoteId' -> (aSession addReachableObjectFor: self) }) asJson! !

!Object methodsFor: '*Orca-Core' stamp: 'hk 4/5/2011 12:51'!
clientSideResposibility

	self error: thisContext sender selector printString, ' is only implemented on client side' ! !

!Object methodsFor: '*Orca-Core' stamp: 'ag 4/27/2011 17:28'!
clientStartupClass
	"The message #start is called on the class returned here, when the system on the client is initialized.
	The result of this #start call will be bound to st.app"
	
	^ nil! !

!Object methodsFor: '*Orca-Core' stamp: 'lw 4/14/2011 12:05'!
isRemote

	^ false! !

!Object methodsFor: '*Orca-Core' stamp: 'ag 4/21/2011 16:32'!
orcaEmptySessionClass
	"An object, that is used as component in orca, might be queried for a class to use as empty session. It might override this, 
	if a subclass of OrcaSessionBase should be used."
	
	^ OrcaSessionBase! !

!Object methodsFor: '*Orca-Core' stamp: 'ag 4/21/2011 16:32'!
orcaSessionClass
	"An object, that is used as component in orca, might be queried for a class to use as session. It might override this, 
	if a subclass of OrcaSession should be used."
	
	^ OrcaSession! !

!Object methodsFor: '*Orca-Core' stamp: 'ag 4/21/2011 16:30'!
orcaTitle
	
	^ self class name! !

!Object methodsFor: '*Orca-Core' stamp: 'ag 4/21/2011 16:30'!
requiredClasses
	"Return the classes, that are needed by Orca to get this class running on the client-side.
	This is on ProtoObject instance-side and implicitely on the class-site."
	
	^ {}! !

!Object methodsFor: '*Orca-Core' stamp: 'ag 4/21/2011 16:30'!
requiredScripts
	"Return the javascript-scripts, that are needed by Orca to get this class running on the client-side.
	This is on ProtoObject instance-side and implicitely on the class-site."
	
	^ {}! !

!Object methodsFor: '*Orca-Core' stamp: 'ag 4/21/2011 16:31'!
requiredStyles
	"Return the css-style-sheets, that are needed by Orca to get this class running on the client-side.
	This is on ProtoObject instance-side and implicitely on the class-site."
	
	^ {}! !

!Object methodsFor: '*Orca-Core' stamp: 'ag 4/27/2011 17:27'!
serverStartupClass
	"Return anything other than nil to have control-flow started on the server. The message #startOnServer: will be called on the result
	of this method, with the new session as argument."
	
	^ nil! !

Object subclass: #OrcaAbstractCodeFactory
	instanceVariableNames: 'codeCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Cache'!

!OrcaAbstractCodeFactory commentStamp: 'ag 12/16/2010 12:46' prior: 0!
S2JCodeFactory readAllCodeInImage.

S2JCodeFactory clearCache.

S2JCodeFactory codeCache size. 84964. 64386. " 62492 !!!! ".

S2JCodeFactory codeFor: 'a doit-snippet squeak code'.

S2JCodeFactory  codeForClass: Object.

S2JCodeFactory codeForMethod: #asString of: Object.
!

!OrcaAbstractCodeFactory class methodsFor: 'converting' stamp: 'ag 1/5/2011 22:12'!
converterClass
	
	self subclassResponsibility! !

!OrcaAbstractCodeFactory methodsFor: 'private' stamp: 'ag 4/14/2011 12:05'!
cacheCodeFor: anObject producedBy: aBlock
	
	^ (self cacheObjectFor: anObject producedBy: aBlock) javaScriptCode! !

!OrcaAbstractCodeFactory methodsFor: 'private' stamp: 'ag 4/18/2011 11:26'!
cacheObjectFor: anObject producedBy: aBlock
	"cacheObject* returns the value in the dictionary, not the javascript-code (this is done by cacheCodeFor*)"
	
	| result |
	result := self codeCache at: anObject ifAbsent: [ nil ].
	result
		ifNil: [
			result := aBlock value.
			result update.
			self codeCache at: anObject put: result ] "Add to the cache only after update has succeeded."
		ifNotNil: [
			result hasChanged ifTrue: [ result update ]]. "If this update fails, the object removes itself from the cache."
	^ result! !

!OrcaAbstractCodeFactory methodsFor: 'private' stamp: 'ag 4/6/2011 15:52'!
cacheObjectForClass: aClassOrClassname
	"cacheObject* returns the value in the dictionary, not the javascript-code (this is done by cacheCodeFor*)"
	
	| cl |
	cl := self class converterClass classNamed: aClassOrClassname.
	^ self
		cacheObjectFor: cl
		producedBy: [ OrcaTranslatedClass from: cl inFactory: self ]! !

!OrcaAbstractCodeFactory methodsFor: 'private' stamp: 'lw 4/11/2011 15:41'!
cacheObjectForClasses: anArrayOfClasses
	"cacheObject* returns the value in the dictionary, not the javascript-code (this is done by cacheCodeFor*)"
	
	^ self
		cacheObjectFor: anArrayOfClasses
		producedBy: [ OrcaTranslatedClasses from: anArrayOfClasses inFactory: self ]! !

!OrcaAbstractCodeFactory methodsFor: 'private' stamp: 'ag 4/6/2011 15:52'!
cacheObjectForMethod: aCompiledMethod
	"cacheObject* returns the value in the dictionary, not the javascript-code (this is done by cacheCodeFor*)"
	
	^ self
		cacheObjectFor: aCompiledMethod
		producedBy: [
			OrcaTranslatedMethod from: aCompiledMethod inFactory: self ]! !

!OrcaAbstractCodeFactory methodsFor: 'cache' stamp: 'LT 3/16/2011 12:24:21.722'!
clearCache
	
	self codeCache removeAll.! !

!OrcaAbstractCodeFactory methodsFor: 'accessing' stamp: 'ag 12/13/2010 14:30'!
codeCache
	^ codeCache! !

!OrcaAbstractCodeFactory methodsFor: 'accessing' stamp: 'ag 12/13/2010 14:30'!
codeCache: anObject
	codeCache := anObject! !

!OrcaAbstractCodeFactory methodsFor: 'javascript code' stamp: 'ag 4/6/2011 15:52'!
codeFor: aString
	"A simple DoIt-code-snippet"
	
	^ self
		cacheCodeFor: aString
		producedBy: [ OrcaTranslatedDoIt from: aString inFactory: self ]! !

!OrcaAbstractCodeFactory methodsFor: 'javascript code' stamp: 'LT 3/16/2011 12:32:24.802'!
codeForClass: aClassOrClassName
	"A JS-Code-String representing the whole class. Without a DoIt code-snippet etc."
	
	^ (self cacheObjectForClass: aClassOrClassName) javaScriptCode! !

!OrcaAbstractCodeFactory methodsFor: 'javascript code' stamp: 'ag 4/6/2011 15:52'!
codeForClasses: anArrayOfClasses
	"An JS-code-string representing all classes in anArrayOfClasses"
	
	^ self
		cacheCodeFor: anArrayOfClasses
		producedBy: [ OrcaTranslatedClasses from: anArrayOfClasses inFactory: self ]! !

!OrcaAbstractCodeFactory methodsFor: 'javascript code' stamp: 'LT 3/16/2011 12:32:24.814'!
codeForCompiledMethod: aCompiledMethod
	"An association from the function-name to the function-declaration-code"
	
	^ (self cacheObjectForMethod: aCompiledMethod) javaScriptCode! !

!OrcaAbstractCodeFactory methodsFor: 'javascript code' stamp: 'ag 1/5/2011 20:13'!
codeForMethod: aMethodSelector of: aClassOrClassName
	
	^ self codeForCompiledMethod:
		(self class converterClass method: aMethodSelector of: aClassOrClassName)! !

!OrcaAbstractCodeFactory methodsFor: 'private' stamp: 'ag 12/16/2010 11:47'!
compareCacheKey: o1 to: o2
	"CompiledMethods are considered equal, if they represent the same selector in the same class.
	Even after recompilation of a method, the old instance of CompiledMethod will be kept in the dictionary."
	
	o1 class = o2 class ifFalse: [ ^ false ].
	^ o1 class = CompiledMethod
		ifTrue: [ o1 methodClass = o2 methodClass and: [ o1 selector = o2 selector ] ]
		ifFalse: [ o1 = o2 ]! !

!OrcaAbstractCodeFactory methodsFor: 'javascript code' stamp: 'ag 4/21/2011 17:01'!
concatenateStatements: aStringCollection
	
	^ aStringCollection separatedBy: self class converterClass newline! !

!OrcaAbstractCodeFactory methodsFor: 'private' stamp: 'ag 4/18/2011 11:24'!
converterClass
	
	^ self class converterClass! !

!OrcaAbstractCodeFactory methodsFor: 'cache' stamp: 'hk 4/27/2011 14:46'!
hasChanged: anObject
	
	| value |
	^ (value := self codeCache at: anObject ifAbsent: [ nil ])
		ifNil: [ true ]
		ifNotNil: [ value hasChanged ]! !

!OrcaAbstractCodeFactory methodsFor: 'private' stamp: 'ag 1/3/2011 22:19'!
hashCodeFor: anObject
	
	^ anObject class = CompiledMethod
		ifTrue: [ 7 * anObject methodClass hash + anObject selector hash]
		ifFalse: [ anObject hash ]! !

!OrcaAbstractCodeFactory methodsFor: 'initialization' stamp: 'LT 3/16/2011 12:24:15.75'!
initialize
	
	super initialize.
	self codeCache: PluggableDictionary new.
	self codeCache
		equalBlock: [ :o1 :o2 | self compareCacheKey: o1 to: o2 ];
		hashBlock: [ :object | self hashCodeFor: object ].! !

!OrcaAbstractCodeFactory methodsFor: 'cache' stamp: 'ag 2/1/2011 14:04'!
invalidateMethod: aSelector of: anObject
	
	^ self invalidateObject: (self class converterClass method: aSelector of: anObject)! !

!OrcaAbstractCodeFactory methodsFor: 'cache' stamp: 'ag 4/18/2011 10:30'!
invalidateObject: anObject
	
	^ self codeCache removeKey: anObject ifAbsent: [ ]! !

!OrcaAbstractCodeFactory methodsFor: 'cache' stamp: 'ag 2/1/2011 13:51'!
isObjectCached: anObject
	
	^ self codeCache includesKey: anObject! !

!OrcaAbstractCodeFactory methodsFor: 'javascript code' stamp: 'ag 1/9/2011 13:58'!
printCodeFor: aCachedObject to: aFilename
	"Print the code for an object, that is already in the cache, to a new (or overridden) file aFilename.
	Return nil, if object already in cache, return the printed code if successfull."
	
	^ self class converterClass
		print: (self cacheCodeFor: aCachedObject producedBy: [ ^ nil ])
		to: aFilename! !

!OrcaAbstractCodeFactory methodsFor: 'javascript code' stamp: 'ag 1/6/2011 21:03'!
readAllCodeInImage
	"Read each method in the image. This will not throw an error, just skip methods, that cannot be read."
	
	self class converterClass allMethodsDoAndReport: [ :eachMethod |
		self codeForCompiledMethod: eachMethod ].! !

OrcaAbstractCodeFactory subclass: #OrcaCodeFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Cache'!

!OrcaCodeFactory commentStamp: 'ag 12/16/2010 12:46' prior: 0!
S2JCodeFactory readAllCodeInImage.

S2JCodeFactory clearCache.

S2JCodeFactory codeCache size. 84964. 64386. " 62492 !!!! ".

S2JCodeFactory codeFor: 'a doit-snippet squeak code'.

S2JCodeFactory  codeForClass: Object.

S2JCodeFactory codeForMethod: #asString of: Object.
!

!OrcaCodeFactory class methodsFor: 'converting' stamp: 'ag 4/6/2011 15:52'!
converterClass
	
	^ OrcaPrinter! !

OrcaAbstractCodeFactory subclass: #OrcaPrettyCodeFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Cache'!

!OrcaPrettyCodeFactory class methodsFor: 'converting' stamp: 'ag 4/6/2011 15:52'!
converterClass
	
	^ OrcaPrettyPrinter! !

Object subclass: #OrcaCommunicationChannel
	instanceVariableNames: 'releaseBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

OrcaCommunicationChannel subclass: #OrcaCometChannel
	instanceVariableNames: 'timeStampOfLatestRequest pendingReplies hasToReply asynchronouslyReceivedAnswer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

!OrcaCometChannel class methodsFor: 'configuration' stamp: 'SW 1/25/2011 12:16'!
HTTPStatusCodeTimeOut
	
	^ 408! !

!OrcaCometChannel methodsFor: 'communication' stamp: 'ag 4/19/2011 12:55'!
addPendingReply: aString

	self pendingReplies addLast: aString.
	self hasToReply signal.
	^ aString! !

!OrcaCometChannel methodsFor: 'accessing' stamp: 'lw 4/14/2011 14:40'!
asynchronouslyReceivedAnswer
	^ asynchronouslyReceivedAnswer! !

!OrcaCometChannel methodsFor: 'accessing' stamp: 'lw 4/14/2011 14:40'!
asynchronouslyReceivedAnswer: anObject
	asynchronouslyReceivedAnswer := anObject! !

!OrcaCometChannel methodsFor: 'communication' stamp: 'ag 4/21/2011 14:28'!
checkForAnswerIn: aRequest
	
	self asynchronouslyReceivedAnswer
		ifNotNil: [ 
			| answer |
			answer := self extractAnswerFrom: aRequest.
			answer ifNotNil: [ self asynchronouslyReceivedAnswer resolveWith: answer ] ]! !

!OrcaCometChannel methodsFor: 'communication' stamp: 'lw 5/5/2011 10:16'!
extractAnswerFrom: aRequest
	
	| matches |
	matches := 'answer=([^&]*)' asRegex submatchesIn: aRequest requestLine requestURI asString.
	matches size = 1 ifFalse: [ ^nil ].
	^ matches first second asString! !

!OrcaCometChannel methodsFor: 'testing' stamp: 'LT 4/12/2011 14:54:13.245'!
hasPendingReplies

	^ self pendingReplies notEmpty! !

!OrcaCometChannel methodsFor: 'accessing' stamp: 'LT 4/12/2011 17:22:15.878'!
hasToReply
	"semaphore that gets increased whenever a message should be replied"
	^ hasToReply! !

!OrcaCometChannel methodsFor: 'accessing' stamp: 'LT 4/12/2011 17:20:26.217'!
hasToReply: anObject
	hasToReply := anObject! !

!OrcaCometChannel methodsFor: 'initialization' stamp: 'ag 4/27/2011 17:05'!
initialize
	
	super initialize.
	self pendingReplies: OrderedCollection new.
	self timeStampOfLatestRequest: TimeStamp now.
	self hasToReply: Semaphore new.! !

!OrcaCometChannel methodsFor: 'testing' stamp: 'ag 4/19/2011 12:54'!
isAlive

	^ self isTimedOut not! !

!OrcaCometChannel methodsFor: 'testing' stamp: 'ag 4/19/2011 12:52'!
isComet

	^ true! !

!OrcaCometChannel methodsFor: 'testing' stamp: 'LT 4/12/2011 18:53:27.285'!
isTimedOut

	^ TimeStamp now - self timeStampOfLatestRequest > self class maximumTimeWithoutContact! !

!OrcaCometChannel methodsFor: 'accessing' stamp: 'LT 4/12/2011 14:53:57.798'!
pendingReplies
	^ pendingReplies! !

!OrcaCometChannel methodsFor: 'accessing' stamp: 'LT 4/12/2011 14:54:01.066'!
pendingReplies: anObject
	pendingReplies := anObject! !

!OrcaCometChannel methodsFor: 'communication' stamp: 'ag 4/27/2011 16:44'!
performSend: aMessage
	
	self addPendingReply: aMessage! !

!OrcaCometChannel methodsFor: 'communication' stamp: 'ag 4/19/2011 12:55'!
removePendingReply

	^ self pendingReplies removeFirst! !

!OrcaCometChannel methodsFor: 'communication' stamp: 'ag 4/19/2011 12:55'!
renewTimestamp

	self timeStampOfLatestRequest: TimeStamp now.! !

!OrcaCometChannel methodsFor: 'communication' stamp: 'ag 4/20/2011 18:53'!
replyNext
	
	^ OrcaDispatcher
		answer: self removePendingReply
		withStatus: 200! !

!OrcaCometChannel methodsFor: 'communication' stamp: 'ag 4/27/2011 16:47'!
sendAndWait: aMessageString
	
	| result |
	self asynchronouslyReceivedAnswer: Promise new.
	self send: aMessageString.
	result := self waitForAnswerWith: self asynchronouslyReceivedAnswer.
	self asynchronouslyReceivedAnswer: nil.
	^ result! !

!OrcaCometChannel methodsFor: 'accessing' stamp: 'SW 1/18/2011 15:59'!
timeStampOfLatestRequest
	^ timeStampOfLatestRequest! !

!OrcaCometChannel methodsFor: 'accessing' stamp: 'SW 1/18/2011 15:59'!
timeStampOfLatestRequest: anObject
	timeStampOfLatestRequest := anObject! !

!OrcaCometChannel methodsFor: 'communication' stamp: 'ag 4/21/2011 15:36'!
updateFrom: aRequest
	
	self renewTimestamp.
	self checkForAnswerIn: aRequest.	
	
	"wait for defaultMillisecondsToWait for a receive request to the client - otherwise let the client renew its poll"
	^ (self hasToReply waitTimeoutMSecs: self class defaultMillisecondsToWait)
		ifTrue: [
			OrcaDispatcher answerStatus: 
				self class HTTPStatusCodeTimeOut ]
		ifFalse: [ 
			self renewTimestamp.
			self replyNext ]! !

!OrcaCommunicationChannel class methodsFor: 'configuration' stamp: 'LT 4/18/2011 13:48'!
defaultMillisecondsToWait

	^ 300000! !

!OrcaCommunicationChannel class methodsFor: 'configuration' stamp: 'ag 4/27/2011 17:07'!
maximumTimeWithoutContact

	^ Duration minutes: 6! !

!OrcaCommunicationChannel methodsFor: 'initialization' stamp: 'ag 4/27/2011 17:06'!
initialize
	
	super initialize.
	[self removeIfTimedOut] fork.! !

!OrcaCommunicationChannel methodsFor: 'testing' stamp: 'ag 4/19/2011 12:12'!
isAlive 

	self subclassResponsibility.! !

!OrcaCommunicationChannel methodsFor: 'testing' stamp: 'ag 4/19/2011 12:11'!
isComet

	^ false! !

!OrcaCommunicationChannel methodsFor: 'testing' stamp: 'ag 4/19/2011 12:12'!
isWebSocket

	^ false! !

!OrcaCommunicationChannel methodsFor: 'communication' stamp: 'ag 4/27/2011 16:43'!
performSend: aMessage
	
	^ self subclassResponsibility! !

!OrcaCommunicationChannel methodsFor: 'accessing' stamp: 'ag 4/27/2011 16:26'!
releaseBlock
	^ releaseBlock! !

!OrcaCommunicationChannel methodsFor: 'accessing' stamp: 'ag 4/27/2011 16:26'!
releaseBlock: anObject
	releaseBlock := anObject! !

!OrcaCommunicationChannel methodsFor: 'communication' stamp: 'ag 4/27/2011 17:14'!
released
	
	self releaseBlock value.
	self become: OrcaNullChannel new.! !

!OrcaCommunicationChannel methodsFor: 'initialization' stamp: 'ag 4/27/2011 17:06'!
removeIfTimedOut
	
	[self isAlive] whileTrue: [(Delay forDuration: self class maximumTimeWithoutContact) wait].
	self released.! !

!OrcaCommunicationChannel methodsFor: 'communication' stamp: 'ag 4/28/2011 13:45'!
send: aMessage
	"Send an asynchronous message to the client, don't wait for an answer."
	
	self isAlive ifFalse: [ ^ self released ].
	^ [ self performSend: aMessage ]
		on: Error
		do: [ :err | self released ]! !

!OrcaCommunicationChannel methodsFor: 'communication' stamp: 'ag 4/20/2011 18:23'!
sendAndWait: aMessageString
	"Send a synchronous message to the client. Wait for an answer and return it."
	
	self subclassResponsibility! !

!OrcaCommunicationChannel methodsFor: 'communication' stamp: 'ag 4/26/2011 16:28'!
waitForAnswerWith: aPromise
	
	^ (aPromise waitTimeoutMSecs: self class defaultMillisecondsToWait)
		ifTrue: [ aPromise value ]
		ifFalse: [ self error: 'No answer from client-side, could not finish sendAndWait: call' ]! !

OrcaCommunicationChannel subclass: #OrcaWebSocketChannel
	instanceVariableNames: 'socket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

!OrcaWebSocketChannel class methodsFor: 'instance creation' stamp: 'HK 12/15/2010 21:46'!
newWithSocket: aWebSocket

	^ self basicNew socket: aWebSocket; initialize.! !

!OrcaWebSocketChannel methodsFor: 'communication' stamp: 'ag 4/21/2011 14:10'!
close

	self isAlive ifTrue: [ self socket close ].! !

!OrcaWebSocketChannel methodsFor: 'testing' stamp: 'ag 4/20/2011 18:45'!
isAlive
	
	^ self socket isOpen! !

!OrcaWebSocketChannel methodsFor: 'testing' stamp: 'ag 4/19/2011 13:00'!
isWebSocket

	^ true! !

!OrcaWebSocketChannel methodsFor: 'communication' stamp: 'ag 4/27/2011 16:44'!
performSend: aMessage
	
	^ self socket send: aMessage asString! !

!OrcaWebSocketChannel methodsFor: 'communication' stamp: 'ag 4/26/2011 16:57'!
sendAndWait: aMessageString
	
	| asynchronouslyReceivedAnswer |
	asynchronouslyReceivedAnswer := Promise new.
	self socket receiveBlock: 
		[ :message |
		asynchronouslyReceivedAnswer resolveWith: message ].
	self send: aMessageString.
	^ self waitForAnswerWith: asynchronouslyReceivedAnswer.! !

!OrcaWebSocketChannel methodsFor: 'accessing' stamp: 'HK 12/14/2010 10:20'!
socket
	^ socket! !

!OrcaWebSocketChannel methodsFor: 'accessing' stamp: 'HK 12/14/2010 10:20'!
socket: anObject
	socket := anObject! !

Object subclass: #OrcaDefaultHTMLRenderer
	instanceVariableNames: 'session request'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

!OrcaDefaultHTMLRenderer class methodsFor: 'instance creation' stamp: 'ag 4/20/2011 17:02'!
forClient: aClient request: aRequest
	
	^ self basicNew
		session: aClient;
		request: aRequest;
		initialize;
		yourself! !

!OrcaDefaultHTMLRenderer methodsFor: 'rendering-content' stamp: 'hk 4/29/2011 10:16'!
htmlBodyContent
	
	^ '<div id="spinner"></div>'! !

!OrcaDefaultHTMLRenderer methodsFor: 'rendering-content' stamp: 'ag 4/19/2011 11:30'!
htmlDocumentType
	
	^ '<!!DOCTYPE html>'! !

!OrcaDefaultHTMLRenderer methodsFor: 'rendering' stamp: 'ag 4/20/2011 16:07'!
initialHTML
	
	^ self render! !

!OrcaDefaultHTMLRenderer methodsFor: 'rendering-private' stamp: 'ag 4/19/2011 11:34'!
render
	
	^ self htmlDocumentType,
		'<html>',
			self renderHeader, 
			self renderBody,
		'</html>'! !

!OrcaDefaultHTMLRenderer methodsFor: 'rendering-private' stamp: 'ag 4/20/2011 17:46'!
renderBody
	
	^'<body onload="', self session onloadCode, '" onunload="">',
		self htmlBodyContent,
	   '</body>'! !

!OrcaDefaultHTMLRenderer methodsFor: 'rendering-private' stamp: 'ag 4/20/2011 17:41'!
renderHeader

	^ '<head>', 
		'<title>', self session title, '</title>',
		self renderJavascript,
		self renderStyles,
	'</head>'.! !

!OrcaDefaultHTMLRenderer methodsFor: 'rendering-scripts' stamp: 'ag 4/20/2011 17:43'!
renderJavascript
	"Render the external javascript resources."
	
	^ self renderScriptTags: self session initialScripts! !

!OrcaDefaultHTMLRenderer methodsFor: 'rendering-scripts' stamp: 'ag 4/19/2011 11:36'!
renderScriptTag: aFilename
	
	^ '<script type="text/javascript" src="', aFilename, '"></script>'! !

!OrcaDefaultHTMLRenderer methodsFor: 'rendering-scripts' stamp: 'ag 4/19/2011 11:36'!
renderScriptTags: aCollectionOfFilenames
	"aCollectionOfFilenames does not contain absolute Urls, but only the filenames of the js-files. The path (url) will be prepended."
	
	^ self
		renderTags: aCollectionOfFilenames
		collectingFrom: [ :url | self renderScriptTag: url]! !

!OrcaDefaultHTMLRenderer methodsFor: 'rendering-scripts' stamp: 'ag 4/19/2011 11:36'!
renderStyleTag: aFilename
	
	^ '<link href="', aFilename, '" rel="stylesheet" type="text/css" />'! !

!OrcaDefaultHTMLRenderer methodsFor: 'rendering-scripts' stamp: 'ag 4/20/2011 17:43'!
renderStyles
	"Render the external css resources."
	
	^ self
		renderTags: self session styles
		collectingFrom: [ :url | self renderStyleTag: url]! !

!OrcaDefaultHTMLRenderer methodsFor: 'rendering-scripts' stamp: 'ag 4/20/2011 17:46'!
renderTags: aCollectionOfFilenames collectingFrom: aBlock
	"aCollectionOfFilenames does not contain absolute Urls, but only the filenames. The path (url) will be prepended."
	
	^ aCollectionOfFilenames
		inject: ''
		into: [ :string :eachUrl | 
			string, (aBlock value: eachUrl) ]! !

!OrcaDefaultHTMLRenderer methodsFor: 'accessing' stamp: 'ag 4/20/2011 16:07'!
request
	^ request! !

!OrcaDefaultHTMLRenderer methodsFor: 'accessing' stamp: 'ag 4/20/2011 16:07'!
request: anObject
	request := anObject! !

!OrcaDefaultHTMLRenderer methodsFor: 'accessing' stamp: 'ag 4/20/2011 16:51'!
session
	^ session! !

!OrcaDefaultHTMLRenderer methodsFor: 'accessing' stamp: 'ag 4/20/2011 16:51'!
session: anObject
	session := anObject! !

Object subclass: #OrcaIndex
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Misc'!

!OrcaIndex class methodsFor: 'clientside' stamp: 'ag 4/28/2011 13:20'!
initializeList: parameters
	
	| apps |
	Js addToBody: '<h3>Installed applications:</h3>'.
	apps := '<ul>'.
	parameters slotsDo: [ :slotName | | appTitle |
		appTitle := parameters perform: slotName.
		apps := apps, '<li>', (self linkTo: slotName asString named: appTitle), '</li>' ].
	Js addToBody: apps, '</ul>'.! !

!OrcaIndex class methodsFor: 'configuration' stamp: 'ag 4/27/2011 10:21'!
isOrcaRootComponent
	
	^ true! !

!OrcaIndex class methodsFor: 'clientside' stamp: 'ag 4/28/2011 13:17'!
linkTo: uniquePath named: title
	
	^ '<a href="/', uniquePath , '" title="', title ,'">', title , '</a>'! !

!OrcaIndex class methodsFor: 'configuration' stamp: 'ag 4/21/2011 13:24'!
orcaTitle
	
	^ 'Index'! !

!OrcaIndex class methodsFor: 'clientside' stamp: 'ag 4/28/2011 13:18'!
start
	"This app has no start-routine.
	The functionality begins on the server (#startOnServer:), then the client is start set up by sending code."
	
	Js addToBody: '<h1>Orca Framework</h1>'.! !

!OrcaIndex class methodsFor: 'serverside' stamp: 'ag 4/28/2011 13:20'!
startOnServer: aSession
	
	| components |
	components := OrcaSlotObject new.
	aSession dispatcher componentMap keysAndValuesDo: [ :key :component |
		components at: key asSymbol put: component orcaTitle ].
	aSession send: 'OrcaIndex.initializeList(', components serializeToJavaScript, ');'! !

!OrcaIndex class methodsFor: 'configuration' stamp: 'ag 4/21/2011 13:24'!
uniquePath
	
	^ 'index'! !

Object subclass: #OrcaNodeNilObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Translator'!

!OrcaNodeNilObject methodsFor: 'nil object selector handling' stamp: 'lw 4/6/2011 09:25'!
add: anObject
	" to ensure not looking up the selector in the superclasses even without doing nothing"! !

!OrcaNodeNilObject methodsFor: 'nil object selector handling' stamp: 'lw 4/6/2011 09:25'!
addAllWhitoutNils: anObject
	" to ensure not looking up the selector in the superclasses even without doing nothing"! !

!OrcaNodeNilObject methodsFor: 'nil object selector handling' stamp: 'ag 4/6/2011 18:05'!
doesNotUnderstand: aMessage
	"This class implements part of the OrcaNode-interface (ignoring implications)"
	
	(OrcaNode canUnderstand: aMessage selector) ifFalse: [ super doesNotUnderstand: aMessage ]! !

Object subclass: #OrcaNullChannel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

!OrcaNullChannel class methodsFor: 'CometChannel interface' stamp: 'ag 5/2/2011 13:13'!
HTTPStatusCodeTimeOut
	
	^ OrcaCometChannel HTTPStatusCodeTimeOut! !

!OrcaNullChannel methodsFor: 'communication' stamp: 'ag 4/27/2011 17:14'!
isAlive
	
	^ false! !

!OrcaNullChannel methodsFor: 'communication' stamp: 'ag 4/27/2011 16:58'!
isComet
	
	^ false! !

!OrcaNullChannel methodsFor: 'communication' stamp: 'ag 4/27/2011 16:58'!
isWebSocket
	
	^ false! !

!OrcaNullChannel methodsFor: 'communication' stamp: 'ag 4/28/2011 14:41'!
released
	"This gets called, if the channel is released before the timeout expires. Then the timeout expires and this method is called again.
	This is just ignored -- a NullChannel cannot be released, because it already is."
! !

!OrcaNullChannel methodsFor: 'CometChannel interface' stamp: 'ag 5/2/2011 13:12'!
renewTimestamp
	! !

!OrcaNullChannel methodsFor: 'CometChannel interface' stamp: 'ag 5/2/2011 13:12'!
replyNext
	! !

!OrcaNullChannel methodsFor: 'communication' stamp: 'ag 4/27/2011 16:57'!
send: aMessage
	"Ignore - this connection is dead."
	! !

!OrcaNullChannel methodsFor: 'communication' stamp: 'ag 4/27/2011 16:57'!
sendAndWait: aMessage
	"Ignore - this connection is dead."
	! !

Object subclass: #OrcaReachableObjectMap
	instanceVariableNames: 'reachableObjects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-RemoteObjects'!

!OrcaReachableObjectMap methodsFor: 'accessing' stamp: 'LT 4/13/2011 14:06:50.211'!
addReachableObjectFor: anObject

	^ reachableObjects 
		indexOf: anObject 
		ifAbsent: 
			[reachableObjects add: anObject.
			reachableObjects size]! !

!OrcaReachableObjectMap methodsFor: 'accessing' stamp: 'lw 4/19/2011 21:36'!
at: aNumber

	^ reachableObjects at: aNumber! !

!OrcaReachableObjectMap methodsFor: 'initialize-release' stamp: 'LT 4/13/2011 14:09:05.696'!
initialize

	reachableObjects := OrderedCollection new.! !

!OrcaReachableObjectMap methodsFor: 'accessing' stamp: 'LT 4/13/2011 13:49:55.515'!
reachableObjects
	^ reachableObjects! !

!OrcaReachableObjectMap methodsFor: 'accessing' stamp: 'LT 4/13/2011 13:49:55.528'!
reachableObjects: anObject
	reachableObjects := anObject! !

Object subclass: #OrcaRequestHandler
	instanceVariableNames: 'pathSuffix'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

OrcaRequestHandler subclass: #OrcaClassesHandler
	instanceVariableNames: 'codeFactory zippedCodeCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

OrcaClassesHandler class
	instanceVariableNames: 'codeFactoryInstance'!

OrcaClassesHandler class
	instanceVariableNames: 'codeFactoryInstance'!

!OrcaClassesHandler class methodsFor: 'codeFactory' stamp: 'ag 4/27/2011 10:05'!
clearCache
	
	codeFactoryInstance ifNotNil: [ codeFactoryInstance clearCache ].! !

!OrcaClassesHandler class methodsFor: 'configuration' stamp: 'ag 4/27/2011 10:02'!
codeFactoryClass
	
	^ OrcaPrettyCodeFactory! !

!OrcaClassesHandler class methodsFor: 'codeFactory' stamp: 'ag 4/27/2011 10:04'!
codeFactoryInstance
	"Lazily create the singleton-like codeFactory-instance, that is shared by default among ClassesHandler-instances.
	Ensure, that it's always of the type configured in #codeFactoryClass"
	
	codeFactoryInstance isNil | (codeFactoryInstance isKindOf: self codeFactoryClass) not
		ifTrue: [ codeFactoryInstance := self codeFactoryClass new ].
	^ codeFactoryInstance! !

!OrcaClassesHandler class methodsFor: 'codeFactory' stamp: 'ag 4/27/2011 10:13'!
codeFactoryInstance: aCodeFactoryInstance
	"Tests need the functionality to restore the code-factory."
	
	codeFactoryInstance := aCodeFactoryInstance! !

!OrcaClassesHandler class methodsFor: 'configuration' stamp: 'ag 4/21/2011 11:32'!
defaultPathSuffix
	
	^ 'classes'! !

!OrcaClassesHandler class methodsFor: 'instance creation' stamp: 'ag 4/27/2011 10:01'!
new
	
	^ self newWithFactory: self codeFactoryInstance! !

!OrcaClassesHandler class methodsFor: 'instance creation' stamp: 'ag 4/21/2011 12:54'!
newWithFactory: aCodeFactory
	
	^ super new
		codeFactory: aCodeFactory;
		yourself! !

!OrcaClassesHandler methodsFor: 'caching' stamp: 'hk 4/27/2011 15:20'!
cacheOutdatedFor: classes

	^ (self zippedCodeCache includesKey: classes) not
		or: [ self codeFactory hasChanged: classes ]! !

!OrcaClassesHandler methodsFor: 'accessing' stamp: 'ag 4/21/2011 12:53'!
codeFactory
	^ codeFactory! !

!OrcaClassesHandler methodsFor: 'accessing' stamp: 'ag 4/21/2011 12:53'!
codeFactory: anObject
	codeFactory := anObject! !

!OrcaClassesHandler methodsFor: 'zipping' stamp: 'hk 4/20/2011 15:10'!
doesSupportGZip: aRequest

	| field |
	field := aRequest headers fieldNamed: 'ACCEPT-ENCODING' ifNone: [ nil ].
	field ifNil: [ ^ false ].
	^ field value includesSubString: 'gzip'
! !

!OrcaClassesHandler methodsFor: 'request handling' stamp: 'hk 4/27/2011 14:18'!
handleRequest: aRequest forSession: aSession
	
	| requiredClasses |
	requiredClasses := aSession requiredClasses.
	^ (self doesSupportGZip: aRequest)
		ifTrue: [ OrcaDispatcher answerZippedCode: (self zippedCodeFor: requiredClasses) ]
		ifFalse: [ OrcaDispatcher answerCode: (self codeFactory codeForClasses: requiredClasses) ]! !

!OrcaClassesHandler methodsFor: 'request handling' stamp: 'ag 4/21/2011 11:24'!
isHandlerFor: aSymbol
	
	^ aSymbol = #classes! !

!OrcaClassesHandler methodsFor: 'request handling' stamp: 'ag 4/20/2011 16:57'!
needsSession
	
	^ false! !

!OrcaClassesHandler methodsFor: 'accessing' stamp: 'hk 4/27/2011 14:35'!
zippedCodeCache

	zippedCodeCache ifNil: [ self zippedCodeCache: Dictionary new ]. 
	^ zippedCodeCache
	
! !

!OrcaClassesHandler methodsFor: 'accessing' stamp: 'hk 4/27/2011 14:31'!
zippedCodeCache: anObject
	zippedCodeCache := anObject! !

!OrcaClassesHandler methodsFor: 'zipping' stamp: 'hk 4/27/2011 15:19'!
zippedCodeFor: classes

	(self cacheOutdatedFor: classes) ifTrue: [
		self zippedCodeCache
			at: classes
			put: (self codeFactory codeForClasses: classes) zipped ].
		
	^ self zippedCodeCache at: classes! !

OrcaRequestHandler subclass: #OrcaCometHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

!OrcaCometHandler class methodsFor: 'configuration' stamp: 'ag 4/21/2011 11:32'!
defaultPathSuffix
	
	^ 'xhr'! !

!OrcaCometHandler methodsFor: 'request handling' stamp: 'ag 4/27/2011 17:01'!
handleRequest: aRequest forSession: aClient
	"Initialize/Create the channel of the client as a comet-channel. Update the channel with the new connection."
	
	(aClient channel isNil or: [ aClient channel isComet not ]) ifTrue: [
		aClient initializeChannel: 
			(OrcaCometChannel new) ].
	
	^ aRequest isGet ifTrue: [
		aClient channel updateFrom: aRequest ]! !

!OrcaCometHandler methodsFor: 'request handling' stamp: 'ag 4/20/2011 16:57'!
needsSession
	
	^ true! !

OrcaRequestHandler subclass: #OrcaFileRequestHandler
	instanceVariableNames: 'rootDirectory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

!OrcaFileRequestHandler class methodsFor: 'configuration' stamp: 'ag 4/21/2011 11:32'!
defaultPathSuffix
	
	^ 'file'! !

!OrcaFileRequestHandler class methodsFor: 'configuration' stamp: 'ag 4/26/2011 09:24'!
defaultRootDirectory
	
	^ Smalltalk
		at: #OrcaFileRootDirectory
		ifAbsent: [
			FileDirectory on: 
				(FileDirectory default pathName, 
				FileDirectory pathNameDelimiter asString, 
				self defaultRootFolderName) ]! !

!OrcaFileRequestHandler class methodsFor: 'configuration' stamp: 'ag 4/21/2011 11:38'!
defaultRootDirectory: aDirectoryName
	"Configure the root-directory, this mini-server is serving. Only required once."
	
	| dir |
	dir := FileDirectory on: aDirectoryName.
	dir exists ifFalse: [ self error: 'Directory does not exist: ', aDirectoryName ].
	^ Smalltalk
		at: #OrcaFileRootDirectory
		put: dir! !

!OrcaFileRequestHandler class methodsFor: 'configuration' stamp: 'ag 4/21/2011 11:35'!
defaultRootFolderName
	
	^ 'orca'! !

!OrcaFileRequestHandler class methodsFor: 'instance creation' stamp: 'ag 4/21/2011 11:37'!
new
	
	^ self newOnRoot: self defaultRootDirectory! !

!OrcaFileRequestHandler class methodsFor: 'instance creation' stamp: 'ag 4/21/2011 12:13'!
newOnRoot: aDirectory
	
	^ super new
		rootDirectory: aDirectory;
		yourself! !

!OrcaFileRequestHandler methodsFor: 'request handling' stamp: 'ag 4/21/2011 11:30'!
extractFile: aRequest
	
	| segments |
	segments := OrcaDispatcher pathSegments: aRequest.
	segments removeFirst. "The app-name"
	segments removeFirst. "The /file -suffix"
	^ self rootDirectory oldFileOrNoneNamed: (segments separatedBy: FileDirectory pathNameDelimiter asString)! !

!OrcaFileRequestHandler methodsFor: 'request handling' stamp: 'ag 4/21/2011 11:35'!
handleRequest: aRequest forSession: aClient
	"This handler ignores the client"

	| file |
	file := self extractFile: aRequest.
	^ file
		ifNotNil: [ 
			file binary. "images require binary mode. plain text works regardless of encoding"
			OrcaDispatcher answerContent: file contents withType: (self mimeTypeOf: file) ]
		ifNil: [ OrcaDispatcher answer: 'Requested file not found' withStatus: 404 ]! !

!OrcaFileRequestHandler methodsFor: 'request handling' stamp: 'ag 4/21/2011 11:36'!
isHandlerFor: aSymbol
	
	^ aSymbol = #files! !

!OrcaFileRequestHandler methodsFor: 'request handling' stamp: 'hk 3/29/2011 11:21'!
mimeTypeOf: aFile

	^ MIMEDocument guessTypeFromName: aFile name! !

!OrcaFileRequestHandler methodsFor: 'request handling' stamp: 'ag 4/20/2011 16:57'!
needsSession
	
	^ false! !

!OrcaFileRequestHandler methodsFor: 'accessing' stamp: 'ag 4/21/2011 11:35'!
rootDirectory
	^ rootDirectory! !

!OrcaFileRequestHandler methodsFor: 'accessing' stamp: 'ag 4/21/2011 11:31'!
rootDirectory: anObject
	rootDirectory := anObject! !

OrcaRequestHandler subclass: #OrcaMethodInvocationHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

!OrcaMethodInvocationHandler class methodsFor: 'configuration' stamp: 'ag 4/21/2011 11:32'!
defaultPathSuffix
	
	^ 'mi'! !

!OrcaMethodInvocationHandler methodsFor: 'request handling' stamp: 'ag 4/26/2011 15:00'!
handleRequest: aRequest forSession: aClient
	
	| code arguments result |
	aRequest ensureFullRead.
	code := (aRequest postDataAt: 'code') value.
	arguments := self retrieveArgumentsFrom: aRequest.
	result := (Compiler evaluate: code) valueWithArguments: arguments asArray.
	^ OrcaDispatcher answerContent: result serializeToJavaScript withType: 'text/javascript'! !

!OrcaMethodInvocationHandler methodsFor: 'request handling' stamp: 'ag 4/20/2011 16:57'!
needsSession
	
	^ false! !

!OrcaMethodInvocationHandler methodsFor: 'request handling' stamp: 'LT 2/21/2011 14:47:33.929'!
retrieveArgumentsFrom: aRequest

	| arguments currentArg |
	arguments := OrderedCollection new.
	currentArg := 0.
	[ currentArg isNil ]
		whileFalse: [ 
			| key arg |
			key := 'arg' , currentArg asString.
			arg := (aRequest postDataAt: key) value.
			arg
				ifNil: [ currentArg := nil ]
				ifNotNil: [ 
					arguments add: arg.
					currentArg := currentArg + 1 ] ].
	^ arguments! !

OrcaRequestHandler subclass: #OrcaRemoteMessageHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-RemoteObjects'!

!OrcaRemoteMessageHandler class methodsFor: 'configuration' stamp: 'ag 4/21/2011 11:32'!
defaultPathSuffix
	
	^ 'send'! !

!OrcaRemoteMessageHandler methodsFor: 'request handling' stamp: 'LT 5/2/2011 15:06'!
createNew: className on: aSession
	
	| object |
	object := (Smalltalk classNamed: className) new.
	^ OrcaDispatcher
			answerContent: (object asRemoteAnswerTo: aSession) 
			withType: 'text/javascript'! !

!OrcaRemoteMessageHandler methodsFor: 'request handling' stamp: 'ag 4/21/2011 18:10'!
createRechableObjectFrom: aRequest for: aSession
	
	^ (aRequest postDataAt: 'newObjectOfClassNamed') value 
			ifNotNil: [ : className | self createNew: className on: aSession ]! !

!OrcaRemoteMessageHandler methodsFor: 'request handling' stamp: 'LT 5/3/2011 17:33'!
handleRequest: aRequest forSession: aSession
	"aSession must be an OrcaRemoteObjectClient here"
	
	aRequest ensureFullRead.

	[ (self createRechableObjectFrom: aRequest for: aSession) ifNotNil: [ :answer | ^ answer ].
	(self sendRemoteMessageFrom: aRequest for: aSession) ifNotNil: [ :answer | ^ answer ]] 
		on: Error 
		do: [ :error | 
			^ OrcaDispatcher 
				answerContent: (error asRemoteAnswerTo: aSession)
				withType: 'application/json' ].! !

!OrcaRemoteMessageHandler methodsFor: 'request handling' stamp: 'ag 4/20/2011 16:57'!
needsSession
	
	^ true! !

!OrcaRemoteMessageHandler methodsFor: 'request handling' stamp: 'LT 5/3/2011 17:30'!
sendRemoteMessageFrom: aRequest for: aSession
	
	| remoteId messageJson selector arguments answer |

	remoteId :=  (aRequest postDataAt: 'rid') value asInteger.
	messageJson := JSJsonParser parse: (aRequest postDataAt: 'message') value.
	selector := (messageJson at: 'selector') asSymbol.
	
	arguments := messageJson at: 'arguments'.
	
	arguments := arguments collect: 
		[:each | (each isDictionary and: [each size = 1] and: [each includesKey: 'remoteId']) 
			ifFalse: [each] 
			ifTrue: [OrcaRemoteObject newWithId: (each at: 'remoteId') andSession: aSession]].
	
	selector isNil | remoteId isNil ifTrue: [ ^nil ].

	answer := (Message selector: selector arguments: arguments) 
		sendTo: (aSession reachableObjectDescribedBy: remoteId).

	^ OrcaDispatcher  
		answerContent: (answer asRemoteAnswerTo: aSession)
		withType: 'text/javascript'! !

!OrcaRequestHandler class methodsFor: 'configuration' stamp: 'ag 4/21/2011 11:34'!
defaultPathSuffix
	"This is a default value the pathSuffix of this handler. This configuration is static for convenience, the actual value is dynamically configurable."
	
	self subclassResponsibility! !

!OrcaRequestHandler class methodsFor: 'instance creation' stamp: 'ag 4/21/2011 11:33'!
new
	
	^ self newOnPath: self defaultPathSuffix! !

!OrcaRequestHandler class methodsFor: 'instance creation' stamp: 'ag 4/21/2011 12:11'!
newOnPath: pathSuffix
	
	^ self basicNew
		pathSuffix: pathSuffix;
		initialize;
		yourself! !

!OrcaRequestHandler methodsFor: 'request handling' stamp: 'ag 4/20/2011 16:57'!
handleRequest: aRequest forSession: aClient
	
	self subclassResponsibility! !

!OrcaRequestHandler methodsFor: 'request handling' stamp: 'ag 4/21/2011 11:24'!
isHandlerFor: aSymbol
	"Some handlers can make themselves available, identified by a symbol."
	
	^ false! !

!OrcaRequestHandler methodsFor: 'request handling' stamp: 'ag 4/20/2011 16:57'!
needsSession
	
	self subclassResponsibility! !

!OrcaRequestHandler methodsFor: 'accessing' stamp: 'ag 4/21/2011 12:09'!
pathSuffix
	^ pathSuffix! !

!OrcaRequestHandler methodsFor: 'accessing' stamp: 'ag 4/21/2011 11:31'!
pathSuffix: anObject
	pathSuffix := anObject! !

OrcaRequestHandler subclass: #OrcaStartupCodeHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

!OrcaStartupCodeHandler class methodsFor: 'configuration' stamp: 'ag 4/26/2011 17:33'!
defaultPathSuffix
	
	^ 'startupCode'! !

!OrcaStartupCodeHandler methodsFor: 'request handling' stamp: 'ag 4/27/2011 10:39'!
handleRequest: aRequest forSession: aSession
	
	^ OrcaDispatcher answerCode: aSession startupCode! !

!OrcaStartupCodeHandler methodsFor: 'request handling' stamp: 'ag 4/26/2011 17:32'!
isHandlerFor: aSymbol
	
	^ aSymbol = #startupCode! !

!OrcaStartupCodeHandler methodsFor: 'request handling' stamp: 'ag 4/26/2011 17:32'!
needsSession
	
	^ false! !

OrcaRequestHandler subclass: #OrcaWebSocketHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

!OrcaWebSocketHandler class methodsFor: 'configuration' stamp: 'ag 4/21/2011 11:32'!
defaultPathSuffix
	
	^ 'ws'! !

!OrcaWebSocketHandler methodsFor: 'request handling' stamp: 'ag 4/26/2011 17:12'!
handleRequest: aRequest forSession: aClient
	
	| webSocket |
	webSocket := aRequest webSocket.
	(aClient channel isNil or: [ aClient channel isWebSocket not ] or: [ aClient channel socket ~= webSocket ])
		ifTrue: [
			webSocket receiveBlock: [ :firstMessage |
				"The client will send one initial message to notify, that the connection is up."
				webSocket receiveBlock: [ :m ].
				aClient initializeChannel: (OrcaWebSocketChannel newWithSocket: webSocket) ] ].
	^ webSocket answerTo: aRequest! !

!OrcaWebSocketHandler methodsFor: 'request handling' stamp: 'ag 4/20/2011 16:57'!
needsSession
	
	^ true! !

Object subclass: #OrcaSessionBase
	instanceVariableNames: 'component dispatcher requiredClasses collectedObjects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

OrcaSessionBase subclass: #OrcaSession
	instanceVariableNames: 'channel id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Server'!

OrcaSession subclass: #OrcaRemoteObjectsSession
	instanceVariableNames: 'reachableObjectMap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-RemoteObjects'!

!OrcaRemoteObjectsSession class methodsFor: 'parsing' stamp: 'LT 5/2/2011 11:43'!
parseJSON: aString
	
	^ JSJsonParser parse: aString! !

!OrcaRemoteObjectsSession class methodsFor: 'configuration' stamp: 'ag 4/21/2011 11:42'!
postloadScripts
	
	^ super postloadScripts, { 'remoteObjects.js' }! !

!OrcaRemoteObjectsSession methodsFor: 'remote object handling' stamp: 'ag 4/19/2011 13:05'!
addReachableObjectFor: anObject
	
	^ self reachableObjectMap addReachableObjectFor: anObject! !

!OrcaRemoteObjectsSession methodsFor: 'remote object handling' stamp: 'ag 4/21/2011 14:32'!
convertAnswer: answerObject

	"These are the three possible types for JSON-Serialization"
	answerObject isDictionary
		ifTrue: [ ^ self convertObjectAnswer: answerObject ].
	answerObject isArray ifTrue: 
		[ ^ self convertArrayAnswer: answerObject ].
	""
	^ answerObject! !

!OrcaRemoteObjectsSession methodsFor: 'remote object handling' stamp: 'ag 4/21/2011 14:31'!
convertArrayAnswer: anArray

	^ anArray collect: [ :each :i | self convertAnswer: each ]! !

!OrcaRemoteObjectsSession methodsFor: 'remote object handling' stamp: 'LT 5/3/2011 17:30'!
convertObjectAnswer: aDictionary
	
	aDictionary size = 1 
		ifTrue: [ | property |
			property := aDictionary keys first.
			"These are the only two types of Objects send, yet"
			property = 'remoteId'
				ifTrue: [ ^ OrcaRemoteObject newWithId: (aDictionary at: 'remoteId') andSession: self ].
			property = 'error'
				ifTrue: [ ^ self error: (aDictionary at: 'error') ]].
	^ self error: 'Unexpected answer from Client. Interpretation failed'! !

!OrcaRemoteObjectsSession methodsFor: 'initialization' stamp: 'ag 4/19/2011 13:05'!
initialize

	super initialize.
	self reachableObjectMap: OrcaReachableObjectMap new.! !

!OrcaRemoteObjectsSession methodsFor: 'remote object handling' stamp: 'LT 5/3/2011 12:53'!
passMessage: aMessage to: anObject
	
	anObject isRemote 
		ifTrue: [ 
			| plainAnswer | 
			plainAnswer := self channel sendAndWait: 
					(self requestLineForMessage: aMessage andId: anObject xxxId).
			^ self convertAnswer: (self class parseJSON: plainAnswer) ]
		ifFalse: [ 
			(aMessage selector = #newOnClient and: [ anObject isBehavior ])
				ifTrue: [ ^ self remoteObjectForClass: anObject ]
				ifFalse: [ anObject error: 'Unexpected remote message send.' ]]! !

!OrcaRemoteObjectsSession methodsFor: 'remote object handling' stamp: 'ag 4/21/2011 14:26'!
reachableObjectDescribedBy: aNumber

	^ self reachableObjectMap at: aNumber! !

!OrcaRemoteObjectsSession methodsFor: 'accessing' stamp: 'ag 4/19/2011 13:05'!
reachableObjectMap
	^ reachableObjectMap! !

!OrcaRemoteObjectsSession methodsFor: 'accessing' stamp: 'ag 4/19/2011 13:05'!
reachableObjectMap: anObject
	reachableObjectMap := anObject! !

!OrcaRemoteObjectsSession methodsFor: 'remote object handling' stamp: 'ag 4/21/2011 15:05'!
remoteObjectForClass: anObject
	
	| requestText answerText remoteObject |
	requestText := TextStream with: ''.
	requestText << 'newObjectOfClassNamed=' << anObject name.
	
	answerText := self channel sendAndWait: requestText contents.
	
	remoteObject := self convertAnswer: (self class parseJSON: answerText).
	remoteObject isRemote ifFalse: [ self error: 'creation of new Object on Client didn''t result in a new Object...' ].
	^ remoteObject! !

!OrcaRemoteObjectsSession methodsFor: 'remote object handling' stamp: 'LT 5/3/2011 14:01'!
requestLineForMessage: aMessage andId: anInteger

	| line |
	line := TextStream with: ''.
	line << 'rid=' << anInteger.
	line << '&message=' << (aMessage asRemoteAnswerTo: self).
	^ line contents! !

!OrcaSession class methodsFor: 'configuration' stamp: 'ag 4/21/2011 13:58'!
initialScripts
	
	^ { 'helpers.js'. 'console.js'. 'communication.js' }! !

!OrcaSession class methodsFor: 'instance creation' stamp: 'ag 4/21/2011 13:16'!
newWithId: anInteger dispatcher: dispatcher rootComponent: anObject
	
	^ self basicNew
		id: anInteger;
		component: anObject;
		dispatcher: dispatcher;
		initialize;
		yourself! !

!OrcaSession class methodsFor: 'configuration' stamp: 'ag 4/21/2011 11:42'!
postloadScripts
	
	^ { 'server.js'. 'perform.js'. 'boxing.js'. 'bootstrap.js' }! !

!OrcaSession class methodsFor: 'configuration' stamp: 'ag 4/21/2011 11:41'!
preloadScripts
	
	^ { 'classes.js' }! !

!OrcaSession class methodsFor: 'configuration' stamp: 'ag 4/21/2011 13:39'!
stylesFolderName
	
	^ 'styles'! !

!OrcaSession methodsFor: 'accessing' stamp: 'ag 4/27/2011 16:40'!
channel
	^ channel! !

!OrcaSession methodsFor: 'accessing' stamp: 'ag 4/20/2011 16:50'!
channel: anObject
	channel := anObject! !

!OrcaSession methodsFor: 'cleanup' stamp: 'ag 4/27/2011 16:54'!
cleanup
	
	self dispatcher sessionDied: self.! !

!OrcaSession methodsFor: 'clientside initialization' stamp: 'ag 4/26/2011 17:46'!
collectPrimitiveScripts
	"For each required class, check whether there is a primitive js-script in the js/primitives/-folder."
	
	| primitivesFolder |
	primitivesFolder := self primitivesFolder.
	^ (self requiredClasses 
		collect: [ :eachClass | eachClass name, '.js' ]
		thenSelect: [ :eachString | primitivesFolder fileExists: eachString ])
			collect: [ :eachString | self class primitivesFolderName, '/', eachString ]! !

!OrcaSession methodsFor: 'clientside initialization' stamp: 'ag 4/27/2011 14:56'!
collectRequiredScripts
	
	^ (self allConfigurations collect: [ :each | each requiredScripts ]) 
		concatenation asOrderedCollection withUniqueEntries! !

!OrcaSession methodsFor: 'clientside initialization' stamp: 'lw 4/27/2011 18:00'!
handleChannelOpened
	"Now, that we have an open channel available, we tell the client, which files it has to load."
	
	self channel send: (self loadScriptsCode: self scriptsForClientInitialization).
	self component serverStartupClass
		ifNotNil: [ :startupClass | startupClass startOnServer: self ].! !

!OrcaSession methodsFor: 'accessing' stamp: 'ag 4/19/2011 13:59'!
id
	^ id! !

!OrcaSession methodsFor: 'accessing' stamp: 'ag 4/19/2011 13:59'!
id: anObject
	id := anObject! !

!OrcaSession methodsFor: 'rendering information' stamp: 'ag 4/21/2011 13:40'!
initialScripts
	
	^ self javascriptUrlForAll: self class initialScripts! !

!OrcaSession methodsFor: 'clientside initialization' stamp: 'ag 4/27/2011 16:52'!
initializeChannel: aChannel
	"When the channel of this session is opened by one of the handlers, this method is called with the new channel."
	
	self channel: aChannel.
	aChannel releaseBlock: [ self cleanup ].
	^ self handleChannelOpened! !

!OrcaSession methodsFor: 'clientside initialization' stamp: 'sec 4/27/2011 11:59'!
javascriptUrlForAll: aCollection
	
	^ aCollection collect: [ :each | self javascriptUrlFor: each ]! !

!OrcaSession methodsFor: 'clientside initialization' stamp: 'ag 4/26/2011 17:45'!
loadScriptsCode: scriptNames
	"This is a function in helpers.js, which downloads and handles (evaluates) a script"
	
	^ 'st.communication.addScriptTags([',
		((scriptNames
			collect: [ :each | OrcaAbstractPrinter asJSString: each ])
			separatedBy: ',')
		,']);'! !

!OrcaSession methodsFor: 'rendering information' stamp: 'ag 4/21/2011 14:04'!
onloadCode
	
	^ 
	'st.communication.setup_session_id(', self id, ');',
	'st.communication.connect();'! !

!OrcaSession methodsFor: 'clientside initialization' stamp: 'ag 4/26/2011 17:45'!
primitivesCode
	
	^ self loadScriptsCode: self collectPrimitiveScripts! !

!OrcaSession methodsFor: 'clientside initialization' stamp: 'ag 4/26/2011 17:45'!
primitivesFolder
	"Ask the handler of #files for the rootDirectory. Navigate to /js/primitives".
	
	^ (self dispatcher handlerForResource: #files)
		rootDirectory / self class javascriptFolderName / self class primitivesFolderName! !

!OrcaSession methodsFor: 'clientside initialization' stamp: 'ag 4/27/2011 14:54'!
scriptsForClientInitialization
	
	^ 
	(self javascriptUrlForAll: self class preloadScripts)
		, { self dispatcher pathForResource: #classes }
		, (self javascriptUrlForAll: 
			self class postloadScripts,
			self collectRequiredScripts,
			self collectPrimitiveScripts)
		, { self dispatcher pathForResource: #startupCode }! !

!OrcaSession methodsFor: 'sending' stamp: 'sw 4/28/2011 14:03'!
sendCode: codeString
	
	self channel send: 
		((self dispatcher handlerForResource: #classes)
			codeFactory codeFor: codeString).! !

!OrcaSession methodsFor: 'clientside initialization' stamp: 'ag 4/21/2011 13:41'!
styleUrlFor: script
	
	^ self fullUrlFor: script inFolder: self class stylesFolderName! !

!OrcaSession methodsFor: 'rendering information' stamp: 'ag 4/27/2011 14:56'!
styles

	^ (self allConfigurations collect: [ :each | each requiredStyles ])
		concatenation asOrderedCollection withUniqueEntries
			collect: [ :each | self styleUrlFor: each ]! !

!OrcaSession methodsFor: 'rendering information' stamp: 'ag 4/21/2011 13:28'!
title
	
	^ self component orcaTitle! !

!OrcaSessionBase class methodsFor: 'collecting' stamp: 'ag 4/21/2011 14:41'!
collectRecursively: aBlock startingWith: aCollection
	"Evaluate aBlock, collecting recursively from the answered objects with the collectionBlock.
	The Result is a Set"
	
	| result pendingItems |
	result := Set new.
	pendingItems := LinkedList new.
	aCollection do: [ :each | pendingItems addLast: (StackLink with: each) ].
	[ pendingItems notEmpty ]
		whileTrue: [	(result includes: pendingItems first element) 
			ifFalse: [
				(aBlock value: pendingItems first element) do: [ :each | pendingItems addLast: (StackLink with: each) ].
				result add: pendingItems removeFirst element ]
			ifTrue: [ pendingItems removeFirst ]].
	^ result! !

!OrcaSessionBase class methodsFor: 'configuration' stamp: 'ag 4/21/2011 17:15'!
javascriptFolderName
	
	^ 'js'! !

!OrcaSessionBase class methodsFor: 'configuration' stamp: 'hk 5/2/2011 13:17'!
mandatoryClasses
	"This is a list of classes, that are needed by the initialization-scripts to perform the setup.
	This must not be modified. If you want to extend this list, use OrcaAbstractComponent class >> #kernelClasses or #requiredClasses on your own component."
	
	^ { True. False. UndefinedObject. BlockClosure. ByteString. Float. Character. Array. Message }! !

!OrcaSessionBase class methodsFor: 'instance creation' stamp: 'ag 4/21/2011 17:28'!
newForComponent: anObject dispatcher: aDispatcher
	
	^ self basicNew
		component: anObject;
		dispatcher: aDispatcher;
		initialize;
		yourself! !

!OrcaSessionBase class methodsFor: 'configuration' stamp: 'ag 4/21/2011 17:15'!
primitivesFolderName
	
	^ 'primitives'! !

!OrcaSessionBase methodsFor: 'collecting' stamp: 'ag 4/27/2011 14:39'!
allConfigurations
	
	| result |
	result := self requiredClasses clone.
	result addAllLast: self collectedObjects.
	^ result! !

!OrcaSessionBase methodsFor: 'collecting' stamp: 'ag 4/27/2011 14:14'!
collectRecursiveObjects
	"Starting from the own component, collect requiredClasses recursively. The passed object are not necessarily classes, filter them afterwards."
	
	^ self
		collectRecursively: [ :each |
			each requiredClasses union: 
				(each isBehavior
					ifTrue: [ each allSuperclasses ]
					ifFalse: [ {} ]) ]
		startingWith: 
			(self class mandatoryClasses, { self component })! !

!OrcaSessionBase methodsFor: 'collecting' stamp: 'ag 4/21/2011 14:01'!
collectRecursively: aBlock startingWith: aCollection
	"Evaluate aBlock, collecting recursively from the answered objects with the collectionBlock.
	The Result is a Set"
	
	| result pendingItems |
	result := Set new.
	pendingItems := LinkedList new.
	aCollection do: [ :each | pendingItems addLast: (StackLink with: each) ].
	[ pendingItems notEmpty ]
		whileTrue: [	(result includes: pendingItems first element) 
			ifFalse: [
				(aBlock value: pendingItems first element) do: [ :each | pendingItems addLast: (StackLink with: each) ].
				result add: pendingItems removeFirst element ]
			ifTrue: [ pendingItems removeFirst ]].
	^ result! !

!OrcaSessionBase methodsFor: 'accessing' stamp: 'ag 4/21/2011 13:34'!
collectedObjects
	^ collectedObjects! !

!OrcaSessionBase methodsFor: 'accessing' stamp: 'ag 4/21/2011 13:34'!
collectedObjects: anObject
	collectedObjects := anObject! !

!OrcaSessionBase methodsFor: 'accessing' stamp: 'ag 4/21/2011 13:16'!
component
	^ component! !

!OrcaSessionBase methodsFor: 'accessing' stamp: 'ag 4/21/2011 13:16'!
component: anObject
	component := anObject! !

!OrcaSessionBase methodsFor: 'accessing' stamp: 'ag 4/21/2011 17:30'!
dispatcher
	^ dispatcher! !

!OrcaSessionBase methodsFor: 'accessing' stamp: 'ag 4/21/2011 17:30'!
dispatcher: anObject
	dispatcher := anObject! !

!OrcaSessionBase methodsFor: 'collecting' stamp: 'ag 4/21/2011 17:18'!
fullUrlFor: script inFolder: folderName
	
	^ (script beginsWith: 'http')
		ifTrue: [ script ]
		ifFalse: [ (self dispatcher pathForResource: #files), '/', folderName, '/', script ]! !

!OrcaSessionBase methodsFor: 'initialization' stamp: 'ag 4/27/2011 14:35'!
initialize
	
	super initialize.
	self collectedObjects: self collectRecursiveObjects.
	self requiredClasses: (self sortClasses: (self collectedObjects select: [ :each | each isBehavior ])).
	
	"Collected objects are all configuration-objects, that are not classes, but implement #requiredClasses etc."
	self collectedObjects removeAll: self requiredClasses.! !

!OrcaSessionBase methodsFor: 'collecting' stamp: 'ag 4/21/2011 17:18'!
javascriptUrlFor: script
	
	^ self fullUrlFor: script inFolder: self class javascriptFolderName! !

!OrcaSessionBase methodsFor: 'accessing' stamp: 'ag 4/20/2011 18:04'!
requiredClasses
	^ requiredClasses! !

!OrcaSessionBase methodsFor: 'accessing' stamp: 'ag 4/20/2011 18:04'!
requiredClasses: anObject
	requiredClasses := anObject! !

!OrcaSessionBase methodsFor: 'collecting' stamp: 'ag 4/21/2011 13:17'!
sortClasses: aSet
	
	| sortedClasses |
	sortedClasses := OrderedCollection new.
	
	[aSet notEmpty] whileTrue: 
		[ aSet do: [ :eachClass | 
			(eachClass superclass = nil 
				or: [sortedClasses includes: eachClass superclass])
					ifTrue: 
						[ sortedClasses addLast: eachClass. 
						aSet remove: eachClass ]]].
	^ sortedClasses! !

!OrcaSessionBase methodsFor: 'clientside initialization' stamp: 'ag 4/27/2011 17:29'!
startupCode
	"This prints the client-side entry-point for the application. We query the component for the startup-class and invoke #start on it.
	The result of this call is stored in the app-variable for convenience."
	
	^ self component clientStartupClass
		ifNil: [ '' ]
		ifNotNil: [ 'st.app=', (OrcaSqueakAstToJsAst escapeGlobal: self component clientStartupClass name), '.start();' ]! !

Object subclass: #OrcaTranslatedObject
	instanceVariableNames: 'squeakObject javaScriptCode codeFactory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Cache'!

OrcaTranslatedObject subclass: #OrcaComposedTranslatedObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Cache'!

!OrcaComposedTranslatedObject methodsFor: 'information gathering' stamp: 'ag 4/13/2011 16:57'!
childTranslatedObjects
	
	self subclassResponsibility! !

!OrcaComposedTranslatedObject methodsFor: 'information gathering' stamp: 'ag 4/13/2011 17:11'!
collectInformation: anInformationDescribingSelectorSymbol
	"Recursively go through the children and collect all their information."
	
	| children |
	children := self childTranslatedObjects.
	^ children ifNotEmpty: 
		[ children
			inject: (children anyOne perform: anInformationDescribingSelectorSymbol)
			into: [ :aSet :aChild | | childInformation |
				childInformation := aChild collectInformation: anInformationDescribingSelectorSymbol.
				childInformation
					ifEmpty: [ aSet ]
					ifNotEmpty: [ aSet union: childInformation ] ] ]! !

OrcaComposedTranslatedObject subclass: #OrcaTranslatedClass
	instanceVariableNames: 'instMethods classMethods instVars superclass classInstVars classVars'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Cache'!

!OrcaTranslatedClass methodsFor: 'converting' stamp: 'ag 1/5/2011 22:01'!
allMethodsOf: aSourceClass
	
	^ aSourceClass methodDict values collect: [ :method | 
		self codeFactory cacheObjectForMethod: method ]! !

!OrcaTranslatedClass methodsFor: 'information gathering' stamp: 'ag 4/13/2011 16:55'!
childTranslatedObjects
	
	^ self instMethods, self classMethods! !

!OrcaTranslatedClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
classInstVars
	
	^ classInstVars! !

!OrcaTranslatedClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
classInstVars: anObject
	
	classInstVars := anObject! !

!OrcaTranslatedClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
classMethods
	
	^ classMethods! !

!OrcaTranslatedClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
classMethods: anObject
	
	classMethods := anObject! !

!OrcaTranslatedClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
classVars
	
	^ classVars! !

!OrcaTranslatedClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:59'!
classVars: anObject
	
	classVars := anObject! !

!OrcaTranslatedClass methodsFor: 'converting' stamp: 'ag 4/18/2011 11:11'!
doUpdate
	"First memorize the fields, then do the actual conversion. Errors during the conversion will happen only once,
	the old version of the javascriptObject will be kept if that happens.
	TODO Bad: this is kind of the same code like in OrcaAbstractPrinter class >> convertClass:to:retrievingFrom: and #hasChanged in this class"
	
	self
		superclass: self squeakObject superclass name;
		instMethods: (self allMethodsOf: self squeakObject);
		classMethods: (self allMethodsOf: self squeakObject class);
		instVars: self squeakObject instVarNames;
		classInstVars: self squeakObject class instVarNames;
		classVars: self squeakObject classVarNames;
		javaScriptCode: (
			self converterClass
				convertClass: self squeakObject
				retrievingFrom: [ :method | "Here the CodeFactory is plugged to the Printer"
					self codeFactory codeForCompiledMethod: method ]).! !

!OrcaTranslatedClass methodsFor: 'converting' stamp: 'ag 1/10/2011 19:36'!
hasChanged
	
	^ 
	((self haveMethodsChanged: self instMethods from: self squeakObject)
	or: [ self haveMethodsChanged: self classMethods from: self squeakObject class ]
	or: [ self superclass ~= self squeakObject superclass name ]
	or: [ self instVars ~= self squeakObject instVarNames ]
	or: [ self classInstVars ~= self squeakObject class instVarNames ])
	or: [ self classVars ~= self squeakObject classVarNames ]! !

!OrcaTranslatedClass methodsFor: 'converting' stamp: 'ag 4/6/2011 18:10'!
haveMethodsChanged: originalMethods from: originalClass
	"Check #hasChanged on each OrcaTranslatedMethod and check, whether there are more methods in the class now."
	
	^ originalMethods isNil or: 
		[ (originalMethods anySatisfy: [ :method | 
			(self codeFactory isObjectCached: method squeakObject) not or: 
			[ method hasChanged ] ]) or:
		[ originalClass methodDict size ~= originalMethods size ]]! !

!OrcaTranslatedClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:57'!
instMethods
	
	^ instMethods! !

!OrcaTranslatedClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:57'!
instMethods: anObject
	
	instMethods := anObject! !

!OrcaTranslatedClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
instVars
	
	^ instVars! !

!OrcaTranslatedClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
instVars: anObject
	
	instVars := anObject! !

!OrcaTranslatedClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
superclass
	
	^ superclass! !

!OrcaTranslatedClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
superclass: aString
	
	superclass := aString! !

!OrcaTranslatedClass methodsFor: 'converting' stamp: 'ag 1/6/2011 16:24'!
updateMessage
	
	^ 'class ', self squeakObject name! !

OrcaComposedTranslatedObject subclass: #OrcaTranslatedClasses
	instanceVariableNames: 'translatedClasses'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Cache'!

!OrcaTranslatedClasses methodsFor: 'information gathering' stamp: 'ag 4/13/2011 16:55'!
childTranslatedObjects
	
	^ self translatedClasses! !

!OrcaTranslatedClasses methodsFor: 'converting' stamp: 'ag 4/18/2011 11:23'!
doUpdate
	"Call convertClasses: of the converterClass and memorize the result. Plug the codeFactory to the converterClass (via the block).
	Actually, the compiledClasses should always stay the same objects, but just in case we refill the Set.
	Might happen, that one single object is removed from the cache."
	
	self translatedClasses removeAll.
	self javaScriptCode: (
		self converterClass
			convertClasses: self squeakObject
			retrievingFrom: [ :class | "Here the CodeFactory is plugged to the Printer"
				self translatedClasses add: (self codeFactory cacheObjectForClass: class).
				self codeFactory codeForClass: class ]
			withCalledFunctionsSource: self).! !

!OrcaTranslatedClasses methodsFor: 'converting' stamp: 'lw 3/28/2011 12:31'!
hasChanged
	
	^ self translatedClasses isNil or:
		[ self translatedClasses anySatisfy: [ :each |
			(self codeFactory isObjectCached: each squeakObject) not
			or: [ each hasChanged ] ]]! !

!OrcaTranslatedClasses methodsFor: 'initialization' stamp: 'lw 3/28/2011 12:32'!
initialize
	
	super initialize.
	self translatedClasses: Set new.! !

!OrcaTranslatedClasses methodsFor: 'accessing' stamp: 'lw 3/28/2011 12:31'!
translatedClasses
	^ translatedClasses! !

!OrcaTranslatedClasses methodsFor: 'accessing' stamp: 'lw 3/28/2011 12:32'!
translatedClasses: aSet
	translatedClasses := aSet! !

!OrcaTranslatedClasses methodsFor: 'converting' stamp: 'ag 1/6/2011 16:25'!
updateMessage
	
	^ 'classes ', self squeakObject asString.! !

OrcaTranslatedObject subclass: #OrcaTranslatedDoIt
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Cache'!

!OrcaTranslatedDoIt methodsFor: 'converting' stamp: 'LT 3/16/2011 12:31:57.566'!
doUpdate
	
	self javaScriptCode: 
		(self converterClass convertDoIt: self squeakObject).! !

!OrcaTranslatedDoIt methodsFor: 'converting' stamp: 'ag 12/14/2010 00:33'!
hasChanged
	"DoIt-code does not change"
	
	^ false! !

OrcaTranslatedObject subclass: #OrcaTranslatedMethod
	instanceVariableNames: 'converter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Cache'!

!OrcaTranslatedMethod methodsFor: 'information gathering' stamp: 'lw 4/11/2011 15:59'!
collectInformation: anInformationDescribingSelectorSymbol

	^ self converter perform: anInformationDescribingSelectorSymbol! !

!OrcaTranslatedMethod methodsFor: 'accessing' stamp: 'lw 4/11/2011 14:31'!
converter
	^ converter! !

!OrcaTranslatedMethod methodsFor: 'accessing' stamp: 'lw 4/11/2011 14:31'!
converter: anObject
	converter := anObject! !

!OrcaTranslatedMethod methodsFor: 'converting' stamp: 'ag 1/19/2011 15:58'!
currentSqueakObject
	
	^ self converterClass
		method: self squeakObject selector
		of: self squeakObject methodClass
		ifAbsent: [ nil ]! !

!OrcaTranslatedMethod methodsFor: 'converting' stamp: 'ag 4/13/2011 14:29'!
doUpdate
	"Additionally to converting the current code, store the current instance of CompiledMethod"
	
	self squeakObject: self currentSqueakObject.
	self converter: (self converterClass converterOfCompiledMethod: self squeakObject).
	self javaScriptCode: self converter result! !

!OrcaTranslatedMethod methodsFor: 'converting' stamp: 'ag 12/17/2010 00:15'!
hasChanged
	"This is the main point of cache-validation. The instance-equality of the CompiledMethod is used.
	If the method-code changed, the method is recompiled. If the method is recompiled, there will be a new instance of CompiledMethod."
	
	^ (self squeakObject == self currentSqueakObject) not! !

!OrcaTranslatedMethod methodsFor: 'converting' stamp: 'ag 1/6/2011 16:27'!
updateMessage
	
	^ 'method ', self squeakObject methodClass name, ' >> ', self squeakObject selector! !

!OrcaTranslatedObject class methodsFor: 'update notification' stamp: 'ag 4/14/2011 12:57'!
callOnUpdateStart: aBlock withId: anId
	
	(self globalDictionaryFor: 'Start')
		at: anId
		put: aBlock! !

!OrcaTranslatedObject class methodsFor: 'update notification' stamp: 'ag 4/14/2011 12:57'!
callOnUpdateStop: aBlock withId: anId
	
	(self globalDictionaryFor: 'Stop')
		at: anId
		put: aBlock! !

!OrcaTranslatedObject class methodsFor: 'instance creation' stamp: 'ag 3/29/2011 11:23'!
from: aSqueakObject inFactory: aCodeFactory
	
	^ self new
		squeakObject: aSqueakObject;
		codeFactory: aCodeFactory;
		yourself! !

!OrcaTranslatedObject class methodsFor: 'update notification' stamp: 'ag 4/14/2011 12:55'!
globalDictionaryFor: aString
	
	^ Smalltalk
		at: (self globalKeyFor: aString)
		ifAbsentPut: [ Dictionary new ]! !

!OrcaTranslatedObject class methodsFor: 'update notification' stamp: 'sw 4/14/2011 15:34'!
globalDictionaryIncludes: anId
	
	^ ((Smalltalk at: (self globalKeyFor: 'Start') ifAbsent: [ ^ false ]) includesKey: anId)
	| ((Smalltalk at: (self globalKeyFor: 'Stop') ifAbsent: [ ^ false ]) includesKey: anId)! !

!OrcaTranslatedObject class methodsFor: 'update notification' stamp: 'ag 4/14/2011 14:29'!
globalKeyFor: aString
	
	^ ('OrcaCacheUpdates', aString, '-', self name) asSymbol! !

!OrcaTranslatedObject class methodsFor: 'update notification' stamp: 'ag 4/14/2011 13:00'!
notifyUpdateEnded: anObject withMessage: aString
	
	self notifyUpdates: 'Stop' of: anObject withMessage: aString! !

!OrcaTranslatedObject class methodsFor: 'update notification' stamp: 'ag 4/14/2011 13:00'!
notifyUpdateStarted: anObject withMessage: aString
	
	self notifyUpdates: 'Start' of: anObject withMessage: aString! !

!OrcaTranslatedObject class methodsFor: 'update notification' stamp: 'ag 4/14/2011 13:00'!
notifyUpdates: aString of: anObject withMessage: aMessage
	
	(self globalDictionaryFor: aString)
		valuesDo: [ :aBlock | aBlock value: anObject value: aMessage ]! !

!OrcaTranslatedObject class methodsFor: 'update notification' stamp: 'ag 4/14/2011 12:58'!
stopCallingOnUpdates: anId
	
	(self globalDictionaryFor: 'Start') removeKey: anId ifAbsent: [].
	(self globalDictionaryFor: 'Stop') removeKey: anId ifAbsent: [].! !

!OrcaTranslatedObject methodsFor: 'information gathering' stamp: 'lw 4/11/2011 16:01'!
calledFunctions 

	^ self collectInformation: #calledFunctions! !

!OrcaTranslatedObject methodsFor: 'accessing' stamp: 'ag 1/5/2011 21:44'!
codeFactory
	
	^ codeFactory! !

!OrcaTranslatedObject methodsFor: 'accessing' stamp: 'ag 1/5/2011 21:44'!
codeFactory: aCodeFactory
	
	codeFactory := aCodeFactory! !

!OrcaTranslatedObject methodsFor: 'information gathering' stamp: 'ag 4/13/2011 16:53'!
collectInformation: anInformationDescribingSelectorSymbol
	
	self subclassResponsibility! !

!OrcaTranslatedObject methodsFor: 'converting' stamp: 'lw 3/28/2011 13:49'!
converterClass
	"Convenience for subclasses"
	
	^ self codeFactory converterClass! !

!OrcaTranslatedObject methodsFor: 'converting' stamp: 'ag 1/6/2011 11:21'!
doUpdate
	"Update converted contents from the current state of the squeak-object."
	
	self subclassResponsibility! !

!OrcaTranslatedObject methodsFor: 'converting' stamp: 'ag 12/13/2010 22:41'!
hasChanged
	"Return true, if the source squeak-object has changed, since it has last been updated."
	
	self subclassResponsibility! !

!OrcaTranslatedObject methodsFor: 'accessing' stamp: 'LT 3/16/2011 12:32:24.779'!
javaScriptCode
	
	^ javaScriptCode! !

!OrcaTranslatedObject methodsFor: 'accessing' stamp: 'LT 3/16/2011 12:31:57.533'!
javaScriptCode: anObject
	
	javaScriptCode := anObject! !

!OrcaTranslatedObject methodsFor: 'converting' stamp: 'ag 4/28/2011 11:56'!
notifyListeners: aMessage started: hasStarted
	
	[ hasStarted
			ifTrue: [ self class notifyUpdateStarted: self withMessage: aMessage ]
			ifFalse: [ self class notifyUpdateEnded: self withMessage: aMessage ] ]
		on: Error do: [ :err | 
			Transcript cr; show: 'Failed to notify listeners about compliation (started: ', hasStarted, ') of ', self, ' (', aMessage, ') due to error: ', err ]! !

!OrcaTranslatedObject methodsFor: 'information gathering' stamp: 'lw 4/11/2011 16:01'!
referedClasses 

	^ self collectInformation: #referedClasses! !

!OrcaTranslatedObject methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:34'!
squeakObject
	
	^ squeakObject! !

!OrcaTranslatedObject methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:34'!
squeakObject: anObject
	
	squeakObject := anObject! !

!OrcaTranslatedObject methodsFor: 'converting' stamp: 'ag 4/28/2011 11:55'!
update
	"To track the updates easily, they are printed to Transript, if activated"
	
	| message |
	message := self updateMessage.
	message ifNotNil: [ message := 'Updating ', message ].
	self notifyListeners: message started: true.
	[ self doUpdate ]
		on: Error do: [ :err | "We don't catch Exception, just Error, to keep basic mechanisms (like halt) intact."
			self codeFactory invalidateObject: self squeakObject.
			self notifyListeners: message started: false.
			Error signal: 'Failed to translate ', self squeakObject, ': ', err ].
	self notifyListeners: message started: false.! !

!OrcaTranslatedObject methodsFor: 'converting' stamp: 'ag 1/6/2011 16:23'!
updateMessage
	"Return something else than nil to print it to Transcript on updates."
	
	^ nil! !

Object subclass: #OrcaTranslationUpdatesPrinter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Orca-Core-Cache'!

!OrcaTranslationUpdatesPrinter commentStamp: 'ag 4/18/2011 10:13' prior: 0!
This class has some class-side methods to configure logging of compilations to Transcript.
The logging uses the notification-mechanism of the Code-Factory. It can be configured on granularity of subclasses of OrcaTranslatedObject.!

!OrcaTranslationUpdatesPrinter class methodsFor: 'private' stamp: 'ag 4/14/2011 14:20'!
perform: aSymbol withArguments: anArray onClass: aClass
		
	(aClass withAllSuperclasses includes: OrcaTranslatedObject)
		ifFalse: [ self error: 'Please pass a subclass of ', OrcaTranslatedObject name, ' as argument.' ].
	aClass perform: aSymbol withArguments: anArray.! !

!OrcaTranslationUpdatesPrinter class methodsFor: 'printing udpates' stamp: 'ag 4/14/2011 13:08'!
printUpdatesEndOf: aClass
	
	^ self
		perform: #callOnUpdateStop:withId:
		withArguments: { self printer. self updatesId }
		onClass: aClass! !

!OrcaTranslationUpdatesPrinter class methodsFor: 'printing udpates' stamp: 'ag 4/14/2011 13:08'!
printUpdatesOf: aClass
	
	^ self
		perform: #callOnUpdateStart:withId:
		withArguments: { self printer. self updatesId }
		onClass: aClass! !

!OrcaTranslationUpdatesPrinter class methodsFor: 'private' stamp: 'ag 4/14/2011 14:05'!
printer
	
	^ [ :anObject :aMessage | Transcript cr; show: aMessage ]! !

!OrcaTranslationUpdatesPrinter class methodsFor: 'printing udpates' stamp: 'ag 4/14/2011 13:09'!
stopPrintingUpdatesOf: aClass
	
	^ self
		perform: #stopCallingOnUpdates:
		withArguments: { self updatesId }
		onClass: aClass! !

!OrcaTranslationUpdatesPrinter class methodsFor: 'private' stamp: 'ag 4/14/2011 13:07'!
updatesId
	"use this class as Id for listening for compiler-updates"
	
	^ self! !
OrcaAbstractPrinter initialize!
